@@ -35,7 +35,7 @@
 
 #include <cstdio>
 
-#include "filesystem.h"
+#include "OpenImageIO/filesystem.h"
 
 #include "CineonStream.h"
 
@@ -96,7 +96,7 @@ bool InStream::Seek(long offset, Origin origin)
 	}
 
 	if (this->fp == 0)
-		return -1;
+		return false;
 	return (::fseek(this->fp, offset, o) == 0);
 }
 
