@@ -111,7 +111,7 @@
 #  define OIIO_CONSTEXPR_OR_CONST constexpr
 #  define OIIO_CONSTEXPR14        constexpr
 #  define OIIO_NOEXCEPT           noexcept
-#elif (__cplusplus >= 201103L)
+#elif (__cplusplus >= 201103L) || _MSC_VER >= 1900
 #  define OIIO_USING_CPP11        1 /* deprecated */
 #  define OIIO_CPLUSPLUS_VERSION  11
 #  define OIIO_CONSTEXPR          constexpr
@@ -149,13 +149,31 @@
 
 
 // Define OIIO_GNUC_VERSION to hold an encoded gcc version (e.g. 40802 for
-// 4.8.2), or 0 if not a GCC release.
-#ifdef __GNUC__
+// 4.8.2), or 0 if not a GCC release. N.B.: This will be 0 for clang.
+#if defined(__GNUC__) && !defined(__clang__)
 #  define OIIO_GNUC_VERSION (10000*__GNUC__ + 100*__GNUC_MINOR__ + __GNUC_PATCHLEVEL__)
 #else
 #  define OIIO_GNUC_VERSION 0
 #endif
 
+// Define OIIO_CLANG_VERSION to hold an encoded generic Clang version (e.g.
+// 30402 for clang 3.4.2), or 0 if not a generic Clang release.
+// N.B. This will be 0 for the clang Apple distributes (which has different
+// version numbers entirely).
+#if defined(__clang__) && !defined(__apple_build_version__)
+#  define OIIO_CLANG_VERSION (10000*__clang_major__ + 100*__clang_minor__ + __clang_patchlevel__)
+#else
+#  define OIIO_CLANG_VERSION 0
+#endif
+
+// Define OIIO_APPLE_CLANG_VERSION to hold an encoded Apple Clang version
+// (e.g. 70002 for clang 7.0.2), or 0 if not an Apple Clang release.
+#if defined(__clang__) && defined(__apple_build_version__)
+#  define OIIO_APPLE_CLANG_VERSION (10000*__clang_major__ + 100*__clang_minor__ + __clang_patchlevel__)
+#else
+#  define OIIO_APPLE_CLANG_VERSION 0
+#endif
+
 // Tests for MSVS versions, always 0 if not MSVS at all.
 #define OIIO_MSVS_AT_LEAST_2013 (defined(_MSC_VER) && _MSC_VER >= 1800)
 #define OIIO_MSVS_BEFORE_2013   (defined(_MSC_VER) && _MSC_VER <  1800)
@@ -282,6 +300,18 @@
 #endif
 
 
+#if OIIO_CPLUSPLUS_VERSION >= 14 && __has_attribute(deprecated)
+#  define OIIO_DEPRECATED(msg) [[deprecated(msg)]]
+#elif OIIO_GNUC_VERSION >= 40600 || defined(__clang__)
+#  define OIIO_DEPRECATED(msg) __attribute__((deprecated(msg)))
+#elif defined(__GNUC__) /* older gcc -- only the one with no message */
+#  define OIIO_DEPRECATED(msg) __attribute__((deprecated))
+#elif defined(_MSC_VER)
+#  define OIIO_DEPRECATED(msg) __declspec(deprecated(msg))
+#else
+#  define OIIO_DEPRECATED(msg)
+#endif
+
 
 // Try to deduce endianness
 #if (defined(_WIN32) || defined(__i386__) || defined(__x86_64__))
@@ -344,16 +374,16 @@ inline void cpuid (int info[4], int infoType, int extra)
 }
 
 
-inline bool cpu_has_sse2  () {int i[4]; cpuid(i,1,0); return i[3] & (1<<26); }
-inline bool cpu_has_sse3  () {int i[4]; cpuid(i,1,0); return i[2] & (1<<0); }
-inline bool cpu_has_ssse3 () {int i[4]; cpuid(i,1,0); return i[2] & (1<<9); }
-inline bool cpu_has_fma   () {int i[4]; cpuid(i,1,0); return i[2] & (1<<12); }
-inline bool cpu_has_sse41 () {int i[4]; cpuid(i,1,0); return i[2] & (1<<19); }
-inline bool cpu_has_sse42 () {int i[4]; cpuid(i,1,0); return i[2] & (1<<20); }
-inline bool cpu_has_popcnt() {int i[4]; cpuid(i,1,0); return i[2] & (1<<23); }
-inline bool cpu_has_avx   () {int i[4]; cpuid(i,1,0); return i[2] & (1<<28); }
-inline bool cpu_has_f16c  () {int i[4]; cpuid(i,1,0); return i[2] & (1<<29); }
-inline bool cpu_has_rdrand() {int i[4]; cpuid(i,1,0); return i[2] & (1<<30); }
+inline bool cpu_has_sse2  () {int i[4]; cpuid(i,1,0); return (i[3] & (1<<26)) != 0; }
+inline bool cpu_has_sse3  () {int i[4]; cpuid(i,1,0); return (i[2] & (1<<0)) != 0; }
+inline bool cpu_has_ssse3 () {int i[4]; cpuid(i,1,0); return (i[2] & (1<<9)) != 0; }
+inline bool cpu_has_fma   () {int i[4]; cpuid(i,1,0); return (i[2] & (1<<12)) != 0; }
+inline bool cpu_has_sse41 () {int i[4]; cpuid(i,1,0); return (i[2] & (1<<19)) != 0; }
+inline bool cpu_has_sse42 () {int i[4]; cpuid(i,1,0); return (i[2] & (1<<20)) != 0; }
+inline bool cpu_has_popcnt() {int i[4]; cpuid(i,1,0); return (i[2] & (1<<23)) != 0; }
+inline bool cpu_has_avx   () {int i[4]; cpuid(i,1,0); return (i[2] & (1<<28)) != 0; }
+inline bool cpu_has_f16c  () {int i[4]; cpuid(i,1,0); return (i[2] & (1<<29)) != 0; }
+inline bool cpu_has_rdrand() {int i[4]; cpuid(i,1,0); return (i[2] & (1<<30)) != 0; }
 
 
 OIIO_NAMESPACE_END
