@@ -133,13 +133,11 @@ public:
     /// color space correction when indicated.
     void pixel_transform (bool srgb_to_linear, int color_mode, int channel);
 
-    bool get_pixels (int xbegin, int xend, int ybegin, int yend,
-                     TypeDesc format, void *result) {
-        if (m_corrected_image.localpixels ()) {
-            return m_corrected_image.get_pixels (xbegin, xend, ybegin, yend,
-                                                  0, 1, format, result);
-        }
-        return ImageBuf::get_pixels (xbegin, xend, ybegin, yend, 0, 1, format, result);
+    bool get_pixels (ROI roi, TypeDesc format, void *result) {
+        if (m_corrected_image.localpixels ())
+            return m_corrected_image.get_pixels (roi, format, result);
+        else
+            return ImageBuf::get_pixels (roi, format, result);
     }
 
     bool auto_subimage (void) const { return m_auto_subimage; }
