@@ -43,8 +43,12 @@ OIIO_PLUGIN_NAMESPACE_BEGIN
 #define DBG if(0)
 
 
+// References:
+//  * JPEG library documentation: /usr/share/doc/libjpeg-devel-6b
+//  * JFIF spec: https://www.w3.org/Graphics/JPEG/jfif3.pdf
+//  * ITU T.871 (aka ISO/IEC 10918-5):
+//      https://www.itu.int/rec/T-REC-T.871-201105-I/en
 
-// See JPEG library documentation in /usr/share/doc/libjpeg-devel-6b
 
 
 class JpgOutput : public ImageOutput {
@@ -174,17 +178,27 @@ JpgOutput::open (const std::string &name, const ImageSpec &newspec,
         m_cinfo.density_unit = 2;
     else
         m_cinfo.density_unit = 0;
+
     m_cinfo.X_density = int (m_spec.get_float_attribute ("XResolution"));
     m_cinfo.Y_density = int (m_spec.get_float_attribute ("YResolution"));
-    float aspect = m_spec.get_float_attribute ("PixelAspectRatio", 1.0f);
-    if (m_cinfo.X_density <= 1 && m_cinfo.Y_density <= 1 && aspect != 1.0f) {
+    const float aspect = m_spec.get_float_attribute ("PixelAspectRatio", 1.0f);
+    if (aspect != 1.0f && m_cinfo.X_density <= 1 && m_cinfo.Y_density <= 1) {
         // No useful [XY]Resolution, but there is an aspect ratio requested.
         // Arbitrarily pick 72 dots per undefined unit, and jigger it to
         // honor it as best as we can.
-        m_cinfo.X_density = 72;
-        m_cinfo.Y_density = int (m_cinfo.X_density * aspect);
-        m_spec.attribute ("XResolution", 72.0f);
-        m_spec.attribute ("YResolution", 72.0f*aspect);
+        //
+        // Here's where things get tricky. By logic and reason, as well as
+        // the JFIF spec and ITU T.871, the pixel aspect ratio is clearly
+        // ydensity/xdensity (because aspect is xlength/ylength, and density
+        // is 1/length). BUT... for reasons lost to history, a number of
+        // apps get this exactly backwards, and these include PhotoShop,
+        // Nuke, and RV. So, alas, we must replicate the mistake, or else
+        // all these common applications will misunderstand the JPEG files
+        // written by OIIO and vice versa.
+        m_cinfo.Y_density = 72;
+        m_cinfo.X_density = int (m_cinfo.Y_density * aspect + 0.5f);
+        m_spec.attribute ("XResolution", float(m_cinfo.Y_density * aspect + 0.5f));
+        m_spec.attribute ("YResolution", float(m_cinfo.Y_density));
     }
 
     m_cinfo.write_JFIF_header = TRUE;
