@@ -33,17 +33,37 @@
 #include "OpenImageIO/fmath.h"
 #include "OpenImageIO/imageio.h"
 
+
 OIIO_PLUGIN_NAMESPACE_BEGIN
 
 
-static void openjpeg_dummy_callback(const char*, void*) {}
+static void
+openjpeg_error_callback(const char *msg, void *data)
+{
+    if (ImageOutput *input = (ImageOutput *)data) {
+        if (! msg || ! msg[0])
+            msg = "Unknown OpenJpeg error";
+        input->error ("%s", msg);
+    }
+}
+
+
+static void
+openjpeg_dummy_callback(const char *msg, void *data)
+{
+}
+
+
 
 class Jpeg2000Output : public ImageOutput {
  public:
     Jpeg2000Output () { init (); }
     virtual ~Jpeg2000Output () { close (); }
     virtual const char *format_name (void) const { return "jpeg2000"; }
-    virtual bool supports (const std::string &feature) const { return false; }
+    virtual int supports (string_view feature) const {
+        return (feature == "alpha");
+        // FIXME: we should support Exif/IPTC, but currently don't.
+    }
     virtual bool open (const std::string &name, const ImageSpec &spec,
                        OpenMode mode=Create);
     virtual bool close ();
@@ -57,25 +77,44 @@ class Jpeg2000Output : public ImageOutput {
     FILE *m_file;
     opj_cparameters_t m_compression_parameters;
     opj_image_t *m_image;
+    opj_codec_t *m_codec;
+    opj_stream_t *m_stream;
     unsigned int m_dither;
+    bool m_convert_alpha;             //< Do we deassociate alpha?
     std::vector<unsigned char> m_tilebuffer;
+    std::vector<unsigned char> m_scratch;
 
     void init (void)
     {
         m_file = NULL;
         m_image = NULL;
+        m_codec = NULL;
+        m_stream = NULL;
+        m_convert_alpha = true;
     }
 
     opj_image_t* create_jpeg2000_image();
 
     void init_components(opj_image_cmptparm_t *components, int precision);
 
-    opj_cinfo_t* create_compressor();
+    opj_codec_t* create_compressor();
+
+    void destroy_compressor () {
+        if (m_codec) {
+            opj_destroy_codec(m_codec);
+            m_codec = NULL;
+        }
+    }
+
+    void destroy_stream () {
+        if (m_stream) {
+            opj_stream_destroy (m_stream);
+            m_stream = NULL;
+        }
+    }
 
     bool save_image();
 
-    uint16_t write_pixel(int p_nativePrecision, int p_pixelData);
-
     template<typename T>
     void write_scanline(int y, int z, const void *data);
 
@@ -112,12 +151,34 @@ Jpeg2000Output::open (const std::string &name, const ImageSpec &spec,
     m_filename = name;
     m_spec = spec;
 
+    // Check for things this format doesn't support
+    if (m_spec.width < 1 || m_spec.height < 1) {
+        error ("Image resolution must be at least 1x1, you asked for %d x %d",
+               m_spec.width, m_spec.height);
+        return false;
+    }
+    if (m_spec.depth < 1)
+        m_spec.depth = 1;
+    if (m_spec.depth > 1) {
+        error ("%s does not support volume images (depth > 1)", format_name());
+        return false;
+    }
+
+    if (m_spec.nchannels != 1 && m_spec.nchannels != 3 &&
+            m_spec.nchannels != 4) {
+        error ("%s does not support %d-channel images\n",
+               format_name(), m_spec.nchannels);
+        return false;
+    }
+
     // If not uint8 or uint16, default to uint8
     if (m_spec.format != TypeDesc::UINT8 && m_spec.format != TypeDesc::UINT16)
         m_spec.set_format (TypeDesc::UINT8);
 
     m_dither = (m_spec.format == TypeDesc::UINT8) ?
                     m_spec.get_int_attribute ("oiio:dither", 0) : 0;
+    m_convert_alpha = m_spec.alpha_channel != -1 &&
+                      !m_spec.get_int_attribute("oiio:UnassociatedAlpha", 0);
 
     m_file = Filesystem::fopen (m_filename, "wb");
     if (m_file == NULL) {
@@ -136,19 +197,66 @@ Jpeg2000Output::open (const std::string &name, const ImageSpec &spec,
 
 
 
+template <class T>
+static void
+deassociateAlpha (T * data, int size, int channels, int alpha_channel, float gamma)
+{
+    unsigned int max = std::numeric_limits<T>::max();
+    if (gamma == 1) {
+        for (int x = 0;  x < size;  ++x, data += channels)
+            if (data[alpha_channel])
+                for (int c = 0;  c < channels;  c++)
+                    if (c != alpha_channel) {
+                        unsigned int f = data[c];
+                        f = (f * max) / data[alpha_channel];
+                        data[c] = (T) std::min (max, f);
+                    }
+    } else {
+        for (int x = 0;  x < size;  ++x, data += channels)
+            if (data[alpha_channel]) {
+                // See associateAlpha() for an explanation.
+                float alpha_deassociate = pow((float)max / data[alpha_channel],
+                                              gamma);
+                for (int c = 0;  c < channels;  c++)
+                    if (c != alpha_channel)
+                        data[c] = static_cast<T> (std::min (max,
+                                (unsigned int)(data[c] * alpha_deassociate)));
+            }
+    }
+}
+
+
+
 bool
 Jpeg2000Output::write_scanline (int y, int z, TypeDesc format,
-                                 const void *data, stride_t xstride)
+                                const void *data, stride_t xstride)
 {
+    y -= m_spec.y;
     if (y > m_spec.height) {
-        error ("Attempt to write too many scanlines to %s", m_filename.c_str());
-        close ();
+        error ("Attempt to write too many scanlines to %s", m_filename);
         return false;
     }
 
-    std::vector<uint8_t> scratch;
-    data = to_native_scanline (format, data, xstride, scratch,
+    m_spec.auto_stride (xstride, format, spec().nchannels);
+    const void *origdata = data;
+    data = to_native_scanline (format, data, xstride, m_scratch,
                                m_dither, y, z);
+    if (data == origdata) {
+        m_scratch.assign ((unsigned char *)data,
+                          (unsigned char *)data+m_spec.scanline_bytes());
+        data = &m_scratch[0];
+    }
+
+    // JPEG-2000 specifically dictates unassociated (un-"premultiplied") alpha
+    if (m_convert_alpha) {
+        if (m_spec.format == TypeDesc::UINT16)
+            deassociateAlpha ((unsigned short *)data, m_spec.width,
+                              m_spec.nchannels, m_spec.alpha_channel, 2.2f);
+        else
+            deassociateAlpha ((unsigned char *)data, m_spec.width,
+                              m_spec.nchannels, m_spec.alpha_channel, 2.2f);
+    }
+
     if (m_spec.format == TypeDesc::UINT8)
         write_scanline<uint8_t>(y, z, data);
     else
@@ -177,9 +285,8 @@ Jpeg2000Output::write_tile (int x, int y, int z, TypeDesc format,
 bool
 Jpeg2000Output::close ()
 {
-    if (! m_file) {         // Already closed
+    if (! m_stream) {         // Already closed
         return true;
-        init();
     }
 
     bool ok = true;
@@ -191,12 +298,16 @@ Jpeg2000Output::close ()
         std::vector<unsigned char>().swap (m_tilebuffer);
     }
 
-    fclose(m_file);
-    m_file = NULL;
     if (m_image) {
         opj_image_destroy(m_image);
         m_image = NULL;
     }
+    destroy_compressor ();
+    destroy_stream ();
+    if (m_file) {
+        fclose (m_file);
+        m_file = NULL;
+    }
     return ok;
 }
 
@@ -205,30 +316,35 @@ Jpeg2000Output::close ()
 bool
 Jpeg2000Output::save_image()
 {
-    opj_cinfo_t* compressor = create_compressor();
-    if (!compressor)
+    m_codec = create_compressor();
+    if (!m_codec)
         return false;
 
-    opj_event_mgr_t event_mgr;
-    event_mgr.error_handler = openjpeg_dummy_callback;
-    event_mgr.warning_handler = openjpeg_dummy_callback;
-    event_mgr.info_handler = openjpeg_dummy_callback;
-    opj_set_event_mgr((opj_common_ptr)compressor, &event_mgr, NULL);
+    opj_set_error_handler (m_codec, openjpeg_error_callback, this);
+    opj_set_warning_handler (m_codec, openjpeg_dummy_callback, NULL);
+    opj_set_info_handler (m_codec, openjpeg_dummy_callback, NULL);
 
-    opj_setup_encoder(compressor, &m_compression_parameters, m_image);
+    opj_setup_encoder(m_codec, &m_compression_parameters, m_image);
 
-    opj_cio_t *cio = opj_cio_open((opj_common_ptr)compressor, NULL, 0);
-
-    opj_encode(compressor, cio, m_image, NULL);
-
-    size_t wb = fwrite(cio->buffer, 1, cio_tell(cio), m_file);
-    if (wb != (size_t)cio_tell(cio)) {
-    	error ("Failed write jpeg2000::save_image (err: %d)", wb);
-    	return false;
+#if defined(OPJ_VERSION_MAJOR)
+    // OpenJpeg >= 2.1
+    m_stream = opj_stream_create_default_file_stream (m_filename.c_str(), false);
+#else
+    // OpenJpeg 2.0: need to open a stream ourselves
+    m_file = Filesystem::fopen (m_filename, "wb");
+    m_stream = opj_stream_create_default_file_stream (m_file, false);
+#endif
+    if (! m_stream) {
+        error ("Failed write jpeg2000::save_image");
+        return false;
     }
 
-    opj_destroy_compress(compressor);
-    opj_cio_close(cio);
+    if (! opj_start_compress (m_codec, m_image, m_stream) ||
+        ! opj_encode (m_codec, m_stream) ||
+        ! opj_end_compress (m_codec, m_stream)) {
+        error ("Failed write jpeg2000::save_image");
+        return false;
+    }
 
     return true;
 }
@@ -239,9 +355,9 @@ Jpeg2000Output::create_jpeg2000_image()
 {
     setup_compression_params();
 
-    OPJ_COLOR_SPACE color_space = CLRSPC_SRGB;
+    OPJ_COLOR_SPACE color_space = OPJ_CLRSPC_SRGB;
     if (m_spec.nchannels == 1)
-        color_space = CLRSPC_GRAY;
+        color_space = OPJ_CLRSPC_GRAY;
 
     int precision = 16;
     const ImageIOParameter *prec = m_spec.find_attribute ("oiio:BitsPerSample",
@@ -261,6 +377,24 @@ Jpeg2000Output::create_jpeg2000_image()
     m_image->y0 = m_compression_parameters.image_offset_y0;
     m_image->x1 = m_compression_parameters.image_offset_x0 + (m_spec.width - 1) * m_compression_parameters.subsampling_dx + 1;
     m_image->y1 = m_compression_parameters.image_offset_y0 + (m_spec.height - 1) * m_compression_parameters.subsampling_dy + 1;
+
+#if 0
+    // FIXME: I seem to get crashes with OpenJpeg 2.x in the presence of ICC
+    // profiles. I have no idea why. It seems like losing the ability to
+    // write ICC profiles is the lesser evil compared to either restricting
+    // ourselves to OpenJpeg 1.5 or living with crashes. I'm at the limit of
+    // my knowledge of OpenJPEG, which frankly has a poor API and abysmal
+    // documentation. So I'll leave the repair of this for later. If
+    // someboody comes along that desperately needs JPEG2000 and ICC
+    // profiles, maybe they will be motivated enough to track down the
+    // problem.
+    const ImageIOParameter *icc = m_spec.find_attribute ("ICCProfile");
+    if (icc && icc->type().basetype == TypeDesc::UINT8 && icc->type().arraylen > 0) {
+        m_image->icc_profile_len = icc->type().arraylen;
+        m_image->icc_profile_buf = (unsigned char *) icc->data();
+    }
+#endif
+
     return m_image;
 }
 
@@ -282,56 +416,38 @@ Jpeg2000Output::init_components(opj_image_cmptparm_t *components, int precision)
 }
 
 
-opj_cinfo_t*
+opj_codec_t*
 Jpeg2000Output::create_compressor()
 {
     std::string ext = Filesystem::extension(m_filename);
-    opj_cinfo_t *compressor = NULL;
+    opj_codec_t *compressor = NULL;
     if (ext == ".j2k")
-        compressor = opj_create_compress(CODEC_J2K);
+        compressor = opj_create_compress (OPJ_CODEC_J2K);
     else if (ext == ".jp2")
-        compressor = opj_create_compress(CODEC_JP2);
+        compressor = opj_create_compress (OPJ_CODEC_JP2);
 
     return compressor;
 }
 
 
+
 template<typename T>
 void
 Jpeg2000Output::write_scanline(int y, int z, const void *data)
 {
+    int bits = sizeof(T)*8;
     const T* scanline = static_cast<const T*>(data);
-    const size_t scanline_pos =  y * m_spec.width;
-    if (m_spec.nchannels == 1) {
-        for (int i = 0; i < m_spec.width; i++)
-        {
-            m_image->comps[0].data[scanline_pos + i] = write_pixel(m_image->comps[0].prec, scanline[i]);
+    const size_t scanline_pos =  (y - m_spec.y) * m_spec.width;
+    for (int i = 0, j = 0; i < m_spec.width; i++) {
+        for (int c = 0; c < m_spec.nchannels; ++c) {
+            unsigned int val = scanline[j++];
+            if (bits != m_image->comps[c].prec)
+                val = bit_range_convert (val, bits, m_image->comps[c].prec);
+            m_image->comps[c].data[scanline_pos + i] = val;
         }
-        return;
-    }
-
-    for (int i = 0, j = 0; i < m_spec.width; i++)
-    {
-        m_image->comps[0].data[scanline_pos + i] = write_pixel(m_image->comps[0].prec, scanline[j++]);
-        m_image->comps[1].data[scanline_pos + i] = write_pixel(m_image->comps[0].prec, scanline[j++]);
-        m_image->comps[2].data[scanline_pos + i] = write_pixel(m_image->comps[0].prec, scanline[j++]);
-
-        if (m_spec.nchannels < 4)
-            continue;
-
-        m_image->comps[3].data[scanline_pos + i] = write_pixel(m_image->comps[0].prec, scanline[j++]);
     }
 }
 
-inline uint16_t
-Jpeg2000Output::write_pixel(int p_nativePrecision, int p_pixelData)
-{
-    if (p_nativePrecision == 10)
-        return p_pixelData >> 6;
-    if (p_nativePrecision == 12)
-        return p_pixelData >> 4;
-    return p_pixelData;
-}
 
 
 void Jpeg2000Output::setup_cinema_compression(OPJ_RSIZ_CAPABILITIES p_rsizCap)
@@ -352,7 +468,7 @@ void Jpeg2000Output::setup_cinema_compression(OPJ_RSIZ_CAPABILITIES p_rsizCap)
     m_compression_parameters.cblockh_init = 32;
     m_compression_parameters.csty |= 0x01;
 
-    m_compression_parameters.prog_order = CPRL;
+    m_compression_parameters.prog_order = OPJ_CPRL;
 
     m_compression_parameters.roi_compno = -1;
 
@@ -362,25 +478,25 @@ void Jpeg2000Output::setup_cinema_compression(OPJ_RSIZ_CAPABILITIES p_rsizCap)
     m_compression_parameters.irreversible = 1;
 
     m_compression_parameters.cp_rsiz = p_rsizCap;
-    if (p_rsizCap == CINEMA4K) {
-        m_compression_parameters.cp_cinema = CINEMA4K_24;
+    if (p_rsizCap == OPJ_CINEMA4K) {
+        m_compression_parameters.cp_cinema = OPJ_CINEMA4K_24;
         m_compression_parameters.POC[0].tile  = 1; 
         m_compression_parameters.POC[0].resno0  = 0; 
         m_compression_parameters.POC[0].compno0 = 0;
         m_compression_parameters.POC[0].layno1  = 1;
         m_compression_parameters.POC[0].resno1  = m_compression_parameters.numresolution-1;
         m_compression_parameters.POC[0].compno1 = 3;
-        m_compression_parameters.POC[0].prg1 = CPRL;
+        m_compression_parameters.POC[0].prg1 = OPJ_CPRL;
         m_compression_parameters.POC[1].tile  = 1;
         m_compression_parameters.POC[1].resno0  = m_compression_parameters.numresolution-1; 
         m_compression_parameters.POC[1].compno0 = 0;
         m_compression_parameters.POC[1].layno1  = 1;
         m_compression_parameters.POC[1].resno1  = m_compression_parameters.numresolution;
         m_compression_parameters.POC[1].compno1 = 3;
-        m_compression_parameters.POC[1].prg1 = CPRL;
+        m_compression_parameters.POC[1].prg1 = OPJ_CPRL;
     }
-    else if (p_rsizCap == CINEMA2K) {
-        m_compression_parameters.cp_cinema = CINEMA2K_24;
+    else if (p_rsizCap == OPJ_CINEMA2K) {
+        m_compression_parameters.cp_cinema = OPJ_CINEMA2K_24;
     }
 }
 
@@ -395,12 +511,12 @@ void Jpeg2000Output::setup_compression_params()
     const ImageIOParameter *is_cinema2k = m_spec.find_attribute ("jpeg2000:Cinema2K",
                                                                  TypeDesc::UINT);
     if (is_cinema2k)
-        setup_cinema_compression(CINEMA2K);
+        setup_cinema_compression(OPJ_CINEMA2K);
 
     const ImageIOParameter *is_cinema4k = m_spec.find_attribute ("jpeg2000:Cinema4K",
                                                                  TypeDesc::UINT);
     if (is_cinema4k)
-        setup_cinema_compression(CINEMA4K);
+        setup_cinema_compression(OPJ_CINEMA4K);
 
     const ImageIOParameter *initial_cb_width = m_spec.find_attribute ("jpeg2000:InitialCodeBlockWidth",
                                                                       TypeDesc::UINT);
@@ -428,16 +544,16 @@ void Jpeg2000Output::setup_compression_params()
 OPJ_PROG_ORDER Jpeg2000Output::get_progression_order(const std::string &progression_order)
 {
     if (progression_order == "LRCP")
-        return LRCP;
+        return OPJ_LRCP;
     else if (progression_order == "RLCP")
-        return RLCP;
+        return OPJ_RLCP;
     else if (progression_order == "RPCL")
-        return RPCL;
+        return OPJ_RPCL;
     else if (progression_order == "PCRL")
-        return PCRL;
+        return OPJ_PCRL;
     else if (progression_order == "PCRL")
-        return CPRL;
-    return PROG_UNKNOWN;
+        return OPJ_CPRL;
+    return OPJ_PROG_UNKNOWN;
 }
 
 OIIO_PLUGIN_NAMESPACE_END
