@@ -90,16 +90,22 @@ ColorConfig::Impl::inventory ()
 {
 #ifdef USE_OCIO
     if (config_) {
-        for (int i = 0, e = config_->getNumColorSpaces();  i < e;  ++i) {
-            std::string name = config_->getColorSpaceNameByIndex(i);
-            add (name, i);
+        bool nonraw = false;
+        for (int i = 0, e = config_->getNumColorSpaces();  i < e;  ++i)
+            nonraw |= ! Strutil::iequals(config_->getColorSpaceNameByIndex(i), "raw");
+        if (nonraw) {
+            for (int i = 0, e = config_->getNumColorSpaces();  i < e;  ++i)
+                add (config_->getColorSpaceNameByIndex(i), i);
+            OCIO::ConstColorSpaceRcPtr lin = config_->getColorSpace ("scene_linear");
+            if (lin)
+                linear_alias = lin->getName();
+            return;   // If any non-"raw" spaces were defined, we're done
         }
-        OCIO::ConstColorSpaceRcPtr lin = config_->getColorSpace ("scene_linear");
-        if (lin)
-            linear_alias = lin->getName();
     }
-    if (colorspaces.size())
-        return;   // If any were defined, we're done
+    // If we had some kind of bogus configuration that seemed to define
+    // only a "raw" color space and nothing else, that's useless, so
+    // figure out our own way to move forward.
+    config_.reset();
 #endif
 
     // If there was no configuration, or we didn't compile with OCIO
@@ -399,11 +405,23 @@ public:
     {
         if (channels > 3)
             channels = 3;
-        for (int y = 0;  y < height;  ++y) {
-            char *d = (char *)data + y*ystride;
-            for (int x = 0;  x < width;  ++x, d += xstride)
-                for (int c = 0;  c < channels;  ++c)
-                    ((float *)d)[c] = sRGB_to_linear (((float *)d)[c]);
+        if (channels == 3) {
+            for (int y = 0;  y < height;  ++y) {
+                char *d = (char *)data + y*ystride;
+                for (int x = 0;  x < width;  ++x, d += xstride) {
+                    simd::float4 r;
+                    r.load ((float *)d, 3);
+                    r = sRGB_to_linear (simd::float4((float *)d));
+                    r.store ((float *)d, 3);
+                }
+            }
+        } else {
+            for (int y = 0;  y < height;  ++y) {
+                char *d = (char *)data + y*ystride;
+                for (int x = 0;  x < width;  ++x, d += xstride)
+                    for (int c = 0;  c < channels;  ++c)
+                        ((float *)d)[c] = sRGB_to_linear (((float *)d)[c]);
+            }
         }
     }
 };
@@ -421,11 +439,23 @@ public:
     {
         if (channels > 3)
             channels = 3;
-        for (int y = 0;  y < height;  ++y) {
-            char *d = (char *)data + y*ystride;
-            for (int x = 0;  x < width;  ++x, d += xstride)
-                for (int c = 0;  c < channels;  ++c)
-                    ((float *)d)[c] = linear_to_sRGB (((float *)d)[c]);
+        if (channels == 3) {
+            for (int y = 0;  y < height;  ++y) {
+                char *d = (char *)data + y*ystride;
+                for (int x = 0;  x < width;  ++x, d += xstride) {
+                    simd::float4 r;
+                    r.load ((float *)d, 3);
+                    r = linear_to_sRGB (simd::float4((float *)d));
+                    r.store ((float *)d, 3);
+                }
+            }
+        } else {
+            for (int y = 0;  y < height;  ++y) {
+                char *d = (char *)data + y*ystride;
+                for (int x = 0;  x < width;  ++x, d += xstride)
+                    for (int c = 0;  c < channels;  ++c)
+                        ((float *)d)[c] = linear_to_sRGB (((float *)d)[c]);
+            }
         }
     }
 };
@@ -477,31 +507,77 @@ public:
 
 
 
+// ColorProcessor that does nothing (identity transform)
+class ColorProcessor_Ident : public ColorProcessor {
+public:
+    ColorProcessor_Ident () : ColorProcessor() { };
+    ~ColorProcessor_Ident () { };
+    virtual void apply (float *data, int width, int height, int channels,
+                        stride_t chanstride, stride_t xstride,
+                        stride_t ystride) const
+    {
+    }
+};
+
+
+
 ColorProcessor*
 ColorConfig::createColorProcessor (string_view inputColorSpace,
                                    string_view outputColorSpace) const
 {
+    return createColorProcessor (inputColorSpace, outputColorSpace, "", "");
+}
+
+
+
+ColorProcessor*
+ColorConfig::createColorProcessor (string_view inputColorSpace,
+                                   string_view outputColorSpace,
+                                   string_view context_key,
+                                   string_view context_value) const
+{
+    string_view inputrole, outputrole;
+    std::string pending_error;
 #ifdef USE_OCIO
     // Ask OCIO to make a Processor that can handle the requested
     // transformation.
+    OCIO::ConstProcessorRcPtr p;
     if (getImpl()->config_) {
         // If the names are roles, convert them to color space names
         string_view name;
         name = getColorSpaceNameByRole (inputColorSpace);
-        if (! name.empty())
+        if (! name.empty()) {
+            inputrole = inputColorSpace;
             inputColorSpace = name;
+        }
         name = getColorSpaceNameByRole (outputColorSpace);
-        if (! name.empty())
+        if (! name.empty()) {
+            outputrole = outputColorSpace;
             outputColorSpace = name;
-        OCIO::ConstProcessorRcPtr p;
+        }
+
+        OCIO::ConstConfigRcPtr config = getImpl()->config_;
+        OCIO::ConstContextRcPtr context = config->getCurrentContext();
+        std::vector<string_view> keys, values;
+        Strutil::split (context_key, keys, ",");
+        Strutil::split (context_value, values, ",");
+        if (keys.size() && values.size() && keys.size() == values.size()) {
+            OCIO::ContextRcPtr ctx = context->createEditableCopy();
+            for (size_t i = 0; i < keys.size(); ++i)
+                ctx->setStringVar (keys[i].c_str(), values[i].c_str());
+            context = ctx;
+        }
+
         try {
             // Get the processor corresponding to this transform.
-            p = getImpl()->config_->getProcessor(inputColorSpace.c_str(),
+            p = getImpl()->config_->getProcessor(context, inputColorSpace.c_str(),
                                                  outputColorSpace.c_str());
         }
         catch(OCIO::Exception &e) {
-            getImpl()->error_ = e.what();
-            return NULL;
+            // Don't quit yet, remember the error and see if any of our
+            // built-in knowledge of some generic spaces will save us.
+            p.reset();
+            pending_error = e.what();
         }
         catch(...) {
             getImpl()->error_ = "An unknown error occurred in OpenColorIO, getProcessor";
@@ -509,31 +585,53 @@ ColorConfig::createColorProcessor (string_view inputColorSpace,
         }
     
         getImpl()->error_ = "";
-        return new ColorProcessor_OCIO(p);
+        if (p && ! p->isNoOp()) {
+            // If we got a valid processor that does something useful,
+            // return it now. If it boils down to a no-op, give a second
+            // chance below to recognize it as a special case.
+            return new ColorProcessor_OCIO(p);
+        }
     }
 #endif
 
     // Either not compiled with OCIO support, or no OCIO configuration
     // was found at all.  There are a few color conversions we know
     // about even in such dire conditions.
-    if (Strutil::iequals(inputColorSpace,"linear") &&
-        Strutil::iequals(outputColorSpace,"sRGB")) {
+    using namespace Strutil;
+    if (iequals(inputColorSpace,outputColorSpace)) {
+        return new ColorProcessor_Ident;
+    }
+    if ((iequals(inputColorSpace,"linear") || iequals(inputrole,"linear") ||
+         iequals(inputColorSpace,"lnf") || iequals(inputColorSpace,"lnh"))
+        && iequals(outputColorSpace,"sRGB")) {
         return new ColorProcessor_linear_to_sRGB;
     }
-    if (Strutil::iequals(inputColorSpace,"sRGB") &&
-        Strutil::iequals(outputColorSpace,"linear")) {
+    if (iequals(inputColorSpace,"sRGB") &&
+        (iequals(outputColorSpace,"linear") || iequals(outputrole,"linear") ||
+         iequals(outputColorSpace,"lnf") || iequals(outputColorSpace,"lnh"))) {
         return new ColorProcessor_sRGB_to_linear;
     }
-    if (Strutil::iequals(inputColorSpace,"linear") &&
-        Strutil::iequals(outputColorSpace,"Rec709")) {
+    if ((iequals(inputColorSpace,"linear") || iequals(inputrole,"linear") ||
+         iequals(inputColorSpace,"lnf") || iequals(inputColorSpace,"lnh")) &&
+        iequals(outputColorSpace,"Rec709")) {
         return new ColorProcessor_linear_to_Rec709;
     }
-    if (Strutil::iequals(inputColorSpace,"Rec709") &&
-        Strutil::iequals(outputColorSpace,"linear")) {
-        // No OCIO, or the OCIO config doesn't know linear->sRGB
+    if (iequals(inputColorSpace,"Rec709") &&
+        (iequals(outputColorSpace,"linear") || iequals(outputrole,"linear") ||
+         iequals(outputColorSpace,"lnf") || iequals(outputColorSpace,"lnh"))) {
         return new ColorProcessor_Rec709_to_linear;
     }
 
+#ifdef USE_OCIO
+    if (p) {
+        // If we found a procesor from OCIO, even if it was a NoOp, and we
+        // still don't have a better idea, return it.
+        return new ColorProcessor_OCIO(p);
+    }
+#endif
+
+    if (pending_error.size())
+        getImpl()->error_ = pending_error;
     return NULL;    // if we get this far, we've failed
 }
 
@@ -545,7 +643,7 @@ ColorConfig::createLookTransform (string_view looks,
                                   string_view outputColorSpace,
                                   bool inverse,
                                   string_view context_key,
-                                  string_view context_val) const
+                                  string_view context_value) const
 {
 #ifdef USE_OCIO
     // Ask OCIO to make a Processor that can handle the requested
@@ -570,9 +668,13 @@ ColorConfig::createLookTransform (string_view looks,
             dir = OCIO::TRANSFORM_DIR_FORWARD;
         }
         OCIO::ConstContextRcPtr context = config->getCurrentContext();
-        if (context_key.size() && context_val.size()) {
+        std::vector<string_view> keys, values;
+        Strutil::split (context_key, keys, ",");
+        Strutil::split (context_value, values, ",");
+        if (keys.size() && values.size() && keys.size() == values.size()) {
             OCIO::ContextRcPtr ctx = context->createEditableCopy();
-            ctx->setStringVar (context_key.c_str(), context_val.c_str());
+            for (size_t i = 0; i < keys.size(); ++i)
+                ctx->setStringVar (keys[i].c_str(), values[i].c_str());
             context = ctx;
         }
 
@@ -624,9 +726,13 @@ ColorConfig::createDisplayTransform (string_view display,
             transform->setLooksOverrideEnabled(false);
         }
         OCIO::ConstContextRcPtr context = config->getCurrentContext();
-        if (context_key.size() && context_value.size()) {
+        std::vector<string_view> keys, values;
+        Strutil::split (context_key, keys, ",");
+        Strutil::split (context_value, values, ",");
+        if (keys.size() && values.size() && keys.size() == values.size()) {
             OCIO::ContextRcPtr ctx = context->createEditableCopy();
-            ctx->setStringVar (context_key.c_str(), context_value.c_str());
+            for (size_t i = 0; i < keys.size(); ++i)
+                ctx->setStringVar (keys[i].c_str(), values[i].c_str());
             context = ctx;
         }
 
@@ -697,11 +803,11 @@ string_view
 ColorConfig::parseColorSpaceFromString (string_view str) const
 {
 #ifdef USE_OCIO
-    string_view result (getImpl()->config_->parseColorSpaceFromString (str.c_str()));
-    return result;
-#else
-    return "";
+    if (getImpl() && getImpl()->config_) {
+        return getImpl()->config_->parseColorSpaceFromString (str.c_str());
+    }
 #endif
+    return "";
 }
 
 
@@ -739,6 +845,20 @@ ImageBufAlgo::colorconvert (ImageBuf &dst, const ImageBuf &src,
                             string_view from, string_view to,
                             bool unpremult, ColorConfig *colorconfig,
                             ROI roi, int nthreads)
+{
+    return colorconvert (dst, src, from, to, unpremult, "", "",
+                         colorconfig, roi, nthreads);
+}
+
+
+
+bool
+ImageBufAlgo::colorconvert (ImageBuf &dst, const ImageBuf &src,
+                            string_view from, string_view to,
+                            bool unpremult, string_view context_key,
+                            string_view context_value,
+                            ColorConfig *colorconfig,
+                            ROI roi, int nthreads)
 {
     if (from.empty() || from == "current") {
         from = src.spec().get_string_attribute ("oiio:Colorspace", "Linear");
@@ -754,12 +874,14 @@ ImageBufAlgo::colorconvert (ImageBuf &dst, const ImageBuf &src,
             colorconfig = default_colorconfig.get();
         if (! colorconfig)
             default_colorconfig.reset (colorconfig = new ColorConfig);
-        processor = colorconfig->createColorProcessor (from, to);
+        processor = colorconfig->createColorProcessor (from, to,
+                                                context_key, context_value);
         if (! processor) {
             if (colorconfig->error())
                 dst.error ("%s", colorconfig->geterror());
             else
-                dst.error ("Could not construct the color transform");
+                dst.error ("Could not construct the color transform %s -> %s",
+                           from, to);
             return false;
         }
     }
