@@ -41,6 +41,7 @@
 #include <boost/scoped_array.hpp>
 
 #include "OpenImageIO/imageio.h"
+#include "OpenImageIO/deepdata.h"
 #include "OpenImageIO/imagebuf.h"
 #include "OpenImageIO/imagebufalgo.h"
 #include "OpenImageIO/imagebufalgo_util.h"
@@ -246,9 +247,26 @@ public:
         return m_spec;
     }
 
-    void threads (int n) { m_threads = n; }
+    void threads (int n) const { m_threads = n; }
     int threads () const { return m_threads; }
 
+    // Allocate m_configspec if not already done
+    void add_configspec (const ImageSpec *config=NULL) {
+        if (! m_configspec)
+            m_configspec.reset (config ? new ImageSpec (*config) : new ImageSpec);
+    }
+
+    // Return the index of pixel (x,y,z). If check_range is true, return
+    // -1 for an invalid coordinate that is not within the data window.
+    int pixelindex (int x, int y, int z, bool check_range=false) const {
+        x -= m_spec.x;  y -= m_spec.y;  z -= m_spec.z;
+        if (check_range && (x < 0 || x >= m_spec.width ||
+                            y < 0 || y >= m_spec.height ||
+                            z < 0 || z >= m_spec.depth))
+            return -1;
+        return (z * m_spec.height + y) * m_spec.width + x;
+    }
+
 private:
     ImageBuf::IBStorage m_storage; ///< Pixel storage class
     ustring m_name;              ///< Filename of the image
@@ -257,7 +275,7 @@ private:
     int m_current_subimage;      ///< Current subimage we're viewing
     int m_current_miplevel;      ///< Current miplevel we're viewing
     int m_nmiplevels;            ///< # of MIP levels in the current subimage
-    int m_threads;               ///< thread policy for this image
+    mutable int m_threads;       ///< thread policy for this image
     ImageSpec m_spec;            ///< Describes the image (size, etc)
     ImageSpec m_nativespec;      ///< Describes the true native image
     boost::scoped_array<char> m_pixels; ///< Pixel data, if local and we own it
@@ -345,8 +363,6 @@ ImageBufImpl::ImageBufImpl (const ImageBufImpl &src)
       m_nmiplevels(src.m_nmiplevels),
       m_threads(src.m_threads),
       m_spec(src.m_spec), m_nativespec(src.m_nativespec),
-      m_pixels(src.m_localpixels ? new char [src.m_spec.image_bytes()] : NULL),
-      m_localpixels(m_pixels.get()),
       m_badfile(src.m_badfile),
       m_pixelaspect(src.m_pixelaspect),
       m_pixel_bytes(src.m_pixel_bytes),
@@ -368,15 +384,18 @@ ImageBufImpl::ImageBufImpl (const ImageBufImpl &src)
     if (src.m_localpixels) {
         // Source had the image fully in memory (no cache)
         if (m_storage == ImageBuf::APPBUFFER) {
-            // Source just wrapped the client app's pixels
-            ASSERT (0 && "ImageBuf wrapping client buffer not yet supported");
+            // Source just wrapped the client app's pixels, we do the same
+            m_localpixels = src.m_localpixels;
         } else {
             // We own our pixels -- copy from source
+            m_pixels.reset (new char [src.m_spec.image_bytes()]);
             memcpy (m_pixels.get(), src.m_pixels.get(), m_spec.image_bytes());
+            m_localpixels = m_pixels.get();
         }
     } else {
         // Source was cache-based or deep
         // nothing else to do
+        m_localpixels = NULL;
     }
     if (src.m_configspec)
         m_configspec.reset (new ImageSpec(*src.m_configspec));
@@ -815,16 +834,6 @@ ImageBufImpl::read (int subimage, int miplevel, bool force, TypeDesc convert,
         // std::cerr << "read was not necessary -- using cache\n";
 #endif
         return true;
-    } else {
-#ifndef NDEBUG
-        // std::cerr << "going to have to read " << m_name << ": "
-        //           << m_spec.format.c_str() << " vs " << convert.c_str() << "\n";
-#endif
-        // FIXME/N.B. - is it really best to go through the ImageCache
-        // for forced IB reads?  Are there circumstances in which we
-        // should just to a straight read_image() to avoid the extra
-        // copies or the memory use of having bytes both in the cache
-        // and in the IB?
     }
 
     if (convert != TypeDesc::UNKNOWN)
@@ -840,15 +849,23 @@ ImageBufImpl::read (int subimage, int miplevel, bool force, TypeDesc convert,
     m_spec.tile_height = m_nativespec.tile_height;
     m_spec.tile_depth = m_nativespec.tile_depth;
 
-    if (convert != TypeDesc::UNKNOWN &&
-        convert != m_cachedpixeltype &&
-        convert.size() >= m_cachedpixeltype.size() &&
-        convert.size() >= m_nativespec.format.size()) {
+    if (force || (convert != TypeDesc::UNKNOWN &&
+                  convert != m_cachedpixeltype &&
+                  convert.size() >= m_cachedpixeltype.size() &&
+                  convert.size() >= m_nativespec.format.size())) {
         // A specific conversion type was requested which is not the cached
         // type and whose bit depth is as much or more than the cached type.
         // Bypass the cache and read directly so that there is no possible
         // loss of range or precision resulting from going through the
         // cache.
+        int unassoc = 0;
+        if (m_imagecache->getattribute ("unassociatedalpha", unassoc)) {
+            // Since IB needs to act as if it's backed by an ImageCache,
+            // even though in this case we're bypassing the IC, we need to
+            // honor the IC's "unassociatedalpha" flag.
+            add_configspec ();
+            m_configspec->attribute ("oiio:UnassociatedAlpha", unassoc);
+        }
         ImageInput *in = ImageInput::open (m_name.string(), m_configspec.get());
         bool ok = true;
         if (in) {
@@ -929,23 +946,74 @@ ImageBuf::write (ImageOutput *out,
     bool ok = true;
     const ImageBufImpl *impl = this->impl();
     ok &= impl->validate_pixels ();
-    const ImageSpec &m_spec (impl->m_spec);
+    const ImageSpec &bufspec (impl->m_spec);
+    const ImageSpec &outspec (out->spec());
+    TypeDesc bufformat = spec().format;
     if (impl->m_localpixels) {
         // In-core pixel buffer for the whole image
-        ok = out->write_image (m_spec.format, impl->m_localpixels, as, as, as,
+        ok = out->write_image (bufformat, impl->m_localpixels, as, as, as,
                                progress_callback, progress_callback_data);
     } else if (deep()) {
         // Deep image record
         ok = out->write_deep_image (impl->m_deepdata);
     } else {
-        // Backed by ImageCache
-        boost::scoped_array<char> tmp (new char [m_spec.image_bytes()]);
-        ok = get_pixels (xbegin(), xend(), ybegin(), yend(), zbegin(), zend(),
-                         m_spec.format, &tmp[0]);
-        ok &= out->write_image (m_spec.format, &tmp[0], as, as, as,
-                                progress_callback, progress_callback_data);
-        // FIXME -- not good for huge images.  Instead, we should read
-        // little bits at a time (scanline or tile blocks).
+        // The image we want to write is backed by ImageCache -- we must be
+        // immediately writing out a file from disk, possibly with file
+        // format or data format conversion, but without any ImageBufAlgo
+        // functions having been applied.
+        const imagesize_t budget = 1024*1024*64; // 64 MB
+        imagesize_t imagesize = bufspec.image_bytes();
+        if (imagesize <= budget) {
+            // whole image can fit within our budget
+            boost::scoped_array<char> tmp (new char [imagesize]);
+            ok &= get_pixels (roi(), bufformat, &tmp[0]);
+            ok &= out->write_image (bufformat, &tmp[0], as, as, as,
+                                    progress_callback, progress_callback_data);
+        } else if (outspec.tile_width) {
+            // Big tiled image: break up into tile strips
+            size_t pixelsize = bufspec.pixel_bytes();
+            size_t chunksize = pixelsize * outspec.width * outspec.tile_height * outspec.tile_depth;
+            boost::scoped_array<char> tmp (new char [chunksize]);
+            for (int z = 0;  z < outspec.depth;  z += outspec.tile_depth) {
+                int zend = std::min (z+outspec.z+outspec.tile_depth,
+                                     outspec.z+outspec.depth);
+                for (int y = 0;  y < outspec.height && ok;  y += outspec.tile_height) {
+                    int yend = std::min (y+outspec.y+outspec.tile_height,
+                                         outspec.y+outspec.height);
+                    ok &= get_pixels (ROI(outspec.x, outspec.x+outspec.width,
+                                          outspec.y+y, yend,
+                                          outspec.z+z, zend),
+                                      bufformat, &tmp[0]);
+                    ok &= out->write_tiles (outspec.x, outspec.x+outspec.width,
+                                            y+outspec.y, yend, z+outspec.z, zend,
+                                            bufformat, &tmp[0]);
+                    if (progress_callback &&
+                        progress_callback (progress_callback_data,
+                                           (float)(z*outspec.height+y)/(outspec.height*outspec.depth)))
+                        return ok;
+                }
+            }
+        } else {
+            // Big scanline image: break up into scanline strips
+            imagesize_t slsize = bufspec.scanline_bytes();
+            int chunk = clamp (round_to_multiple(int(budget/slsize), 64), 1, 1024);
+            boost::scoped_array<char> tmp (new char [chunk*slsize]);
+            for (int z = 0;  z < outspec.depth;  ++z) {
+                for (int y = 0;  y < outspec.height && ok;  y += chunk) {
+                    int yend = std::min (y+outspec.y+chunk, outspec.y+outspec.height);
+                    ok &= get_pixels (ROI(outspec.x, outspec.x+outspec.width,
+                                          outspec.y+y, yend,
+                                          outspec.z, outspec.z+outspec.depth),
+                                      bufformat, &tmp[0]);
+                    ok &= out->write_scanlines (y+outspec.y, yend, z+outspec.z,
+                                                bufformat, &tmp[0]);
+                    if (progress_callback &&
+                        progress_callback (progress_callback_data,
+                                           (float)(z*outspec.height+y)/(outspec.height*outspec.depth)))
+                        return ok;
+                }
+            }
+        }
     }
     if (! ok)
         error ("%s", out->geterror ());
@@ -1234,7 +1302,7 @@ ImageBuf::initialized () const
 void
 ImageBuf::threads (int n) const
 {
-    impl()->threads();
+    impl()->threads(n);
 }
 
 
@@ -1253,21 +1321,48 @@ namespace {
 // Pixel-by-pixel copy fully templated by both data types.
 // The roi is guaranteed to exist in both images.
 template<class D, class S>
-bool copy_pixels_2 (ImageBuf &dst, const ImageBuf &src, const ROI &roi)
+static bool
+copy_pixels_impl (ImageBuf &dst, const ImageBuf &src, ROI roi, int nthreads=0)
 {
+    if (nthreads != 1 && roi.npixels() >= 16384) {
+        // Lots of pixels and request for multi threads? Parallelize.
+        ImageBufAlgo::parallel_image (
+            OIIO::bind(copy_pixels_impl<D,S>, OIIO::ref(dst), OIIO::cref(src),
+                       _1 /*roi*/, 1 /*nthreads*/),
+            roi, nthreads);
+        return true;
+    }
+    // Serial case:
+
     int nchannels = roi.nchannels();
     if (is_same<D,S>::value) {
         // If both bufs are the same type, just directly copy the values
-        ImageBuf::Iterator<D,D> d (dst, roi);
-        ImageBuf::ConstIterator<D,D> s (src, roi);
-        for ( ; ! d.done();  ++d, ++s) {
-            for (int c = 0;  c < nchannels;  ++c)
-                d[c] = s[c];
+        if (src.localpixels() && roi.chbegin == 0 &&
+            roi.chend == dst.nchannels() && roi.chend == src.nchannels()) {
+            // Extra shortcut -- totally local pixels for src, copying all
+            // channels, so we can copy memory around line by line, rather
+            // than value by value.
+            int nxvalues = roi.width() * dst.nchannels();
+            for (int z = roi.zbegin; z < roi.zend; ++z)
+                for (int y = roi.ybegin; y < roi.yend; ++y) {
+                    D *draw = (D *) dst.pixeladdr (roi.xbegin, y, z);
+                    const S *sraw = (const S *) src.pixeladdr (roi.xbegin, y, z);
+                    DASSERT (draw && sraw);
+                    for (int x = 0; x < nxvalues; ++x)
+                        draw[x] = sraw[x];
+                }
+        } else {
+            ImageBuf::Iterator<D,D> d (dst, roi);
+            ImageBuf::ConstIterator<D,D> s (src, roi);
+            for ( ; ! d.done();  ++d, ++s) {
+                for (int c = 0;  c < nchannels;  ++c)
+                    d[c] = s[c];
+            }
         }
     } else {
         // If the two bufs are different types, convert through float
-        ImageBuf::Iterator<D,float> d (dst, roi);
-        ImageBuf::ConstIterator<S,float> s (src, roi);
+        ImageBuf::Iterator<D> d (dst, roi);
+        ImageBuf::ConstIterator<S> s (src, roi);
         for ( ; ! d.done();  ++d, ++s) {
             for (int c = 0;  c < nchannels;  ++c)
                 d[c] = s[c];
@@ -1298,7 +1393,7 @@ ImageBuf::copy_pixels (const ImageBuf &src)
         ImageBufAlgo::zero (*this);
 
     bool ok;
-    OIIO_DISPATCH_TYPES2 (ok, "copy_pixels", copy_pixels_2,
+    OIIO_DISPATCH_TYPES2 (ok, "copy_pixels", copy_pixels_impl,
                           spec().format, src.spec().format, *this, src, roi);
     return ok;
 }
@@ -1558,7 +1653,7 @@ ImageBuf::setpixel (int x, int y, int z, const float *pixel, int maxchannels)
     case TypeDesc::UINT64: setpixel_<unsigned long long> (*this, x, y, z, pixel, n); break;
     case TypeDesc::INT64 : setpixel_<long long> (*this, x, y, z, pixel, n); break;
     default:
-        ASSERT (0);
+        ASSERTMSG (0, "Unknown/unsupported data type %d", spec().format.basetype);
     }
 }
 
@@ -1717,33 +1812,23 @@ ImageBuf::deep_samples (int x, int y, int z) const
     impl()->validate_pixels();
     if (! deep())
         return 0;
-    const ImageSpec &m_spec (spec());
-    if (x < m_spec.x || y < m_spec.y || z < m_spec.z)
-        return 0;
-    x -= m_spec.x;  y -= m_spec.y;  z -= m_spec.z;
-    if (x >= m_spec.width || y >= m_spec.height || z >= m_spec.depth)
-        return 0;
-    int p = (z * m_spec.height + y) * m_spec.width  + x;
-    return deepdata()->samples(p);
+    int p = impl()->pixelindex (x, y, z, true);
+    return p >= 0 ? deepdata()->samples(p) : 0;
 }
 
 
 
 const void *
-ImageBuf::deep_pixel_ptr (int x, int y, int z, int c) const
+ImageBuf::deep_pixel_ptr (int x, int y, int z, int c, int s) const
 {
     impl()->validate_pixels();
     if (! deep())
         return NULL;
     const ImageSpec &m_spec (spec());
-    if (x < m_spec.x || y < m_spec.y || z < m_spec.z)
+    int p = impl()->pixelindex (x, y, z, true);
+    if (p < 0 || c < 0 || c >= m_spec.nchannels)
         return NULL;
-    x -= m_spec.x;  y -= m_spec.y;  z -= m_spec.z;
-    if (x >= m_spec.width || y >= m_spec.height || z >= m_spec.depth ||
-        c < 0 || c >= m_spec.nchannels)
-        return NULL;
-    int p = (z * m_spec.height + y) * m_spec.width  + x;
-    return deepdata()->samples(p) ? deepdata()->channel_ptr (p, c) : NULL;
+    return (s < deepdata()->samples(p)) ? deepdata()->data_ptr (p, c, s) : NULL;
 }
 
 
@@ -1754,9 +1839,7 @@ ImageBuf::deep_value (int x, int y, int z, int c, int s) const
     impl()->validate_pixels();
     if (! deep())
         return 0.0f;
-    const ImageSpec &m_spec (spec());
-    x -= m_spec.x;  y -= m_spec.y;  z -= m_spec.z;
-    int p = (z * m_spec.height + y) * m_spec.width + x;
+    int p = impl()->pixelindex (x, y, z);
     return impl()->m_deepdata.deep_value (p, c, s);
 }
 
@@ -1768,9 +1851,7 @@ ImageBuf::deep_value_uint (int x, int y, int z, int c, int s) const
     impl()->validate_pixels();
     if (! deep())
         return 0;
-    const ImageSpec &m_spec (spec());
-    x -= m_spec.x;  y -= m_spec.y;  z -= m_spec.z;
-    int p = (z * m_spec.height + y) * m_spec.width + x;
+    int p = impl()->pixelindex (x, y, z);
     return impl()->m_deepdata.deep_value_uint (p, c, s);
 }
 
@@ -1781,47 +1862,74 @@ ImageBuf::set_deep_samples (int x, int y, int z, int samps)
 {
     if (! deep())
         return ;
-    const ImageSpec &m_spec (spec());
-    x -= m_spec.x;  y -= m_spec.y;  z -= m_spec.z;
-    int p = (z * m_spec.height + y) * m_spec.width + x;
+    int p = impl()->pixelindex (x, y, z);
     impl()->m_deepdata.set_samples (p, samps);
 }
 
 
 
+void
+ImageBuf::deep_insert_samples (int x, int y, int z,
+                               int samplepos, int nsamples)
+{
+    if (! deep())
+        return ;
+    int p = impl()->pixelindex (x, y, z);
+    impl()->m_deepdata.insert_samples (p, samplepos, nsamples);
+}
+
+
+
+void
+ImageBuf::deep_erase_samples (int x, int y, int z,
+                              int samplepos, int nsamples)
+{
+    if (! deep())
+        return ;
+    int p = impl()->pixelindex (x, y, z);
+    impl()->m_deepdata.erase_samples (p, samplepos, nsamples);
+}
+
+
+
 void
 ImageBuf::set_deep_value (int x, int y, int z, int c, int s, float value)
 {
     impl()->validate_pixels();
     if (! deep())
         return ;
-    const ImageSpec &m_spec (spec());
-    x -= m_spec.x;  y -= m_spec.y;  z -= m_spec.z;
-    int p = (z * m_spec.height + y) * m_spec.width + x;
+    int p = impl()->pixelindex (x, y, z);
     return impl()->m_deepdata.set_deep_value (p, c, s, value);
 }
 
 
 
 void
-ImageBuf::set_deep_value_uint (int x, int y, int z, int c, int s, uint32_t value)
+ImageBuf::set_deep_value (int x, int y, int z, int c, int s, uint32_t value)
 {
     impl()->validate_pixels();
     if (! deep())
         return;
-    const ImageSpec &m_spec (spec());
-    x -= m_spec.x;  y -= m_spec.y;  z -= m_spec.z;
-    int p = (z * m_spec.height + y) * m_spec.width + x;
+    int p = impl()->pixelindex (x, y, z);
     return impl()->m_deepdata.set_deep_value (p, c, s, value);
 }
 
 
 
+// DEPRECATED (1.7): old name
+void
+ImageBuf::set_deep_value_uint (int x, int y, int z, int c, int s, uint32_t value)
+{
+    return set_deep_value (x, y, z, c, s, value);
+}
+
+
+
+// DEPRECATED (1.7)
 void
 ImageBuf::deep_alloc ()
 {
-    deepdata()->alloc ();
-    m_impl->m_storage = ImageBuf::LOCALBUFFER;
+    ASSERT (m_impl->m_storage == ImageBuf::LOCALBUFFER);
 }
 
 
@@ -2038,6 +2146,19 @@ ImageBuf::set_roi_full (const ROI &newroi)
 
 
 
+bool
+ImageBuf::contains_roi (ROI roi) const
+{
+    ROI myroi = roi;
+    return (roi.defined() && myroi.defined() &&
+            roi.xbegin >= myroi.xbegin && roi.xend <= myroi.xend &&
+            roi.ybegin >= myroi.ybegin && roi.yend <= myroi.yend &&
+            roi.zbegin >= myroi.zbegin && roi.zend <= myroi.zend &&
+            roi.chbegin >= myroi.chbegin && roi.chend <= myroi.chend);
+}
+
+
+
 const void *
 ImageBufImpl::pixeladdr (int x, int y, int z) const
 {
@@ -2086,6 +2207,14 @@ ImageBuf::pixeladdr (int x, int y, int z)
 
 
 
+int
+ImageBuf::pixelindex (int x, int y, int z, bool check_range) const
+{
+    return impl()->pixelindex (x, y, z, check_range);
+}
+
+
+
 const void *
 ImageBuf::blackpixel () const
 {
