@@ -93,18 +93,30 @@ ImageBufAlgo::IBAprep (ROI &roi, ImageBuf *dst, const ImageBuf *A,
             dst->error ("Uninitialized input image");
         return false;
     }
-    int maxchans = 10000;
-    if (prepflags & IBAprep_CLAMP_MUTUAL_NCHANNELS) {
-        // Instructions to clamp chend to the highest of the inputs
-        if (dst && dst->initialized())
-            maxchans = std::min (maxchans, dst->spec().nchannels);
-        if (A && A->initialized())
-            maxchans = std::min (maxchans, A->spec().nchannels);
-        if (B && B->initialized())
-            maxchans = std::min (maxchans, B->spec().nchannels);
-        if (C && C->initialized())
-            maxchans = std::min (maxchans, C->spec().nchannels);
+
+    int minchans, maxchans;
+    if (dst || A || B || C) {
+        minchans = 10000; maxchans = 1;
+        if (dst && dst->initialized()) {
+            minchans = std::min (minchans, dst->spec().nchannels);
+            maxchans = std::max (maxchans, dst->spec().nchannels);
+        }
+        if (A && A->initialized()) {
+            minchans = std::min (minchans, A->spec().nchannels);
+            maxchans = std::max (maxchans, A->spec().nchannels);
+        }
+        if (B && B->initialized()) {
+            minchans = std::min (minchans, B->spec().nchannels);
+            maxchans = std::max (maxchans, B->spec().nchannels);
+        }
+        if (C && C->initialized()) {
+            minchans = std::min (minchans, C->spec().nchannels);
+            maxchans = std::max (maxchans, C->spec().nchannels);
+        }
+    } else {
+        minchans = maxchans = 1;
     }
+
     if (dst->initialized()) {
         // Valid destination image.  Just need to worry about ROI.
         if (roi.defined()) {
@@ -150,11 +162,23 @@ ImageBufAlgo::IBAprep (ROI &roi, ImageBuf *dst, const ImageBuf *A,
         if (A) {
             // If there's an input image, give dst A's spec (with
             // modifications detailed below...)
-            spec = force_spec ? (*force_spec) : A->spec();
+            if (force_spec) {
+                spec = *force_spec;
+            } else {
+                // If dst is uninitialized and no force_spec was supplied,
+                // make it like A, but having number of channels as large as
+                // any of the inputs.
+                spec = A->spec();
+                if (prepflags & IBAprep_MINIMIZE_NCHANNELS)
+                    spec.nchannels = minchans;
+                else
+                    spec.nchannels = maxchans;
+            }
             // For multiple inputs, if they aren't the same data type, punt and
             // allocate a float buffer. If the user wanted something else,
             // they should have pre-allocated dst with their desired format.
-            if (B && A->spec().format != B->spec().format)
+            if ((B && A->spec().format != B->spec().format)
+                    || (prepflags & IBAprep_DST_FLOAT_PIXELS))
                 spec.set_format (TypeDesc::FLOAT);
             if (C && (A->spec().format != C->spec().format ||
                       B->spec().format != C->spec().format))
@@ -192,7 +216,25 @@ ImageBufAlgo::IBAprep (ROI &roi, ImageBuf *dst, const ImageBuf *A,
         }
 
         dst->reset (spec);
+
+        // If we just allocated more channels than the caller will write,
+        // clear the extra channels.
+        if (prepflags & IBAprep_CLAMP_MUTUAL_NCHANNELS)
+            roi.chend = std::min (roi.chend, minchans);
+        roi.chend = std::min (roi.chend, spec.nchannels);
+        if (roi.chbegin > 0) {
+            ROI r = roi;
+            r.chbegin = 0; r.chend = roi.chbegin;
+            ImageBufAlgo::zero (*dst, r, 1);
+        }
+        if (roi.chend < dst->nchannels()) {
+            ROI r = roi;
+            r.chbegin = roi.chend; r.chend = dst->nchannels();
+            ImageBufAlgo::zero (*dst, r, 1);
+        }
     }
+    if (prepflags & IBAprep_CLAMP_MUTUAL_NCHANNELS)
+        roi.chend = std::min (roi.chend, minchans);
     roi.chend = std::min (roi.chend, maxchans);
     if (prepflags & IBAprep_REQUIRE_ALPHA) {
         if (dst->spec().alpha_channel < 0 ||
@@ -306,6 +348,11 @@ convolve_ (ImageBuf &dst, const ImageBuf &src, const ImageBuf &kernel,
 
     // Serial case
 
+    ASSERT (kernel.spec().format == TypeDesc::FLOAT && kernel.localpixels() &&
+            "kernel should be float and in local memory");
+    ROI kroi = kernel.roi();
+    int kchans = kernel.nchannels();
+
     float scale = 1.0f;
     if (normalize) {
         scale = 0.0f;
@@ -313,23 +360,23 @@ convolve_ (ImageBuf &dst, const ImageBuf &src, const ImageBuf &kernel,
             scale += k[0];
         scale = 1.0f / scale;
     }
-
     float *sum = ALLOCA (float, roi.chend);
-    ROI kroi = get_roi (kernel.spec());
+
+    // Final case: handle full generality
     ImageBuf::Iterator<DSTTYPE> d (dst, roi);
     ImageBuf::ConstIterator<SRCTYPE> s (src, roi, ImageBuf::WrapClamp);
     for ( ; ! d.done();  ++d) {
-
         for (int c = roi.chbegin; c < roi.chend; ++c)
             sum[c] = 0.0f;
-
-        for (ImageBuf::ConstIterator<float> k (kernel, kroi); !k.done(); ++k) {
-            float kval = k[0];
-            s.pos (d.x() + k.x(), d.y() + k.y(), d.z() + k.z());
+        const float *k = (const float *)kernel.localpixels();
+        s.rerange (d.x() + kroi.xbegin, d.x() + kroi.xend,
+                   d.y() + kroi.ybegin, d.y() + kroi.yend,
+                   d.z() + kroi.zbegin, d.z() + kroi.zend,
+                   ImageBuf::WrapClamp);
+        for ( ; ! s.done(); ++s, k += kchans) {
             for (int c = roi.chbegin; c < roi.chend; ++c)
-                sum[c] += kval * s[c];
+                sum[c] += k[0] * s[c];
         }
-        
         for (int c = roi.chbegin; c < roi.chend; ++c)
             d[c] = scale * sum[c];
     }
@@ -347,9 +394,10 @@ ImageBufAlgo::convolve (ImageBuf &dst, const ImageBuf &src,
     if (! IBAprep (roi, &dst, &src, IBAprep_REQUIRE_SAME_NCHANNELS))
         return false;
     bool ok;
+    // Ensure that the kernel is float and in local memory
     const ImageBuf *K = &kernel;
     ImageBuf Ktmp;
-    if (kernel.spec().format != TypeDesc::FLOAT) {
+    if (kernel.spec().format != TypeDesc::FLOAT || ! kernel.localpixels()) {
         Ktmp.copy (kernel, TypeDesc::FLOAT);
         K = &Ktmp;
     }
@@ -417,20 +465,28 @@ ImageBufAlgo::make_kernel (ImageBuf &dst, string_view name,
                 dfilter[i] = binomial (depth-1, i);
         for (ImageBuf::Iterator<float> p (dst);  ! p.done();  ++p)
             p[0] = wfilter[p.x()-spec.x] * hfilter[p.y()-spec.y] * dfilter[p.z()-spec.z];
+    } else if (Strutil::iequals (name, "laplacian") && w == 3 && h == 3 && d == 1) {
+        const float vals[9] = { 0,  1,  0,
+                                1, -4,  1,
+                                0,  1,  0 };
+        dst.set_pixels (dst.roi(), TypeDesc::FLOAT, vals,
+                        sizeof(float), h*sizeof(float));
+        normalize = false; // sums to zero, so don't normalize it */
     } else {
         // No filter -- make a box
         float val = normalize ? 1.0f / ((w*h*d)) : 1.0f;
         for (ImageBuf::Iterator<float> p (dst);  ! p.done();  ++p)
             p[0] = val;
-        dst.error ("Unknown kernel \"%s\"", name);
+        dst.error ("Unknown kernel \"%s\" %gx%g", name, width, height);
         return false;
     }
     if (normalize) {
         float sum = 0;
         for (ImageBuf::Iterator<float> p (dst);  ! p.done();  ++p)
             sum += p[0];
-        for (ImageBuf::Iterator<float> p (dst);  ! p.done();  ++p)
-            p[0] = p[0] / sum;
+        if (sum != 0.0f)  /* don't normalize a 0-sum kernel */
+            for (ImageBuf::Iterator<float> p (dst);  ! p.done();  ++p)
+                p[0] = p[0] / sum;
     }
     return true;
 }
@@ -518,6 +574,26 @@ ImageBufAlgo::unsharp_mask (ImageBuf &dst, const ImageBuf &src,
 
 
 
+bool
+ImageBufAlgo::laplacian (ImageBuf &dst, const ImageBuf &src,
+                         ROI roi, int nthreads)
+{
+    if (! IBAprep (roi, &dst, &src,
+            IBAprep_REQUIRE_SAME_NCHANNELS | IBAprep_NO_SUPPORT_VOLUME))
+        return false;
+
+    ImageBuf K;
+    if (! make_kernel (K, "laplacian", 3, 3)) {
+        dst.error ("%s", K.geterror());
+        return false;
+    }
+    // K.write ("K.exr");
+    bool ok = convolve (dst, src, K, false, roi, nthreads);
+    return ok;
+}
+
+
+
 template<class Rtype, class Atype>
 static bool
 median_filter_impl (ImageBuf &R, const ImageBuf &A, int width, int height,
@@ -593,6 +669,102 @@ ImageBufAlgo::median_filter (ImageBuf &dst, const ImageBuf &src,
 
 
 
+enum MorphOp { MorphDilate, MorphErode };
+
+template<class Rtype, class Atype>
+static bool
+morph_impl (ImageBuf &R, const ImageBuf &A, int width, int height,
+            MorphOp op, ROI roi, int nthreads)
+{
+    if (nthreads != 1 && roi.npixels() >= 1000) {
+        // Possible multiple thread case -- recurse via parallel_image
+        ImageBufAlgo::parallel_image (
+            OIIO::bind(morph_impl<Rtype,Atype>,
+                        OIIO::ref(R), OIIO::cref(A),
+                        width, height, op, _1 /*roi*/, 1 /*nthreads*/),
+            roi, nthreads);
+        return true;
+    }
+
+    if (width < 1)
+        width = 1;
+    if (height < 1)
+        height = width;
+    int w_2 = std::max (1, width/2);
+    int h_2 = std::max (1, height/2);
+    int nchannels = R.nchannels();
+    float *vals = OIIO_ALLOCA (float, nchannels);
+
+    ImageBuf::ConstIterator<Atype> a (A, roi);
+    for (ImageBuf::Iterator<Rtype> r (R, roi);  !r.done();  ++r) {
+        a.rerange (r.x()-w_2, r.x()-w_2+width,
+                   r.y()-h_2, r.y()-h_2+height,
+                   r.z(), r.z()+1, ImageBuf::WrapClamp);
+        if (op == MorphDilate) {
+            for (int c = 0; c < nchannels; ++c)
+                vals[c] = -std::numeric_limits<float>::max();
+            for ( ;  ! a.done(); ++a) {
+                if (a.exists()) {
+                    for (int c = 0;  c < nchannels;  ++c)
+                        vals[c] = std::max(vals[c], a[c]);
+                }
+            }
+        } else if (op == MorphErode) {
+            for (int c = 0; c < nchannels; ++c)
+                vals[c] = std::numeric_limits<float>::max();
+            for ( ;  ! a.done(); ++a) {
+                if (a.exists()) {
+                    for (int c = 0;  c < nchannels;  ++c)
+                        vals[c] = std::min(vals[c], a[c]);
+                }
+            }
+        } else {
+            ASSERT (0 && "Unknown morphological operator");
+        }
+        for (int c = 0;  c < nchannels;  ++c)
+            r[c] = vals[c];
+    }
+    return true;
+}
+
+
+
+bool
+ImageBufAlgo::dilate (ImageBuf &dst, const ImageBuf &src,
+                      int width, int height, ROI roi, int nthreads)
+{
+    if (! IBAprep (roi, &dst, &src,
+            IBAprep_REQUIRE_SAME_NCHANNELS | IBAprep_NO_SUPPORT_VOLUME))
+        return false;
+
+    bool ok;
+    OIIO_DISPATCH_COMMON_TYPES2 (ok, "dilate",
+                                 morph_impl, dst.spec().format,
+                                 src.spec().format, dst, src,
+                                 width, height, MorphDilate, roi, nthreads);
+    return ok;
+}
+
+
+
+bool
+ImageBufAlgo::erode (ImageBuf &dst, const ImageBuf &src,
+                      int width, int height, ROI roi, int nthreads)
+{
+    if (! IBAprep (roi, &dst, &src,
+            IBAprep_REQUIRE_SAME_NCHANNELS | IBAprep_NO_SUPPORT_VOLUME))
+        return false;
+
+    bool ok;
+    OIIO_DISPATCH_COMMON_TYPES2 (ok, "erode",
+                                 morph_impl, dst.spec().format,
+                                 src.spec().format, dst, src,
+                                 width, height, MorphErode, roi, nthreads);
+    return ok;
+}
+
+
+
 // Helper function: fft of the horizontal rows
 static bool
 hfft_ (ImageBuf &dst, const ImageBuf &src, bool inverse, bool unitary,
