@@ -41,6 +41,7 @@
 #include "OpenImageIO/imagebuf.h"
 #include "OpenImageIO/imagebufalgo.h"
 #include "OpenImageIO/imagebufalgo_util.h"
+#include "OpenImageIO/deepdata.h"
 #include "OpenImageIO/thread.h"
 
 
@@ -106,13 +107,13 @@ ImageBufAlgo::paste (ImageBuf &dst, int xbegin, int ybegin,
 
 template<class D, class S>
 static bool
-crop_ (ImageBuf &dst, const ImageBuf &src,
+copy_ (ImageBuf &dst, const ImageBuf &src,
        ROI roi, int nthreads=1)
 {
     if (nthreads != 1 && roi.npixels() >= 1000) {
         // Lots of pixels and request for multi threads? Parallelize.
         ImageBufAlgo::parallel_image (
-            OIIO::bind(crop_<D,S>, OIIO::ref(dst), OIIO::cref(src),
+            OIIO::bind(copy_<D,S>, OIIO::ref(dst), OIIO::cref(src),
                         _1 /*roi*/, 1 /*nthreads*/),
             roi, nthreads);
         return true;
@@ -122,14 +123,26 @@ crop_ (ImageBuf &dst, const ImageBuf &src,
 
     // Deep
     if (dst.deep()) {
+        DeepData &dstdeep (*dst.deepdata());
+        const DeepData &srcdeep (*src.deepdata());
         ImageBuf::ConstIterator<S,D> s (src, roi);
         for (ImageBuf::Iterator<D,D> d (dst, roi);  ! d.done();  ++d, ++s) {
-            int samples = d.deep_samples ();
+            int samples = s.deep_samples ();
+            // The caller should ALREADY have set the samples, since that
+            // is not thread-safe against the copying below.
+            // d.set_deep_samples (samples);
+            DASSERT (d.deep_samples() == samples);
             if (samples == 0)
                 continue;
-            for (int c = roi.chbegin;  c < roi.chend;  ++c)
-                for (int samp = 0; samp < samples; ++samp)
-                    d.set_deep_value (c, samp, (float)s.deep_value(c, samp));
+            for (int c = roi.chbegin;  c < roi.chend;  ++c) {
+                if (dstdeep.channeltype(c) == TypeDesc::UINT32 &&
+                        srcdeep.channeltype(c) == TypeDesc::UINT32)
+                    for (int samp = 0; samp < samples; ++samp)
+                        d.set_deep_value (c, samp, (uint32_t)s.deep_value_uint(c, samp));
+                else
+                    for (int samp = 0; samp < samples; ++samp)
+                        d.set_deep_value (c, samp, (float)s.deep_value(c, samp));
+            }
         }
         return true;
     }
@@ -145,8 +158,40 @@ crop_ (ImageBuf &dst, const ImageBuf &src,
 }
 
 
+bool
+ImageBufAlgo::copy (ImageBuf &dst, const ImageBuf &src, TypeDesc convert,
+                    ROI roi, int nthreads)
+{
+    if (&dst == &src)   // trivial copy to self
+        return true;
 
-bool 
+    roi.chend = std::min (roi.chend, src.nchannels());
+    if (! dst.initialized()) {
+        ImageSpec newspec = src.spec();
+        set_roi (newspec, roi);
+        newspec.nchannels = roi.chend;
+        if (convert != TypeDesc::UNKNOWN)
+            newspec.set_format (convert);
+        dst.reset (newspec);
+    }
+    IBAprep (roi, &dst, &src, IBAprep_SUPPORT_DEEP);
+    if (dst.deep()) {
+        // If it's deep, figure out the sample allocations first, because
+        // it's not thread-safe to do that simultaneously with copying the
+        // values.
+        ImageBuf::ConstIterator<float> s (src, roi);
+        for (ImageBuf::Iterator<float> d (dst, roi);  !d.done();  ++d, ++s)
+            d.set_deep_samples (s.deep_samples());
+    }
+    bool ok;
+    OIIO_DISPATCH_TYPES2 (ok, "copy", copy_, dst.spec().format, src.spec().format,
+                          dst, src, roi, nthreads);
+    return ok;
+}
+
+
+
+bool
 ImageBufAlgo::crop (ImageBuf &dst, const ImageBuf &src,
                     ROI roi, int nthreads)
 {
@@ -156,15 +201,16 @@ ImageBufAlgo::crop (ImageBuf &dst, const ImageBuf &src,
         return false;
 
     if (dst.deep()) {
-        // If it's deep, figure out the sample allocations first
+        // If it's deep, figure out the sample allocations first, because
+        // it's not thread-safe to do that simultaneously with copying the
+        // values.
         ImageBuf::ConstIterator<float> s (src, roi);
         for (ImageBuf::Iterator<float> d (dst, roi);  !d.done();  ++d, ++s)
             d.set_deep_samples (s.deep_samples());
-        dst.deep_alloc ();
     }
 
     bool ok;
-    OIIO_DISPATCH_TYPES2 (ok, "crop", crop_, dst.spec().format, src.spec().format,
+    OIIO_DISPATCH_TYPES2 (ok, "crop", copy_, dst.spec().format, src.spec().format,
                           dst, src, roi, nthreads);
     return ok;
 }
@@ -410,14 +456,6 @@ ImageBufAlgo::rotate180 (ImageBuf &dst, const ImageBuf &src,
 }
 
 
-bool
-ImageBufAlgo::flipflop (ImageBuf &dst, const ImageBuf &src,
-                        ROI roi, int nthreads)
-{
-    return rotate180 (dst, src, roi, nthreads);
-}
-
-
 
 template<class D, class S>
 static bool
@@ -758,7 +796,6 @@ ImageBufAlgo::channels (ImageBuf &dst, const ImageBuf &src,
         // The earlier dst.alloc() already called dstdata.init()
         for (int p = 0, npels = (int)newspec.image_pixels(); p < npels; ++p)
             dstdata.set_samples (p, srcdata.samples(p));
-        dst.deep_alloc ();
         for (int p = 0, npels = (int)newspec.image_pixels(); p < npels; ++p) {
             if (! dstdata.samples(p))
                 continue;   // no samples for this pixel
@@ -769,10 +806,6 @@ ImageBufAlgo::channels (ImageBuf &dst, const ImageBuf &src,
                     float val = channelvalues ? channelvalues[c] : 0.0f;
                     for (int s = 0, ns = dstdata.samples(p); s < ns; ++s)
                         dstdata.set_deep_value (p, c, s, val);
-                } else if (dstdata.channeltype(c) == srcdata.channeltype(csrc)) {
-                    // Same channel types -- copy all samples at once
-                    memcpy (dstdata.channel_ptr(p,c), srcdata.channel_ptr(p,csrc),
-                            dstdata.samples(p) * srcdata.channeltype(csrc).size());
                 } else {
                     if (dstdata.channeltype(c) == TypeDesc::UINT)
                         for (int s = 0, ns = dstdata.samples(p); s < ns; ++s)
@@ -798,7 +831,7 @@ ImageBufAlgo::channels (ImageBuf &dst, const ImageBuf &src,
 
 
 
-template<class ABtype>
+template<class Rtype, class ABtype>
 static bool
 channel_append_impl (ImageBuf &dst, const ImageBuf &A, const ImageBuf &B,
                      ROI roi, int nthreads)
@@ -806,12 +839,10 @@ channel_append_impl (ImageBuf &dst, const ImageBuf &A, const ImageBuf &B,
     if (nthreads == 1 || roi.npixels() < 1000) {
         int na = A.nchannels(), nb = B.nchannels();
         int n = std::min (dst.nchannels(), na+nb);
-        ImageBuf::Iterator<float> r (dst, roi);
+        ImageBuf::Iterator<Rtype> r (dst, roi);
         ImageBuf::ConstIterator<ABtype> a (A, roi);
         ImageBuf::ConstIterator<ABtype> b (B, roi);
-        for (;  !r.done();  ++r) {
-            a.pos (r.x(), r.y(), r.z());
-            b.pos (r.x(), r.y(), r.z());
+        for (;  !r.done();  ++r, ++a, ++b) {
             for (int c = 0; c < n; ++c) {
                 if (c < na)
                     r[c] = a.exists() ? a[c] : 0.0f;
@@ -822,7 +853,7 @@ channel_append_impl (ImageBuf &dst, const ImageBuf &A, const ImageBuf &B,
     } else {
         // Possible multiple thread case -- recurse via parallel_image
         ImageBufAlgo::parallel_image (
-            OIIO::bind (channel_append_impl<ABtype>, OIIO::ref(dst),
+            OIIO::bind (channel_append_impl<Rtype,ABtype>, OIIO::ref(dst),
                          OIIO::cref(A), OIIO::cref(B), _1, 1),
             roi, nthreads);
     }
@@ -849,9 +880,20 @@ ImageBufAlgo::channel_append (ImageBuf &dst, const ImageBuf &A,
         dstspec.nchannels = A.spec().nchannels + B.spec().nchannels;
         for (int c = 0;  c < B.spec().nchannels;  ++c) {
             std::string name = B.spec().channelnames[c];
-            // Eliminate duplicates
-            if (std::find(dstspec.channelnames.begin(), dstspec.channelnames.end(), name) != dstspec.channelnames.end())
+            // It's a duplicate channel name. This will wreak havoc for
+            // OpenEXR, so we need to choose a unique name.
+            if (std::find(dstspec.channelnames.begin(), dstspec.channelnames.end(), name) != dstspec.channelnames.end()) {
+                // First, let's see if the original image had a subimage
+                // name and use that.
+                std::string subname = B.spec().get_string_attribute("oiio:subimagename");
+                if (subname.size())
+                    name = subname + "." + name;
+            }
+            if (std::find(dstspec.channelnames.begin(), dstspec.channelnames.end(), name) != dstspec.channelnames.end()) {
+                // If it's still a duplicate, fall back on a totally
+                // artificial name that contains the channel number.
                 name = Strutil::format ("channel%d", A.spec().nchannels+c);
+            }
             dstspec.channelnames.push_back (name);
         }
         if (dstspec.alpha_channel < 0 && B.spec().alpha_channel >= 0)
@@ -862,18 +904,17 @@ ImageBufAlgo::channel_append (ImageBuf &dst, const ImageBuf &A,
         dst.reset (dstspec);
     }
 
-    // For now, only support float destination, and equivalent A and B
-    // types.
-    if (dst.spec().format != TypeDesc::FLOAT ||
-        A.spec().format != B.spec().format) {
+    // For now, only support A and B having the same type.
+    if (A.spec().format != B.spec().format) {
         dst.error ("Unable to perform channel_append of %s, %s -> %s",
                    A.spec().format, B.spec().format, dst.spec().format);
         return false;
     }
 
     bool ok;
-    OIIO_DISPATCH_TYPES (ok, "channel_append", channel_append_impl,
-                         A.spec().format, dst, A, B, roi, nthreads);
+    OIIO_DISPATCH_TYPES2 (ok, "channel_append", channel_append_impl,
+                          dst.spec().format, A.spec().format,
+                          dst, A, B, roi, nthreads);
     return ok;
 }
 
