@@ -41,9 +41,9 @@
 #include "OpenImageIO/imagebuf.h"
 #include "OpenImageIO/imagebufalgo.h"
 #include "OpenImageIO/imagebufalgo_util.h"
-// #include "OpenImageIO/deepdata.h"
-#include "OpenImageIO/array_view.h"
+#include "OpenImageIO/deepdata.h"
 #include "OpenImageIO/dassert.h"
+#include "OpenImageIO/simd.h"
 
 
 
@@ -164,7 +164,7 @@ add_impl (ImageBuf &R, const ImageBuf &A, const float *b,
 
     if (R.deep()) {
         // Deep case
-        array_view<TypeDesc> channeltypes (R.deepdata()->channeltypes);
+        array_view<const TypeDesc> channeltypes (R.deepdata()->all_channeltypes());
         ImageBuf::Iterator<Rtype> r (R, roi);
         ImageBuf::ConstIterator<Atype> a (A, roi);
         for ( ;  !r.done();  ++r, ++a) {
@@ -194,12 +194,29 @@ bool
 ImageBufAlgo::add (ImageBuf &dst, const ImageBuf &A, const ImageBuf &B,
                    ROI roi, int nthreads)
 {
-    if (! IBAprep (roi, &dst, &A, &B, NULL, IBAprep_CLAMP_MUTUAL_NCHANNELS))
+    if (! IBAprep (roi, &dst, &A, &B))
         return false;
+    ROI origroi = roi;
+    roi.chend = std::min (roi.chend, std::min (A.nchannels(), B.nchannels()));
     bool ok;
     OIIO_DISPATCH_COMMON_TYPES3 (ok, "add", add_impl, dst.spec().format,
                                  A.spec().format, B.spec().format,
                                  dst, A, B, roi, nthreads);
+
+    if (roi.chend < origroi.chend && A.nchannels() != B.nchannels()) {
+        // Edge case: A and B differed in nchannels, we allocated dst to be
+        // the bigger of them, but adjusted roi to be the lesser. Now handle
+        // the channels that got left out because they were not common to
+        // all the inputs.
+        ASSERT (roi.chend <= dst.nchannels());
+        roi.chbegin = roi.chend;
+        roi.chend = origroi.chend;
+        if (A.nchannels() > B.nchannels()) { // A exists
+            copy (dst, A, dst.spec().format, roi, nthreads);
+        } else { // B exists
+            copy (dst, B, dst.spec().format, roi, nthreads);
+        }
+    }
     return ok;
 }
 
@@ -215,8 +232,7 @@ ImageBufAlgo::add (ImageBuf &dst, const ImageBuf &A, const float *b,
 
     if (dst.deep()) {
         // While still serial, set up all the sample counts
-        dst.deepdata()->nsamples = A.deepdata()->nsamples;
-        dst.deep_alloc ();
+        dst.deepdata()->set_all_samples (A.deepdata()->all_samples());
     }
 
     bool ok;
@@ -277,12 +293,29 @@ bool
 ImageBufAlgo::sub (ImageBuf &dst, const ImageBuf &A, const ImageBuf &B,
                    ROI roi, int nthreads)
 {
-    if (! IBAprep (roi, &dst, &A, &B, NULL, IBAprep_CLAMP_MUTUAL_NCHANNELS))
+    if (! IBAprep (roi, &dst, &A, &B))
         return false;
+    ROI origroi = roi;
+    roi.chend = std::min (roi.chend, std::min (A.nchannels(), B.nchannels()));
     bool ok;
     OIIO_DISPATCH_COMMON_TYPES3 (ok, "sub", sub_impl, dst.spec().format,
                                  A.spec().format, B.spec().format,
                                  dst, A, B, roi, nthreads);
+
+    if (roi.chend < origroi.chend && A.nchannels() != B.nchannels()) {
+        // Edge case: A and B differed in nchannels, we allocated dst to be
+        // the bigger of them, but adjusted roi to be the lesser. Now handle
+        // the channels that got left out because they were not common to
+        // all the inputs.
+        ASSERT (roi.chend <= dst.nchannels());
+        roi.chbegin = roi.chend;
+        roi.chend = origroi.chend;
+        if (A.nchannels() > B.nchannels()) { // A exists
+            copy (dst, A, dst.spec().format, roi, nthreads);
+        } else { // B exists
+            sub (dst, dst, B, roi, nthreads);
+        }
+    }
     return ok;
 }
 
@@ -298,8 +331,7 @@ ImageBufAlgo::sub (ImageBuf &dst, const ImageBuf &A, const float *b,
 
     if (dst.deep()) {
         // While still serial, set up all the sample counts
-        dst.deepdata()->nsamples = A.deepdata()->nsamples;
-        dst.deep_alloc ();
+        dst.deepdata()->set_all_samples (A.deepdata()->all_samples());
     }
 
     int nc = A.nchannels();
@@ -389,12 +421,29 @@ bool
 ImageBufAlgo::absdiff (ImageBuf &dst, const ImageBuf &A, const ImageBuf &B,
                        ROI roi, int nthreads)
 {
-    if (! IBAprep (roi, &dst, &A, &B, NULL, IBAprep_CLAMP_MUTUAL_NCHANNELS))
+    if (! IBAprep (roi, &dst, &A, &B))
         return false;
+    ROI origroi = roi;
+    roi.chend = std::min (roi.chend, std::min (A.nchannels(), B.nchannels()));
     bool ok;
     OIIO_DISPATCH_COMMON_TYPES3 (ok, "absdiff", absdiff_impl, dst.spec().format,
                                  A.spec().format, B.spec().format,
                                  dst, A, B, roi, nthreads);
+
+    if (roi.chend < origroi.chend && A.nchannels() != B.nchannels()) {
+        // Edge case: A and B differed in nchannels, we allocated dst to be
+        // the bigger of them, but adjusted roi to be the lesser. Now handle
+        // the channels that got left out because they were not common to
+        // all the inputs.
+        ASSERT (roi.chend <= dst.nchannels());
+        roi.chbegin = roi.chend;
+        roi.chend = origroi.chend;
+        if (A.nchannels() > B.nchannels()) { // A exists
+            abs (dst, A, roi, nthreads);
+        } else { // B exists
+            abs (dst, B, roi, nthreads);
+        }
+    }
     return ok;
 }
 
@@ -420,7 +469,7 @@ ImageBufAlgo::absdiff (ImageBuf &dst, const ImageBuf &A, float b,
 {
     if (! IBAprep (roi, &dst, &A, IBAprep_CLAMP_MUTUAL_NCHANNELS))
         return false;
-    int nc = A.nchannels();
+    int nc = dst.nchannels();
     float *vals = ALLOCA (float, nc);
     for (int c = 0;  c < nc;  ++c)
         vals[c] = b;
@@ -479,6 +528,9 @@ ImageBufAlgo::mul (ImageBuf &dst, const ImageBuf &A, const ImageBuf &B,
     OIIO_DISPATCH_COMMON_TYPES3 (ok, "mul", mul_impl, dst.spec().format,
                                  A.spec().format, B.spec().format,
                                  dst, A, B, roi, nthreads);
+    // N.B. No need to consider the case where A and B have differing number
+    // of channels. Missing channels are assumed 0, multiplication by 0 is
+    // 0, so it all just works through the magic of IBAprep.
     return ok;
 }
 
@@ -501,7 +553,7 @@ mul_impl (ImageBuf &R, const ImageBuf &A, const float *b,
 
     if (R.deep()) {
         // Deep case
-        array_view<TypeDesc> channeltypes (R.deepdata()->channeltypes);
+        array_view<const TypeDesc> channeltypes (R.deepdata()->all_channeltypes());
         ImageBuf::Iterator<Rtype> r (R, roi);
         ImageBuf::ConstIterator<Atype> a (A, roi);
         for ( ;  !r.done();  ++r, ++a) {
@@ -535,8 +587,7 @@ ImageBufAlgo::mul (ImageBuf &dst, const ImageBuf &A, const float *b,
 
     if (dst.deep()) {
         // While still serial, set up all the sample counts
-        dst.deepdata()->nsamples = A.deepdata()->nsamples;
-        dst.deep_alloc ();
+        dst.deepdata()->set_all_samples (A.deepdata()->all_samples());
     }
 
     bool ok;
@@ -620,8 +671,7 @@ ImageBufAlgo::div (ImageBuf &dst, const ImageBuf &A, const float *b,
 
     if (dst.deep()) {
         // While still serial, set up all the sample counts
-        dst.deepdata()->nsamples = A.deepdata()->nsamples;
-        dst.deep_alloc ();
+        dst.deepdata()->set_all_samples (A.deepdata()->all_samples());
     }
 
     int nc = dst.nchannels();
@@ -669,15 +719,55 @@ mad_impl (ImageBuf &R, const ImageBuf &A, const ImageBuf &B, const ImageBuf &C,
             roi, nthreads);
         return true;
     }
-
     // Serial case
-    ImageBuf::Iterator<Rtype> r (R, roi);
-    ImageBuf::ConstIterator<ABCtype> a (A, roi);
-    ImageBuf::ConstIterator<ABCtype> b (B, roi);
-    ImageBuf::ConstIterator<ABCtype> c (C, roi);
-    for ( ;  !r.done();  ++r, ++a, ++b, ++c)
-        for (int ch = roi.chbegin;  ch < roi.chend;  ++ch)
-            r[ch] = a[ch] * b[ch] + c[ch];
+
+    if (   (is_same<Rtype,float>::value || is_same<Rtype,half>::value)
+        && (is_same<ABCtype,float>::value || is_same<ABCtype,half>::value)
+        // && R.localpixels() // has to be, because it's writeable
+        && A.localpixels() && B.localpixels() && C.localpixels()
+        // && R.contains_roi(roi)  // has to be, because IBAPrep
+        && A.contains_roi(roi) && B.contains_roi(roi) && C.contains_roi(roi)
+        && roi.chbegin == 0 && roi.chend == R.nchannels()
+        && roi.chend == A.nchannels() && roi.chend == B.nchannels()
+        && roi.chend == C.nchannels()) {
+        // Special case when all inputs are either float or half, with in-
+        // memory contiguous data and we're operating on the full channel
+        // range: skip iterators: For these circumstances, we can operate on
+        // the raw memory very efficiently. Otherwise, we will need the
+        // magic of the the Iterators (and pay the price).
+        int nxvalues = roi.width() * R.nchannels();
+        for (int z = roi.zbegin; z < roi.zend; ++z)
+            for (int y = roi.ybegin; y < roi.yend; ++y) {
+                Rtype         *rraw =         (Rtype *) R.pixeladdr (roi.xbegin, y, z);
+                const ABCtype *araw = (const ABCtype *) A.pixeladdr (roi.xbegin, y, z);
+                const ABCtype *braw = (const ABCtype *) B.pixeladdr (roi.xbegin, y, z);
+                const ABCtype *craw = (const ABCtype *) C.pixeladdr (roi.xbegin, y, z);
+                DASSERT (araw && braw && craw);
+                // The straightforward loop auto-vectorizes very well,
+                // there's no benefit to using explicit SIMD here.
+                for (int x = 0; x < nxvalues; ++x)
+                    rraw[x] = araw[x] * braw[x] + craw[x];
+                // But if you did want to explicitly vectorize, this is
+                // how it would look:
+                // int simdend = nxvalues & (~3); // how many float4's?
+                // for (int x = 0; x < simdend; x += 4) {
+                //     simd::float4 a_simd(araw+x), b_simd(braw+x), c_simd(craw+x);
+                //     simd::float4 r_simd = a_simd * b_simd + c_simd;
+                //     r_simd.store (rraw+x);
+                // }
+                // for (int x = simdend; x < nxvalues; ++x)
+                //     rraw[x] = araw[x] * braw[x] + craw[x];
+            }
+    } else {
+        ImageBuf::Iterator<Rtype> r (R, roi);
+        ImageBuf::ConstIterator<ABCtype> a (A, roi);
+        ImageBuf::ConstIterator<ABCtype> b (B, roi);
+        ImageBuf::ConstIterator<ABCtype> c (C, roi);
+        for ( ;  !r.done();  ++r, ++a, ++b, ++c) {
+            for (int ch = roi.chbegin;  ch < roi.chend;  ++ch)
+                r[ch] = a[ch] * b[ch] + c[ch];
+        }
+    }
     return true;
 }
 
@@ -1280,9 +1370,10 @@ bool fixNonFinite_ (ImageBuf &dst, ImageBufAlgo::NonFiniteFixMode mode,
     ROI dstroi = get_roi (dst.spec());
     int count = 0;   // Number of pixels with nonfinite values
 
-    if (mode == ImageBufAlgo::NONFINITE_NONE) {
+    if (mode == ImageBufAlgo::NONFINITE_NONE ||
+        mode == ImageBufAlgo::NONFINITE_ERROR) {
         // Just count the number of pixels with non-finite values
-        for (ImageBuf::Iterator<T,T> pixel (dst);  ! pixel.done();  ++pixel) {
+        for (ImageBuf::Iterator<T,T> pixel (dst, roi);  ! pixel.done();  ++pixel) {
             for (int c = roi.chbegin;  c < roi.chend;  ++c) {
                 T value = pixel[c];
                 if (! isfinite(value)) {
@@ -1293,7 +1384,7 @@ bool fixNonFinite_ (ImageBuf &dst, ImageBufAlgo::NonFiniteFixMode mode,
         }
     } else if (mode == ImageBufAlgo::NONFINITE_BLACK) {
         // Replace non-finite pixels with black
-        for (ImageBuf::Iterator<T,T> pixel (dst);  ! pixel.done();  ++pixel) {
+        for (ImageBuf::Iterator<T,T> pixel (dst, roi);  ! pixel.done();  ++pixel) {
             bool fixed = false;
             for (int c = roi.chbegin;  c < roi.chend;  ++c) {
                 T value = pixel[c];
@@ -1308,7 +1399,7 @@ bool fixNonFinite_ (ImageBuf &dst, ImageBufAlgo::NonFiniteFixMode mode,
     } else if (mode == ImageBufAlgo::NONFINITE_BOX3) {
         // Replace non-finite pixels with a simple 3x3 window average
         // (the average excluding non-finite pixels, of course)
-        for (ImageBuf::Iterator<T,T> pixel (dst);  ! pixel.done();  ++pixel) {
+        for (ImageBuf::Iterator<T,T> pixel (dst, roi);  ! pixel.done();  ++pixel) {
             bool fixed = false;
             for (int c = roi.chbegin;  c < roi.chend;  ++c) {
                 T value = pixel[c];
@@ -1344,6 +1435,70 @@ bool fixNonFinite_ (ImageBuf &dst, ImageBufAlgo::NonFiniteFixMode mode,
     return true;
 }
 
+
+bool fixNonFinite_deep_ (ImageBuf &dst, ImageBufAlgo::NonFiniteFixMode mode,
+                         int *pixelsFixed, ROI roi, int nthreads)
+{
+    if (nthreads != 1 && roi.npixels() >= 1000) {
+        // Lots of pixels and request for multi threads? Parallelize.
+        ImageBufAlgo::parallel_image (
+            OIIO::bind(fixNonFinite_deep_, OIIO::ref(dst), mode, pixelsFixed,
+                        _1 /*roi*/, 1 /*nthreads*/),
+            roi, nthreads);
+        return true;
+    }
+
+    // Serial case
+
+    int count = 0;   // Number of pixels with nonfinite values
+    if (mode == ImageBufAlgo::NONFINITE_NONE ||
+        mode == ImageBufAlgo::NONFINITE_ERROR) {
+        // Just count the number of pixels with non-finite values
+        for (ImageBuf::Iterator<float> pixel (dst, roi);  ! pixel.done();  ++pixel) {
+            int samples = pixel.deep_samples ();
+            if (samples == 0)
+                continue;
+            bool bad = false;
+            for (int samp = 0; samp < samples && !bad; ++samp)
+                for (int c = roi.chbegin;  c < roi.chend;  ++c) {
+                    float value = pixel.deep_value (c, samp);
+                    if (! isfinite(value)) {
+                        ++count;
+                        bad = true;
+                        break;
+                    }
+                }
+        }
+    } else {
+        // We don't know what to do with BOX3, so just always set to black.
+        // Replace non-finite pixels with black
+        for (ImageBuf::Iterator<float> pixel (dst, roi);  ! pixel.done();  ++pixel) {
+            int samples = pixel.deep_samples ();
+            if (samples == 0)
+                continue;
+            bool fixed = false;
+            for (int samp = 0; samp < samples; ++samp)
+                for (int c = roi.chbegin;  c < roi.chend;  ++c) {
+                    float value = pixel.deep_value (c, samp);
+                    if (! isfinite(value)) {
+                        pixel.set_deep_value (c, samp, 0.0f);
+                        fixed = true;
+                    }
+                }
+            if (fixed)
+                ++count;
+        }
+    }
+
+    if (pixelsFixed) {
+        // Update pixelsFixed atomically -- that's what makes this whole
+        // function thread-safe.
+        *(atomic_int *)pixelsFixed += count;
+    }
+
+    return true;
+}
+
 } // anon namespace
 
 
@@ -1356,36 +1511,43 @@ ImageBufAlgo::fixNonFinite (ImageBuf &dst, const ImageBuf &src,
 {
     if (mode != ImageBufAlgo::NONFINITE_NONE &&
         mode != ImageBufAlgo::NONFINITE_BLACK &&
-        mode != ImageBufAlgo::NONFINITE_BOX3) {
+        mode != ImageBufAlgo::NONFINITE_BOX3 &&
+        mode != ImageBufAlgo::NONFINITE_ERROR) {
         // Something went wrong
         dst.error ("fixNonFinite: unknown repair mode");
         return false;
     }
 
-    if (! IBAprep (roi, &dst, &src))
+    if (! IBAprep (roi, &dst, &src, IBAprep_SUPPORT_DEEP))
         return false;
 
     // Initialize
-    if (pixelsFixed)
-        *pixelsFixed = 0;
+    bool ok = true;
+    int pixelsFixed_local;
+    if (! pixelsFixed)
+        pixelsFixed = &pixelsFixed_local;
+    *pixelsFixed = 0;
 
     // Start by copying dst to src, if they aren't the same image
     if (&dst != &src)
-        ImageBufAlgo::paste (dst, roi.xbegin, roi.ybegin, roi.zbegin, 0,
-                             src, roi, nthreads);
+        ok &= ImageBufAlgo::copy (dst, src, TypeDesc::UNKNOWN, roi, nthreads);
 
-    switch (src.spec().format.basetype) {
-    case TypeDesc::FLOAT :
-        return fixNonFinite_<float> (dst, mode, pixelsFixed, roi, nthreads);
-    case TypeDesc::HALF  :
-        return fixNonFinite_<half> (dst, mode, pixelsFixed, roi, nthreads);
-    case TypeDesc::DOUBLE:
-        return fixNonFinite_<double> (dst, mode, pixelsFixed, roi, nthreads);
-    default:
-        // All other format types aren't capable of having nonfinite
-        // pixel values, so the copy was enough.
-        return true;
+    if (dst.deep())
+        ok &= fixNonFinite_deep_ (dst, mode, pixelsFixed, roi, nthreads);
+    else if (src.spec().format.basetype == TypeDesc::FLOAT)
+        ok &= fixNonFinite_<float> (dst, mode, pixelsFixed, roi, nthreads);
+    else if (src.spec().format.basetype == TypeDesc::HALF)
+        ok &= fixNonFinite_<half> (dst, mode, pixelsFixed, roi, nthreads);
+    else if (src.spec().format.basetype == TypeDesc::DOUBLE)
+        ok &= fixNonFinite_<double> (dst, mode, pixelsFixed, roi, nthreads);
+    // All other format types aren't capable of having nonfinite
+    // pixel values, so the copy was enough.
+
+    if (mode == ImageBufAlgo::NONFINITE_ERROR && *pixelsFixed) {
+        dst.error ("Nonfinite pixel values found");
+        ok = false;
     }
+    return ok;
 }
 
 
