@@ -87,7 +87,7 @@ void test_zero_fill ()
     ImageBuf A (spec);
     
     // Set a pixel to an odd value, make sure it takes
-    const float arbitrary1[CHANNELS] = { 0.2, 0.3, 0.4, 0.5 };
+    const float arbitrary1[CHANNELS] = { 0.2f, 0.3f, 0.4f, 0.5f };
     A.setpixel (1, 1, arbitrary1);
     float pixel[CHANNELS];   // test pixel
     A.getpixel (1, 1, pixel);
@@ -106,7 +106,7 @@ void test_zero_fill ()
     }
 
     // Test fill of whole image
-    const float arbitrary2[CHANNELS] = { 0.6, 0.7, 0.3, 0.9 };
+    const float arbitrary2[CHANNELS] = { 0.6f, 0.7f, 0.3f, 0.9f };
     ImageBufAlgo::fill (A, arbitrary2);
     for (int j = 0;  j < HEIGHT;  ++j) {
         for (int i = 0;  i < WIDTH;  ++i) {
@@ -118,7 +118,7 @@ void test_zero_fill ()
     }
 
     // Test fill of partial image
-    const float arbitrary3[CHANNELS] = { 0.42, 0.43, 0.44, 0.45 };
+    const float arbitrary3[CHANNELS] = { 0.42f, 0.43f, 0.44f, 0.45f };
     {
         const int xbegin = 3, xend = 5, ybegin = 0, yend = 4;
         ImageBufAlgo::fill (A, arbitrary3, ROI(xbegin, xend, ybegin, yend));
@@ -153,10 +153,10 @@ void test_crop ()
     A.reset (spec);
     B.reset (spec);
     float arbitrary1[4];
-    arbitrary1[0] = 0.2;
-    arbitrary1[1] = 0.3;
-    arbitrary1[2] = 0.4;
-    arbitrary1[3] = 0.5;
+    arbitrary1[0] = 0.2f;
+    arbitrary1[1] = 0.3f;
+    arbitrary1[2] = 0.4f;
+    arbitrary1[3] = 0.5f;
     ImageBufAlgo::fill (A, arbitrary1);
 
     // Test CUT crop
@@ -532,6 +532,54 @@ void test_computePixelStats ()
 
 
 
+// Tests histogram computation.
+void histogram_computation_test ()
+{
+    const int INPUT_WIDTH   = 64;
+    const int INPUT_HEIGHT  = 64;
+    const int INPUT_CHANNEL = 0;
+
+    const int HISTOGRAM_BINS = 256;
+
+    const int SPIKE1 = 51;  // 0.2f in range 0->1 maps to 51 in range 0->255
+    const int SPIKE2 = 128; // 0.5f in range 0->1 maps to 128 in range 0->255
+    const int SPIKE3 = 204; // 0.8f in range 0->1 maps to 204 in range 0->255
+
+    const int SPIKE1_COUNT = INPUT_WIDTH * 8;
+    const int SPIKE2_COUNT = INPUT_WIDTH * 16;
+    const int SPIKE3_COUNT = INPUT_WIDTH * 40;
+
+    // Create input image with three regions with different pixel values.
+    ImageSpec spec (INPUT_WIDTH, INPUT_HEIGHT, 1, TypeDesc::FLOAT);
+    ImageBuf A (spec);
+
+    float value[] = {0.2f};
+    ImageBufAlgo::fill (A, value, ROI(0, INPUT_WIDTH, 0, 8));
+
+    value[0] = 0.5f;
+    ImageBufAlgo::fill (A, value, ROI(0, INPUT_WIDTH, 8, 24));
+
+    value[0] = 0.8f;
+    ImageBufAlgo::fill (A, value, ROI(0, INPUT_WIDTH, 24, 64));
+
+    // Compute A's histogram.
+    std::vector<imagesize_t> hist;
+    ImageBufAlgo::histogram (A, INPUT_CHANNEL, hist, HISTOGRAM_BINS);
+
+    // Does the histogram size equal the number of bins?
+    OIIO_CHECK_EQUAL (hist.size(), (imagesize_t)HISTOGRAM_BINS);
+
+    // Are the histogram values as expected?
+    OIIO_CHECK_EQUAL (hist[SPIKE1], (imagesize_t)SPIKE1_COUNT);
+    OIIO_CHECK_EQUAL (hist[SPIKE2], (imagesize_t)SPIKE2_COUNT);
+    OIIO_CHECK_EQUAL (hist[SPIKE3], (imagesize_t)SPIKE3_COUNT);
+    for (int i = 0; i < HISTOGRAM_BINS; i++)
+        if (i!=SPIKE1 && i!=SPIKE2 && i!=SPIKE3)
+            OIIO_CHECK_EQUAL (hist[i], 0);
+}
+
+
+
 // Test ability to do a maketx directly from an ImageBuf
 void
 test_maketx_from_imagebuf()
@@ -541,7 +589,7 @@ test_maketx_from_imagebuf()
     const int WIDTH = 16, HEIGHT = 16, CHANNELS = 3;
     ImageSpec spec (WIDTH, HEIGHT, CHANNELS, TypeDesc::FLOAT);
     ImageBuf A (spec);
-    float pink[] = { .5, .3, .3 }, green[] = { .1, .5, .1 };
+    float pink[] = { 0.5f, 0.3f, 0.3f }, green[] = { 0.1f, 0.5f, 0.1f };
     ImageBufAlgo::checker (A, 4, 4, 4, pink, green);
 
     // Write it
@@ -563,6 +611,74 @@ test_maketx_from_imagebuf()
 
 
 
+// Test various IBAprep features
+void
+test_IBAprep ()
+{
+    using namespace ImageBufAlgo;
+    ImageBuf rgb  (ImageSpec(256, 256, 3));  // Basic RGB uint8 image
+    ImageBuf rgba (ImageSpec(256, 256, 4));  // Basic RGBA uint8 image
+
+#define CHECK(...)  { ImageBuf dst; ROI roi; OIIO_CHECK_ASSERT (IBAprep (__VA_ARGS__)); }
+#define CHECK0(...) { ImageBuf dst; ROI roi; OIIO_CHECK_ASSERT (!IBAprep (__VA_ARGS__)); }
+
+    // Test REQUIRE_ALPHA
+    CHECK  (roi, &dst, &rgba, IBAprep_REQUIRE_ALPHA);
+    CHECK0 (roi, &dst, &rgb,  IBAprep_REQUIRE_ALPHA);
+
+    // Test REQUIRE_Z
+    ImageSpec rgbaz_spec (256, 256, 5);
+    rgbaz_spec.channelnames[4] = "Z";
+    rgbaz_spec.z_channel = 4;
+    ImageBuf rgbaz (rgbaz_spec);
+    CHECK  (  roi, &dst, &rgbaz, IBAprep_REQUIRE_Z);
+    CHECK0 (roi, &dst, &rgb, IBAprep_REQUIRE_Z);
+
+    // Test REQUIRE_SAME_NCHANNELS
+    CHECK  (roi, &dst, &rgb, &rgb,  NULL, NULL, IBAprep_REQUIRE_SAME_NCHANNELS);
+    CHECK0 (roi, &dst, &rgb, &rgba, NULL, NULL, IBAprep_REQUIRE_SAME_NCHANNELS);
+
+    // Test NO_SUPPOERT_VOLUME
+    ImageSpec volspec (256, 256, 3);  volspec.depth = 256;
+    ImageBuf vol (volspec);
+    CHECK  (roi, &dst, &rgb, IBAprep_NO_SUPPORT_VOLUME);
+    CHECK0 (roi, &dst, &vol, IBAprep_NO_SUPPORT_VOLUME);
+
+    // Test SUPPORT_DEEP
+    ImageSpec deepspec (256, 256, 3);  deepspec.deep = true;
+    ImageBuf deep (deepspec);
+    CHECK  (roi, &dst, &deep, IBAprep_SUPPORT_DEEP);
+    CHECK0 (roi, &dst, &deep);  // deep should be rejected
+
+    // Test DEEP_MIXED
+    CHECK  (roi, &dst, &deep, &deep, NULL, IBAprep_SUPPORT_DEEP | IBAprep_DEEP_MIXED);
+    CHECK  (roi, &dst, &deep, &rgb, NULL, IBAprep_SUPPORT_DEEP | IBAprep_DEEP_MIXED);
+    CHECK  (roi, &dst, &deep, &deep, NULL, IBAprep_SUPPORT_DEEP);
+    CHECK0 (roi, &dst, &deep, &rgb, NULL, IBAprep_SUPPORT_DEEP);
+
+    // Test DST_FLOAT_PIXELS
+    {
+        ROI roi1, roi2;
+        ImageBuf dst1, dst2;
+        OIIO_CHECK_ASSERT (IBAprep (roi1, &dst1, &rgb));
+        OIIO_CHECK_EQUAL (dst1.spec().format, TypeDesc::UINT8);
+        OIIO_CHECK_ASSERT (IBAprep (roi2, &dst2, &rgb, IBAprep_DST_FLOAT_PIXELS));
+        OIIO_CHECK_EQUAL (dst2.spec().format, TypeDesc::FLOAT);
+    }
+
+    // Test MINIMIZE_NCHANNELS
+    {
+        ROI roi1, roi2;
+        ImageBuf dst1, dst2;
+        OIIO_CHECK_ASSERT (IBAprep (roi1, &dst1, &rgb, &rgba));
+        OIIO_CHECK_EQUAL (dst1.nchannels(), 4);
+        OIIO_CHECK_ASSERT (IBAprep (roi2, &dst2, &rgb, &rgba, NULL, NULL, IBAprep_MINIMIZE_NCHANNELS));
+        OIIO_CHECK_EQUAL (dst2.nchannels(), 3);
+    }
+#undef CHECK
+}
+
+
 
 int
 main (int argc, char **argv)
@@ -581,7 +697,9 @@ main (int argc, char **argv)
     test_isConstantChannel ();
     test_isMonochrome ();
     test_computePixelStats ();
+    histogram_computation_test ();
     test_maketx_from_imagebuf ();
+    test_IBAprep ();
     
     return unit_test_failures;
 }
