@@ -56,9 +56,11 @@ recursive_mutex imageio_mutex;
 atomic_int oiio_threads (Sysutil::hardware_concurrency());
 atomic_int oiio_exr_threads (Sysutil::hardware_concurrency());
 atomic_int oiio_read_chunk (256);
+int tiff_half (0);
 ustring plugin_searchpath (OIIO_DEFAULT_PLUGIN_SEARCHPATH);
 std::string format_list;   // comma-separated list of all formats
 std::string extension_list;   // list of all extensions for all formats
+std::string library_list;   // list of all libraries for all formats
 }
 
 using namespace pvt;
@@ -158,7 +160,11 @@ attribute (string_view name, TypeDesc type, const void *val)
         return true;
     }
     if (name == "exr_threads" && type == TypeDesc::TypeInt) {
-        oiio_exr_threads = Imath::clamp (*(const int *)val, 0, maxthreads);
+        oiio_exr_threads = Imath::clamp (*(const int *)val, -1, maxthreads);
+        return true;
+    }
+    if (name == "tiff:half" && type == TypeDesc::TypeInt) {
+        tiff_half = *(const int *)val;
         return true;
     }
     if (name == "debug" && type == TypeDesc::TypeInt) {
@@ -198,10 +204,20 @@ getattribute (string_view name, TypeDesc type, void *val)
         *(ustring *)val = ustring(extension_list);
         return true;
     }
+    if (name == "library_list" && type == TypeDesc::TypeString) {
+        if (library_list.empty())
+            pvt::catalog_all_plugins (plugin_searchpath.string());
+        *(ustring *)val = ustring(library_list);
+        return true;
+    }
     if (name == "exr_threads" && type == TypeDesc::TypeInt) {
         *(int *)val = oiio_exr_threads;
         return true;
     }
+    if (name == "tiff:half" && type == TypeDesc::TypeInt) {
+        *(int *)val = tiff_half;
+        return true;
+    }
     if (name == "debug" && type == TypeDesc::TypeInt) {
         *(int *)val = print_debug;
         return true;
@@ -429,18 +445,6 @@ pvt::convert_from_float (const float *src, void *dst, size_t nvals,
 }
 
 
-// DEPRECATED (1.4)
-const void *
-pvt::convert_from_float (const float *src, void *dst, size_t nvals,
-                         long long quant_black, long long quant_white,
-                         long long quant_min, long long quant_max,
-                         TypeDesc format)
-{
-    return convert_from_float (src, dst, nvals, quant_min, quant_max, format);
-}
-
-
-
 const void *
 pvt::parallel_convert_from_float (const float *src, void *dst, size_t nvals,
                                   TypeDesc format, int nthreads)
@@ -479,19 +483,6 @@ pvt::parallel_convert_from_float (const float *src, void *dst, size_t nvals,
 
 
 
-// DEPRECATED (1.4)
-const void *
-pvt::parallel_convert_from_float (const float *src, void *dst,
-                                  size_t nvals,
-                                  long long quant_black, long long quant_white,
-                                  long long quant_min, long long quant_max,
-                                  TypeDesc format, int nthreads)
-{
-    return parallel_convert_from_float (src, dst, nvals, format, nthreads);
-}
-
-
-
 bool
 convert_types (TypeDesc src_type, const void *src, 
                TypeDesc dst_type, void *dst, int n)
@@ -543,17 +534,6 @@ convert_types (TypeDesc src_type, const void *src,
 
 
 
-// Deprecated version -- keep for link compatibiity
-bool
-convert_types (TypeDesc src_type, const void *src, 
-               TypeDesc dst_type, void *dst, int n,
-               int alpha_channel, int z_channel)
-{
-    return convert_types (src_type, src, dst_type, dst, n);
-}
-
-
-
 bool
 convert_image (int nchannels, int width, int height, int depth,
                const void *src, TypeDesc src_type,
@@ -590,14 +570,12 @@ convert_image (int nchannels, int width, int height, int depth,
                 // unit.  (Note that within convert_types, a memcpy will
                 // be used if the formats are identical.)
                 result &= convert_types (src_type, f, dst_type, t,
-                                         nchannels*width,
-                                         alpha_channel, z_channel);
+                                         nchannels*width);
             } else {
                 // General case -- anything goes with strides.
                 for (int x = 0;  x < width;  ++x) {
                     result &= convert_types (src_type, f, dst_type, t,
-                                             nchannels,
-                                             alpha_channel, z_channel);
+                                             nchannels);
                     f += src_xstride;
                     t += dst_xstride;
                 }
