@@ -163,40 +163,6 @@ datestring (time_t t)
 
 
 
-// Copy src into dst, but only for the range [x0,x1) x [y0,y1).
-template<typename SRCTYPE>
-static bool
-copy_block_ (ImageBuf &dst, const ImageBuf &src, ROI roi=ROI())
-{
-    int nchannels = dst.spec().nchannels;
-    ASSERT (src.spec().nchannels == nchannels);
-    ROI image_roi = get_roi (dst.spec());
-    if (roi.defined())
-        image_roi = roi_intersection (image_roi, roi);
-    ImageBuf::Iterator<float> d (dst, image_roi);
-    ImageBuf::ConstIterator<SRCTYPE> s (src, image_roi, ImageBuf::WrapBlack);
-    for (  ; ! d.done();  ++d, ++s) {
-        for (int c = 0;  c < nchannels;  ++c)
-            d[c] = s[c];
-    }
-    return true;
-}
-
-
-
-// Copy src into dst, but only for the range [x0,x1) x [y0,y1).
-static bool
-copy_block (ImageBuf &dst, const ImageBuf &src, ROI roi)
-{
-    ASSERT (dst.spec().format == TypeDesc::TypeFloat);
-    bool ok;
-    OIIO_DISPATCH_TYPES (ok, "copy_block", copy_block_, src.spec().format,
-                         dst, src, roi);
-    return ok;
-}
-
-
-
 template<class SRCTYPE>
 static void
 interppixel_NDC_clamped (const ImageBuf &buf, float x, float y, float *pixel,
@@ -611,7 +577,7 @@ write_mipmap (ImageBufAlgo::MakeTextureMode mode,
     bool verbose = configspec.get_int_attribute ("maketx:verbose") != 0;
     if (verbose) {
         outstream << "  Writing file: " << outputfilename << std::endl;
-        outstream << "  Filter \"" << filtername << "\n";
+        outstream << "  Filter \"" << filtername << "\"\n";
         outstream << "  Top level is " << formatres(outspec) << std::endl;
     }
 
@@ -862,8 +828,11 @@ make_texture_impl (ImageBufAlgo::MakeTextureMode mode,
     // chance a crash during texture conversion will leave behind a
     // partially formed tx with incomplete mipmaps levels which happesn to
     // be extremely slow to use in a raytracer.
+    // We also force a unique filename to protect against multiple maketx
+    // processes running at the same time on the same file.
     std::string extension = Filesystem::extension(outputfilename);
-    std::string tmpfilename = Filesystem::replace_extension (outputfilename, ".temp"+extension);
+    std::string tmpfilename = Filesystem::replace_extension (outputfilename, ".%%%%%%%%.temp"+extension);
+    tmpfilename = Filesystem::unique_path(tmpfilename);
 
     // When was the input file last modified?
     // This is only used when we're reading from a filename
@@ -1363,8 +1332,7 @@ make_texture_impl (ImageBufAlgo::MakeTextureMode mode,
     } else  if (! do_resize) {
         // Need format conversion, but no resize -- just copy the pixels
         toplevel.reset (new ImageBuf (dstspec));
-        ImageBufAlgo::parallel_image (OIIO::bind(copy_block,OIIO::ref(*toplevel),OIIO::cref(*src),_1),
-                                      OIIO::get_roi(dstspec));
+        toplevel->copy_pixels (*src);
     } else {
         // Resize
         if (verbose)
@@ -1416,7 +1384,7 @@ make_texture_impl (ImageBufAlgo::MakeTextureMode mode,
     }
     
     // The hash is only computed for the top mipmap level of pixel data.
-    // Thus, any additional information that will effect the lower levels
+    // Thus, any additional information that will affect the lower levels
     // (such as filtering information) needs to be manually added into the
     // hash.
     std::ostringstream addlHashData;
@@ -1426,6 +1394,8 @@ make_texture_impl (ImageBufAlgo::MakeTextureMode mode,
         addlHashData << "sharpen_A=" << sharpen << " ";
         // NB if we change the sharpening algorithm, change the letter!
     }
+    if (configspec.get_int_attribute ("maketx:highlightcomp", 0))
+        addlHashData << "highlightcomp=1 ";
 
     const int sha1_blocksize = 256;
     std::string hash_digest = configspec.get_int_attribute("maketx:hash", 1) ?
