@@ -108,8 +108,6 @@ public:
     }
 
 
-    virtual void clear () { }
-
     // Retrieve options
     void get_commontoworld (Imath::M44f &result) const {
         result = m_Mc2w;
@@ -321,7 +319,17 @@ private:
         texturefile = m_imagecache->verify_file (texturefile, thread_info);
         if (!texturefile || texturefile->broken()) {
             std::string err = m_imagecache->geterror();
-            error ("%s", err.size() ? err.c_str() : "(unknown error)");
+            if (err.size())
+                error ("%s", err.c_str());
+#if 0
+            // If the file is "broken", at least one verbose error message
+            // has already been issued about it, so don't belabor the point.
+            // But for debugging purposes, these might help:
+            else if (texturefile && texturefile->broken())
+                error ("(unknown error - broken texture \"%s\")", texturefile->filename());
+            else
+                error ("(unknown error - NULL texturefile)");
+#endif
         }
         return texturefile;
     }
@@ -523,6 +531,7 @@ private:
     Imath::M44f m_Mw2c;          ///< world-to-"common" matrix
     Imath::M44f m_Mc2w;          ///< common-to-world matrix
     bool m_gray_to_rgb;          ///< automatically copy gray to rgb channels?
+    bool m_flip_t;               ///< Flip direction of t coord?
     int m_max_tile_channels;     ///< narrow tile ID channel range when
                                  ///<   the file has more channels
     /// Saved error string, per-thread
