@@ -40,17 +40,18 @@
 #include <boost/foreach.hpp>
 #include <boost/regex.hpp>
 
+#include "OpenImageIO/platform.h"
 #include "OpenImageIO/dassert.h"
 #include "OpenImageIO/ustring.h"
 #include "OpenImageIO/filesystem.h"
 #include "OpenImageIO/refcnt.h"
 
 #ifdef _WIN32
-#include <windows.h>
-#include <shellapi.h>
-#include <direct.h>
+// # include <windows.h>   // Already done by platform.h
+# include <shellapi.h>
+# include <direct.h>
 #else
-#include <unistd.h>
+# include <unistd.h>
 #endif
 
 
@@ -166,12 +167,16 @@ Filesystem::searchpath_find (const std::string &filename_utf8,
 #else
     const boost::filesystem::path filename (filename_utf8);
 #endif
+#if BOOST_FILESYSTEM_VERSION == 3
     bool abs = filename.is_absolute();
+#else
+    bool abs = path_is_absolute (filename.string());
+#endif
 
     // If it's an absolute filename, or if we want to check "." first,
     // then start by checking filename outright.
     if (testcwd || abs) {
-        if (boost::filesystem::is_regular_file (filename))
+        if (is_regular (filename.string()))
             return filename_utf8;
     }
 
@@ -185,7 +190,7 @@ Filesystem::searchpath_find (const std::string &filename_utf8,
 #endif
         boost::filesystem::path f = d / filename;
         // std::cerr << "\tTesting '" << f.string() << "'\n";
-        if (boost::filesystem::is_regular_file (f)) {
+        if (is_regular (f.string())) {
             // std::cerr << "Found '" << f.string() << "'\n";
 #ifdef _WIN32
             return Strutil::utf16_to_utf8 (f.native());
@@ -194,11 +199,11 @@ Filesystem::searchpath_find (const std::string &filename_utf8,
 #endif
         }
 
-        if (recursive && boost::filesystem::is_directory (d)) {
+        if (recursive && is_directory (d.string())) {
             std::vector<std::string> subdirs;
             boost::filesystem::directory_iterator end_iter;
             for (boost::filesystem::directory_iterator s(d); s != end_iter; ++s) {
-                if (boost::filesystem::is_directory (s->status())) {
+                if (is_directory (s->path().string())) {
 #ifdef _WIN32
                     subdirs.push_back (Strutil::utf16_to_utf8 (s->path().native()));
 #else
@@ -243,7 +248,11 @@ Filesystem::get_directory_entries (const std::string &dirname,
              s != boost::filesystem::recursive_directory_iterator();  ++s) {
             std::string file = s->path().string();
             if (!filter_regex.size() || boost::regex_search (file, re))
+#ifdef _WIN32
+                filenames.push_back (Strutil::utf16_to_utf8(s->path().native()));
+#else
                 filenames.push_back (file);
+#endif
         }
     } else {
 #ifdef _WIN32
@@ -255,7 +264,11 @@ Filesystem::get_directory_entries (const std::string &dirname,
              s != boost::filesystem::directory_iterator();  ++s) {
             std::string file = s->path().string();
             if (!filter_regex.size() || boost::regex_search (file, re))
+#ifdef _WIN32
+                filenames.push_back (Strutil::utf16_to_utf8(s->path().native()));
+#else
                 filenames.push_back (file);
+#endif
         }
     }
     return true;
@@ -1037,7 +1050,7 @@ Filesystem::scan_for_matching_filenames(const std::string &pattern_,
 #else
     for (boost::filesystem::directory_iterator it(directory); it != end_it; ++it) {
 #endif
-        if (boost::filesystem::is_regular_file(it->status())) {
+        if (is_regular(it->path().string())) {
             const std::string f = it->path().string();
             boost::match_results<std::string::const_iterator> frame_match;
             if (boost::regex_match (f, frame_match, pattern_re)) {
