@@ -39,26 +39,17 @@
 #include <boost/foreach.hpp>
 #include <boost/algorithm/string.hpp>
 
-#ifdef _WIN32
-// defining NOMINMAX to prevent problems with std::min/std::max
-// and std::numeric_limits<type>::min()/std::numeric_limits<type>::max()
-// when including windows.h
-#ifdef _MSC_VER
-# define WIN32_LEAN_AND_MEAN
-# define VC_EXTRALEAN
-# ifndef NOMINMAX
-#   define NOMINMAX
-# endif
-#endif
-#include <windows.h>
-#include <shellapi.h>
-#endif
-
+#include "OpenImageIO/platform.h"
 #include "OpenImageIO/dassert.h"
 #include "OpenImageIO/strutil.h"
 #include "OpenImageIO/ustring.h"
 #include "OpenImageIO/string_view.h"
 
+#ifdef _WIN32
+# include <shellapi.h>
+#endif
+
+
 
 OIIO_NAMESPACE_BEGIN
 
@@ -309,15 +300,30 @@ Strutil::wordwrap (string_view src, int columns, int prefix)
 
 
 
-namespace {
-static std::locale loc = std::locale::classic();
+
+static std::locale& loc ()
+{
+    // Rather than have a file-level static locale which could suffer from
+    // the static initialization order fiasco, make it a static pointer
+    // inside the function that is allocated the first time it's needed. It
+    // will "leak" in the sense that this one locale is never deleted, but
+    // so what?
+#if OIIO_MSVS_BEFORE_2015
+    // MSVS prior to 2015 could not be trusted to make in-function static
+    // initialization thread-safe, as required by C++11.
+    static spin_mutex mutex;
+    spin_lock lock (mutex);
+#endif
+    static std::locale *loc = new std::locale (std::locale::classic());
+    return *loc;
 }
 
 
+
 bool
 Strutil::iequals (string_view a, string_view b)
 {
-    return boost::algorithm::iequals (a, b, loc);
+    return boost::algorithm::iequals (a, b, loc());
 }
 
 
@@ -331,7 +337,7 @@ Strutil::starts_with (string_view a, string_view b)
 bool
 Strutil::istarts_with (string_view a, string_view b)
 {
-    return boost::algorithm::istarts_with (a, b, loc);
+    return boost::algorithm::istarts_with (a, b, loc());
 }
 
 
@@ -345,7 +351,7 @@ Strutil::ends_with (string_view a, string_view b)
 bool
 Strutil::iends_with (string_view a, string_view b)
 {
-    return boost::algorithm::iends_with (a, b, loc);
+    return boost::algorithm::iends_with (a, b, loc());
 }
 
 
@@ -359,21 +365,21 @@ Strutil::contains (string_view a, string_view b)
 bool
 Strutil::icontains (string_view a, string_view b)
 {
-    return boost::algorithm::icontains (a, b, loc);
+    return boost::algorithm::icontains (a, b, loc());
 }
 
 
 void
 Strutil::to_lower (std::string &a)
 {
-    boost::algorithm::to_lower (a, loc);
+    boost::algorithm::to_lower (a, loc());
 }
 
 
 void
 Strutil::to_upper (std::string &a)
 {
-    boost::algorithm::to_upper (a, loc);
+    boost::algorithm::to_upper (a, loc());
 }
 
 
@@ -501,14 +507,40 @@ Strutil::repeat (string_view str, int n)
 
 
 
+std::string
+Strutil::replace (string_view str, string_view pattern,
+                  string_view replacement, bool global)
+{
+    std::string r;
+    while (1) {
+        size_t f = str.find (pattern);
+        if (f != str.npos) {
+            // Pattern found -- copy the part of str prior to the pattern,
+            // then copy the replacement, and skip str up to the part after
+            // the pattern and continue for another go at it.
+            r.append (str.data(), f);
+            r.append (replacement.data(), replacement.size());
+            str.remove_prefix (f + pattern.size());
+            if (global)
+                continue;   // Try for another match
+        }
+        // Pattern not found -- copy remaining string and we're done
+        r.append (str.data(), str.size());
+        break;
+    }
+    return r;
+}
+
+
+
 #ifdef _WIN32
 std::wstring
 Strutil::utf8_to_utf16 (string_view str)
 {
     std::wstring native;
     
-    native.resize(MultiByteToWideChar (CP_UTF8, 0, str.c_str(), -1, NULL, 0));
-    MultiByteToWideChar (CP_UTF8, 0, str.c_str(), -1, &native[0], (int)native.size());
+    native.resize(MultiByteToWideChar (CP_UTF8, 0, str.data(), str.length(), NULL, 0));
+    MultiByteToWideChar (CP_UTF8, 0, str.data(), str.length(), &native[0], (int)native.size());
 
     return native;
 }
@@ -520,8 +552,8 @@ Strutil::utf16_to_utf8 (const std::wstring& str)
 {
     std::string utf8;
 
-    utf8.resize(WideCharToMultiByte (CP_UTF8, 0, str.c_str(), -1, NULL, 0, NULL, NULL));
-    WideCharToMultiByte (CP_UTF8, 0, str.c_str(), -1, &utf8[0], (int)utf8.size(), NULL, NULL);
+    utf8.resize(WideCharToMultiByte (CP_UTF8, 0, str.data(), str.length(), NULL, 0, NULL, NULL));
+    WideCharToMultiByte (CP_UTF8, 0, str.data(), str.length(), &utf8[0], (int)utf8.size(), NULL, NULL);
 
     return utf8;
 }
@@ -647,14 +679,6 @@ Strutil::parse_float (string_view &str, float &val, bool eat)
 
 
 
-bool
-Strutil::parse_string (string_view &str, string_view &val, bool eat)
-{
-    return parse_string (str, val, eat, DeleteQuotes);
-}
-
-
-
 bool
 Strutil::parse_string (string_view &str, string_view &val,
                        bool eat, QuoteBehavior keep_quotes)
@@ -754,6 +778,20 @@ Strutil::parse_identifier (string_view &str, string_view allowed, bool eat)
 
 
 
+bool
+Strutil::parse_identifier_if (string_view &str, string_view id, bool eat)
+{
+    string_view head = parse_identifier (str, false /* don't eat */);
+    if (head == id) {
+        if (eat)
+            parse_identifier (str);
+        return true;
+    }
+    return false;
+}
+
+
+
 string_view
 Strutil::parse_until (string_view &str, string_view sep, bool eat)
 {
@@ -872,7 +910,7 @@ decode(uint32_t* state, uint32_t* codep, uint32_t byte)
 }
 
 void
-Strutil::utf8_to_unicode (string_view &str, std::vector<uint32_t> &uvec)
+Strutil::utf8_to_unicode (string_view str, std::vector<uint32_t> &uvec)
 {
     const char* begin = str.begin();
     const char* end = str.end();
@@ -884,4 +922,66 @@ Strutil::utf8_to_unicode (string_view &str, std::vector<uint32_t> &uvec)
     }
 }
 
+
+
+/* base64 code is based upon: http://www.adp-gmbh.ch/cpp/common/base64.html
+   Copyright (C) 2004-2008 René Nyffenegger
+
+   This source code is provided 'as-is', without any express or implied
+   warranty. In no event will the author be held liable for any damages
+   arising from the use of this software.
+
+   Permission is granted to anyone to use this software for any purpose,
+   including commercial applications, and to alter it and redistribute it
+   freely, subject to the following restrictions:
+
+   1. The origin of this source code must not be misrepresented; you must not
+      claim that you wrote the original source code. If you use this source code
+      in a product, an acknowledgment in the product documentation would be
+      appreciated but is not required.
+   2. Altered source versions must be plainly marked as such, and must not be
+      misrepresented as being the original source code.
+   3. This notice may not be removed or altered from any source distribution.
+
+   René Nyffenegger rene.nyffenegger@adp-gmbh.ch
+*/
+std::string
+Strutil::base64_encode (string_view str)
+{
+    static const char *base64_chars =
+        "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/";
+    std::string ret;
+    ret.reserve ((str.size() * 4 + 2)/ 3);
+    int i = 0;
+    unsigned char char_array_3[3];
+    unsigned char char_array_4[4];
+    while (str.size()) {
+        char_array_3[i++] = str.front();
+        str.remove_prefix (1);
+        if (i == 3) {
+            char_array_4[0] = (char_array_3[0] & 0xfc) >> 2;
+            char_array_4[1] = ((char_array_3[0] & 0x03) << 4) + ((char_array_3[1] & 0xf0) >> 4);
+            char_array_4[2] = ((char_array_3[1] & 0x0f) << 2) + ((char_array_3[2] & 0xc0) >> 6);
+            char_array_4[3] = char_array_3[2] & 0x3f;
+            for (int j = 0; j < 4; j++)
+                ret += base64_chars[char_array_4[j]];
+            i = 0;
+        }
+    }
+    if (i) {
+        for (int j = i; j < 3; j++)
+            char_array_3[j] = '\0';
+        char_array_4[0] = (char_array_3[0] & 0xfc) >> 2;
+        char_array_4[1] = ((char_array_3[0] & 0x03) << 4) + ((char_array_3[1] & 0xf0) >> 4);
+        char_array_4[2] = ((char_array_3[1] & 0x0f) << 2) + ((char_array_3[2] & 0xc0) >> 6);
+        char_array_4[3] = char_array_3[2] & 0x3f;
+        for (int j = 0; j < i + 1; j++)
+            ret += base64_chars[char_array_4[j]];
+        while (i++ < 3)
+            ret += '=';
+    }
+    return ret;
+}
+
+
 OIIO_NAMESPACE_END
