@@ -52,15 +52,6 @@ TypeDesc::TypeDesc (string_view typestring)
 
 
 
-TypeDesc::TypeDesc (const char *typestring)
-    : basetype(UNKNOWN), aggregate(SCALAR), vecsemantics(NOXFORM),
-      reserved(0), arraylen(0)
-{
-    fromstring (typestring);
-}
-
-
-
 namespace {
 
 static int basetype_size[] = {
@@ -271,14 +262,6 @@ copy_until (const char *src, const char *delim, char *dst, size_t maxlen)
 
 
 
-size_t
-TypeDesc::fromstring (const char *typestring)
-{
-    return fromstring (string_view(typestring));
-}
-
-
-
 size_t
 TypeDesc::fromstring (string_view typestring)
 {
@@ -316,6 +299,8 @@ TypeDesc::fromstring (string_view typestring)
         t = TypeMatrix33;
     else if (type == "matrix" || type == "matrix44")
         t = TypeMatrix44;
+    else if (type == "timecode")
+        t = TypeTimeCode;
     else {
         return 0;  // unknown
     }
