@@ -229,7 +229,9 @@ static const int one = 1;
 //**************************************
 // Macros
 //**************************************
-#define XXH_STATIC_ASSERT(c)   { enum { XXH_static_assert = 1/(!!(c)) }; }    // use only *after* variable declarations
+// FIXME -- replace with with static_assert when we're on C++11.
+// This construct gives warnings for some compilers.
+#define XXH_STATIC_ASSERT(c)   { enum { XXH_static_assert = 1/int(!!(c)) }; }    // use only *after* variable declarations
 
 
 //****************************
