@@ -70,9 +70,6 @@ static bool envlatlmode = false;
 static bool envcubemode = false;
 static bool lightprobemode = false;
 
-static ColorConfig colorconfig;
-
-
 
 
 static std::string
@@ -115,6 +112,7 @@ colorconvert_help_string ()
     "to the proper bit depth using the -d option. ";
     
     s += " (choices: ";
+    ColorConfig colorconfig;
     if (colorconfig.error() || colorconfig.getNumColorSpaces()==0) {
         s += "NONE";
     } else {
@@ -224,8 +222,6 @@ getargs (int argc, char *argv[], ImageSpec &configspec)
                   "--help", &help, "Print help message",
                   "-v", &verbose, "Verbose status messages",
                   "-o %s", &outputfilename, "Output filename",
-                  "--new", NULL, "",
-                  "--old", NULL, "Old mode",
                   "--threads %d", &nthreads, "Number of threads (default: #cores)",
                   "-u", &updatemode, "Update mode",
                   "--format %s", &fileformatname, "Specify output file format (default: guess from extension)",
@@ -262,7 +258,6 @@ getargs (int argc, char *argv[], ImageSpec &configspec)
                           &Mscr[2][0], &Mscr[2][1], &Mscr[2][2], &Mscr[2][3], 
                           &Mscr[3][0], &Mscr[3][1], &Mscr[3][2], &Mscr[3][3], 
                           "Set the screen matrix",
-                  "--hash", NULL, "",
                   "--prman-metadata", &prman_metadata, "Add prman specific metadata",
                   "--attrib %L %L", &any_attrib_names, &any_attrib_values, "Sets metadata attribute (name, value)",
                   "--sattrib %L %L", &string_attrib_names, &string_attrib_values, "Sets string metadata attribute (name, value)",
