@@ -57,6 +57,7 @@
 #include <OpenEXR/ImfKeyCodeAttribute.h>
 #include <OpenEXR/ImfEnvmapAttribute.h>
 #include <OpenEXR/ImfCompressionAttribute.h>
+#include <OpenEXR/ImfChromaticitiesAttribute.h>
 #include <OpenEXR/IexBaseExc.h>
 #include <OpenEXR/IexThrowErrnoExc.h>
 #ifdef USE_OPENEXR_VERSION2
@@ -84,6 +85,7 @@
 #include "OpenImageIO/fmath.h"
 #include "OpenImageIO/filesystem.h"
 #include "OpenImageIO/imagebufalgo_util.h"
+#include "OpenImageIO/deepdata.h"
 #include "OpenImageIO/sysutil.h"
 
 #include <boost/scoped_array.hpp>
@@ -298,12 +300,11 @@ private:
         // FIXME: Things to consider in the future:
         // preview
         // screenWindowCenter
-        // chromaticities whiteLuminance adoptedNeutral
+        // adoptedNeutral
         // renderingTransform, lookModTransform
         // utcOffset
         // longitude latitude altitude
         // focus isoSpeed
-        // keyCode timeCode framesPerSecond
     }
 };
 
@@ -576,6 +577,7 @@ OpenEXRInput::PartInfo::parse_header (const Imf::Header *header)
         const Imf::Box2fAttribute *b2fattr;
         const Imf::TimeCodeAttribute *tattr;
         const Imf::KeyCodeAttribute *kcattr;
+        const Imf::ChromaticitiesAttribute *crattr;
 #ifdef USE_OPENEXR_VERSION2
         const Imf::StringVectorAttribute *svattr;
         const Imf::DoubleAttribute *dattr;
@@ -698,6 +700,11 @@ OpenEXRInput::PartInfo::parse_header (const Imf::Header *header)
             if (oname == "keyCode")
                 oname = "smpte:KeyCode";
             spec.attribute(oname, TypeDesc::TypeKeyCode, keycode);
+        } else if (type == "chromaticities" &&
+                   (crattr = header->findTypedAttribute<Imf::ChromaticitiesAttribute> (name))) {
+            const Imf::Chromaticities *chroma = &crattr->value();
+            spec.attribute (oname, TypeDesc(TypeDesc::FLOAT,8),
+                            (const float *)chroma);
         }
         else {
 #if 0
@@ -735,7 +742,7 @@ TypeDesc_from_ImfPixelType (Imf::PixelType ptype)
     case Imf::UINT  : return TypeDesc::UINT;  break;
     case Imf::HALF  : return TypeDesc::HALF;  break;
     case Imf::FLOAT : return TypeDesc::FLOAT; break;
-    default: ASSERT (0);
+    default: ASSERT_MSG (0, "Unknown Imf::PixelType %d", int(ptype));
     }
 }
 
@@ -763,8 +770,9 @@ struct ChanNameHolder {
             suffix = string_view (fullname.data()+dot+1, fullname.size()-dot-1);
         }
         static const char * special[] = {
-            "R", "Red", "G", "Green", "B", "Blue", "real", "imag",
-            "A", "Alpha", "RA", "RG", "RB", "Z", "Depth", "Zback", NULL
+            "R", "Red", "G", "Green", "B", "Blue", "Y", "real", "imag",
+            "A", "Alpha", "AR", "RA", "AG", "GA", "AB", "BA",
+            "Z", "Depth", "Zback", NULL
         };
         special_index = 10000;
         for (int i = 0; special[i]; ++i)
@@ -886,7 +894,6 @@ OpenEXRInput::seek_subimage (int subimage, int miplevel, ImageSpec &newspec)
             m_tiled_input_part = NULL;
             m_deep_scanline_input_part = NULL;
             m_deep_tiled_input_part = NULL;
-            ASSERT(0);
             return false;
         } catch (...) {   // catch-all for edge cases or compiler bugs
             error ("OpenEXR exception: unknown");
@@ -894,7 +901,6 @@ OpenEXRInput::seek_subimage (int subimage, int miplevel, ImageSpec &newspec)
             m_tiled_input_part = NULL;
             m_deep_scanline_input_part = NULL;
             m_deep_tiled_input_part = NULL;
-            ASSERT(0);
             return false;
         }
     }
@@ -930,7 +936,7 @@ OpenEXRInput::seek_subimage (int subimage, int miplevel, ImageSpec &newspec)
     } else if (part.levelmode == Imf::RIPMAP_LEVELS) {
         // FIXME
     } else {
-        ASSERT(0);
+        ASSERT_MSG (0, "Unknown levelmode %d", int(part.levelmode));
     }
 
     m_spec.width = w;
@@ -1040,7 +1046,8 @@ OpenEXRInput::read_native_scanlines (int ybegin, int yend, int z,
             m_scanline_input_part->readPixels (ybegin, yend-1);
 #endif
         } else {
-            ASSERT (0);
+            error ("Attempted to read scanline from a non-scanline file.");
+            return false;
         }
     } catch (const std::exception &e) {
         error ("Failed OpenEXR read: %s", e.what());
@@ -1146,7 +1153,8 @@ OpenEXRInput::read_native_tiles (int xbegin, int xend, int ybegin, int yend,
                                            m_miplevel, m_miplevel);
 #endif
         } else {
-            ASSERT (0);
+            error ("Attempted to read tiles from a non-tiled file");
+            return false;
         }
         if (data != origdata) {
             stride_t user_scanline_bytes = (xend-xbegin) * pixelbytes;
@@ -1189,24 +1197,28 @@ OpenEXRInput::read_native_deep_scanlines (int ybegin, int yend, int z,
         // Set up the count and pointers arrays and the Imf framebuffer
         std::vector<TypeDesc> channeltypes;
         m_spec.get_channelformats (channeltypes);
-        deepdata.init (npixels, nchans, &channeltypes[chbegin],
-                       &channeltypes[chend]);
+        deepdata.init (npixels, nchans,
+                       array_view<const TypeDesc>(&channeltypes[chbegin], chend-chbegin),
+                       spec().channelnames);
+        std::vector<unsigned int> all_samples (npixels);
+        std::vector<void*> pointerbuf (npixels*nchans);
         Imf::DeepFrameBuffer frameBuffer;
         Imf::Slice countslice (Imf::UINT,
-                               (char *)(&deepdata.nsamples[0]
+                               (char *)(&all_samples[0]
                                         - m_spec.x
                                         - ybegin*m_spec.width),
                                sizeof(unsigned int),
                                sizeof(unsigned int) * m_spec.width);
         frameBuffer.insertSampleCountSlice (countslice);
+
         for (int c = chbegin;  c < chend;  ++c) {
             Imf::DeepSlice slice (part.pixeltype[c],
-                                  (char *)(&deepdata.pointers[c-chbegin]
+                                  (char *)(&pointerbuf[0]+(c-chbegin)
                                            - m_spec.x * nchans
                                            - ybegin*m_spec.width*nchans),
                                   sizeof(void*) * nchans, // xstride of pointer array
                                   sizeof(void*) * nchans*m_spec.width, // ystride of pointer array
-                                  part.chanbytes[c]); // stride of data sample
+                                  deepdata.samplesize()); // stride of data sample
             frameBuffer.insert (m_spec.channelnames[c].c_str(), slice);
         }
         m_deep_scanline_input_part->setFrameBuffer (frameBuffer);
@@ -1214,10 +1226,12 @@ OpenEXRInput::read_native_deep_scanlines (int ybegin, int yend, int z,
         // Get the sample counts for each pixel and compute the total
         // number of samples and resize the data area appropriately.
         m_deep_scanline_input_part->readPixelSampleCounts (ybegin, yend-1);
-        deepdata.alloc ();
+        deepdata.set_all_samples (all_samples);
+        deepdata.get_pointers (pointerbuf);
 
         // Read the pixels
         m_deep_scanline_input_part->readPixels (ybegin, yend-1);
+        // deepdata.import_chansamp (pointerbuf);
     } catch (const std::exception &e) {
         error ("Failed OpenEXR read: %s", e.what());
         return false;
@@ -1258,11 +1272,14 @@ OpenEXRInput::read_native_deep_tiles (int xbegin, int xend,
         // Set up the count and pointers arrays and the Imf framebuffer
         std::vector<TypeDesc> channeltypes;
         m_spec.get_channelformats (channeltypes);
-        deepdata.init (npixels, nchans, &channeltypes[chbegin],
-                       &channeltypes[chend]);
+        deepdata.init (npixels, nchans,
+                       array_view<const TypeDesc>(&channeltypes[chbegin], chend-chbegin),
+                       spec().channelnames);
+        std::vector<unsigned int> all_samples (npixels);
+        std::vector<void*> pointerbuf (npixels * nchans);
         Imf::DeepFrameBuffer frameBuffer;
         Imf::Slice countslice (Imf::UINT,
-                               (char *)(&deepdata.nsamples[0]
+                               (char *)(&all_samples[0]
                                         - xbegin
                                         - ybegin*width),
                                sizeof(unsigned int),
@@ -1270,12 +1287,12 @@ OpenEXRInput::read_native_deep_tiles (int xbegin, int xend,
         frameBuffer.insertSampleCountSlice (countslice);
         for (int c = chbegin;  c < chend;  ++c) {
             Imf::DeepSlice slice (part.pixeltype[c],
-                                  (char *)(&deepdata.pointers[c-chbegin]
+                                  (char *)(&pointerbuf[0]+(c-chbegin)
                                            - xbegin*nchans
                                            - ybegin*width*nchans),
                                   sizeof(void*) * nchans, // xstride of pointer array
                                   sizeof(void*) * nchans*width, // ystride of pointer array
-                                  part.chanbytes[c]); // stride of data sample
+                                  deepdata.samplesize()); // stride of data sample
             frameBuffer.insert (m_spec.channelnames[c].c_str(), slice);
         }
         m_deep_tiled_input_part->setFrameBuffer (frameBuffer);
@@ -1291,7 +1308,8 @@ OpenEXRInput::read_native_deep_tiles (int xbegin, int xend,
         m_deep_tiled_input_part->readPixelSampleCounts (
                 firstxtile, firstxtile+xtiles-1,
                 firstytile, firstytile+ytiles-1);
-        deepdata.alloc ();
+        deepdata.set_all_samples (all_samples);
+        deepdata.get_pointers (pointerbuf);
 
         // Read the pixels
         m_deep_tiled_input_part->readTiles (
