@@ -56,6 +56,7 @@
 #include <OpenEXR/ImfBoxAttribute.h>
 #include <OpenEXR/ImfEnvmapAttribute.h>
 #include <OpenEXR/ImfCompressionAttribute.h>
+#include <OpenEXR/ImfChromaticitiesAttribute.h>
 #include <OpenEXR/ImfCRgbaFile.h>  // JUST to get symbols to figure out version!
 #include <OpenEXR/IexBaseExc.h>
 #include <OpenEXR/IexThrowErrnoExc.h>
@@ -80,6 +81,7 @@
 
 #include "OpenImageIO/dassert.h"
 #include "OpenImageIO/imageio.h"
+#include "OpenImageIO/deepdata.h"
 #include "OpenImageIO/filesystem.h"
 #include "OpenImageIO/thread.h"
 #include "OpenImageIO/strutil.h"
@@ -217,6 +219,10 @@ private:
     // Decode the IlmImf MIP parameters from the spec.
     static void figure_mip (const ImageSpec &spec, int &nmiplevels,
                             int &levelmode, int &roundingmode);
+
+    // Helper: if the channel names are nonsensical, fix them to keep the
+    // app from shooting itself in the foot.
+    void sanity_check_channelnames ();
 };
 
 
@@ -233,6 +239,14 @@ openexr_output_imageio_create ()
 
 OIIO_EXPORT int openexr_imageio_version = OIIO_PLUGIN_VERSION;
 
+OIIO_EXPORT const char* openexr_imageio_library_version () {
+#ifdef OPENEXR_PACKAGE_STRING
+    return OPENEXR_PACKAGE_STRING;
+#else
+    return "OpenEXR 1.x";
+#endif
+}
+
 OIIO_EXPORT const char * openexr_output_extensions[] = {
     "exr", "sxr", "mxr", NULL
 };
@@ -342,6 +356,7 @@ OpenEXROutput::open (const std::string &name, const ImageSpec &userspec,
         m_miplevel = 0;
         m_headers.resize (1);
         m_spec = userspec;  // Stash the spec
+        sanity_check_channelnames ();
 
         if (! spec_to_header (m_spec, m_subimage, m_headers[m_subimage]))
             return false;
@@ -403,7 +418,8 @@ OpenEXROutput::open (const std::string &name, const ImageSpec &userspec,
                 delete m_deep_scanline_output_part;
                 m_deep_scanline_output_part = new Imf::DeepScanLineOutputPart (*m_output_multipart, m_subimage);
             } else {
-                ASSERT (0);
+                error ("Called open with AppendSubimage mode, but no appropriate part is found. Application bug?");
+                return false;
             }
         } catch (const std::exception &e) {
             error ("OpenEXR exception: %s", e.what());
@@ -421,6 +437,7 @@ OpenEXROutput::open (const std::string &name, const ImageSpec &userspec,
             return false;
         }
         m_spec = m_subimagespecs[m_subimage];
+        sanity_check_channelnames ();
         compute_pixeltypes(m_spec);
         return true;
 #else
@@ -457,7 +474,7 @@ OpenEXROutput::open (const std::string &name, const ImageSpec &userspec,
         }
     }
 
-    ASSERTMSG (0, "Unknown open mode %d", int(mode));
+    error ("Unknown open mode %d", int(mode));
     return false;
 }
 
@@ -506,6 +523,7 @@ OpenEXROutput::open (const std::string &name, int subimages,
     }
 
     m_spec = m_subimagespecs[0];
+    sanity_check_channelnames ();
     compute_pixeltypes(m_spec);
 
     // Create an ImfMultiPartOutputFile
@@ -681,14 +699,31 @@ OpenEXROutput::spec_to_header (ImageSpec &spec, int subimage, Imf::Header &heade
 #endif
     }
 
-    // Default to ZIP compression if no request came with the user spec.
-    if (! spec.find_attribute("compression"))
-        spec.attribute ("compression", "zip");
-
+    // See what compression has been requested, default to ZIP compression
+    // if no request came with the user spec.
+    string_view compression = spec.get_string_attribute ("compression", "zip");
     // It seems that zips is the only compression that can reliably work
-    // on deep files.
-    if (spec.deep)
-        spec.attribute ("compression", "zips");
+    // on deep files (but allow "none" as well)
+    if (spec.deep && compression != "none")
+        compression = "zips";
+    // Separate any appended quality from the name
+    size_t sep = compression.find_first_of (":");
+    if (sep != compression.npos) {
+        string_view qual = compression.substr (sep+1);
+        compression = compression.substr (0, sep);
+        if (qual.size() && Strutil::istarts_with (compression, "dwa")) {
+            float q = Strutil::from_string<float>(qual);
+            q = clamp (q, 10.0f, 250000.0f);  // useful range
+            spec.attribute ("openexr:dwaCompressionLevel", q);
+        }
+    }
+    spec.attribute ("compression", compression);
+
+    // If compression is one of the DWA types and no compression level
+    // was set, default to 45.
+    if (Strutil::istarts_with (compression, "dwa") &&
+        ! spec.find_attribute("openexr:dwaCompressionLevel"))
+        spec.attribute ("openexr:dwaCompressionLevel", 45.0f);
 
     // Default to increasingY line order
     if (! spec.find_attribute("openexr:lineOrder"))
@@ -1188,6 +1223,14 @@ OpenEXROutput::put_parameter (const std::string &name, TypeDesc type,
 #endif
             }
         }
+        if (type.basetype == TypeDesc::FLOAT && type.aggregate * type.arraylen == 8
+            && Strutil::iequals (xname, "chromaticities")) {
+            const float *f = (const float *)data;
+            Imf::Chromaticities c (Imath::V2f(f[0], f[1]), Imath::V2f(f[2], f[3]),
+                                   Imath::V2f(f[4], f[5]), Imath::V2f(f[6], f[7]));
+            header.insert ("chromaticities", Imf::ChromaticitiesAttribute (c));
+            return true;
+        }
 #ifdef USE_OPENEXR_VERSION2
         // String Vector
         if (type.basetype == TypeDesc::STRING) {
@@ -1212,6 +1255,26 @@ OpenEXROutput::put_parameter (const std::string &name, TypeDesc type,
 
 
 
+void
+OpenEXROutput::sanity_check_channelnames ()
+{
+    m_spec.channelnames.resize (m_spec.nchannels, "");
+    for (int c = 1; c < m_spec.nchannels; ++c) {
+        for (int i = 0; i < c; ++i) {
+            if (m_spec.channelnames[c].empty() ||
+                m_spec.channelnames[c] == m_spec.channelnames[i]) {
+                // Duplicate or missing channel name! We don't want
+                // libIlmImf to drop the channel (as it will do for
+                // duplicates), so rename it and hope for the best.
+                m_spec.channelnames[c] = Strutil::format ("channel%d", c);
+                break;
+            }
+        }
+    }
+}
+
+
+
 bool
 OpenEXROutput::close ()
 {
@@ -1285,7 +1348,8 @@ OpenEXROutput::write_scanline (int y, int z, TypeDesc format,
             m_scanline_output_part->writePixels (1);
 #endif
         } else {
-            ASSERT (0);
+            error ("Attempt to write scanline to a non-scanline file.");
+            return false;
         }
     } catch (const std::exception &e) {
         error ("Failed OpenEXR write: %s", e.what());
@@ -1362,7 +1426,8 @@ OpenEXROutput::write_scanlines (int ybegin, int yend, int z,
                 m_scanline_output_part->writePixels (nscanlines);
 #endif
             } else {
-                ASSERT (0);
+                error ("Attempt to write scanlines to a non-scanline file.");
+                return false;
             }
         } catch (const std::exception &e) {
             error ("Failed OpenEXR write: %s", e.what());
@@ -1492,7 +1557,8 @@ OpenEXROutput::write_tiles (int xbegin, int xend, int ybegin, int yend,
                                              m_miplevel, m_miplevel);
 #endif
         } else {
-            ASSERT (0);
+            error ("Attempt to write tiles for a non-tiled file.");
+            return false;
         }
     } catch (const std::exception &e) {
         error ("Failed OpenEXR write: %s", e.what());
@@ -1515,8 +1581,8 @@ OpenEXROutput::write_deep_scanlines (int ybegin, int yend, int z,
         error ("called OpenEXROutput::write_deep_scanlines without an open file");
         return false;
     }
-    if (m_spec.width*(yend-ybegin) != deepdata.npixels ||
-        m_spec.nchannels != deepdata.nchannels) {
+    if (m_spec.width*(yend-ybegin) != deepdata.pixels() ||
+        m_spec.nchannels != deepdata.channels()) {
         error ("called OpenEXROutput::write_deep_scanlines with non-matching DeepData size");
         return false;
     }
@@ -1527,21 +1593,22 @@ OpenEXROutput::write_deep_scanlines (int ybegin, int yend, int z,
         // Set up the count and pointers arrays and the Imf framebuffer
         Imf::DeepFrameBuffer frameBuffer;
         Imf::Slice countslice (Imf::UINT,
-                               (char *)(&deepdata.nsamples[0]
+                               (char *)(deepdata.all_samples().data()
                                         - m_spec.x
                                         - ybegin*m_spec.width),
                                sizeof(unsigned int),
                                sizeof(unsigned int) * m_spec.width);
         frameBuffer.insertSampleCountSlice (countslice);
+        std::vector<void*> pointerbuf;
+        deepdata.get_pointers (pointerbuf);
         for (int c = 0;  c < nchans;  ++c) {
-            size_t chanbytes = deepdata.channeltypes[c].size();
             Imf::DeepSlice slice (m_pixeltype[c],
-                                  (char *)(&deepdata.pointers[c]
+                                  (char *)(&pointerbuf[c]
                                            - m_spec.x * nchans
                                            - ybegin*m_spec.width*nchans),
                                   sizeof(void*) * nchans, // xstride of pointer array
                                   sizeof(void*) * nchans*m_spec.width, // ystride of pointer array
-                                  chanbytes); // stride of data sample
+                                  deepdata.samplesize()); // stride of data sample
             frameBuffer.insert (m_spec.channelnames[c].c_str(), slice);
         }
         m_deep_scanline_output_part->setFrameBuffer (frameBuffer);
@@ -1575,8 +1642,8 @@ OpenEXROutput::write_deep_tiles (int xbegin, int xend, int ybegin, int yend,
         error ("called OpenEXROutput::write_deep_tiles without an open file");
         return false;
     }
-    if ((xend-xbegin)*(yend-ybegin)*(zend-zbegin) != deepdata.npixels ||
-        m_spec.nchannels != deepdata.nchannels) {
+    if ((xend-xbegin)*(yend-ybegin)*(zend-zbegin) != deepdata.pixels() ||
+        m_spec.nchannels != deepdata.channels()) {
         error ("called OpenEXROutput::write_deep_tiles with non-matching DeepData size");
         return false;
     }
@@ -1589,21 +1656,22 @@ OpenEXROutput::write_deep_tiles (int xbegin, int xend, int ybegin, int yend,
         // Set up the count and pointers arrays and the Imf framebuffer
         Imf::DeepFrameBuffer frameBuffer;
         Imf::Slice countslice (Imf::UINT,
-                               (char *)(&deepdata.nsamples[0]
+                               (char *)(deepdata.all_samples().data()
                                         - xbegin
                                         - ybegin*width),
                                sizeof(unsigned int),
                                sizeof(unsigned int) * width);
         frameBuffer.insertSampleCountSlice (countslice);
+        std::vector<void*> pointerbuf;
+        deepdata.get_pointers (pointerbuf);
         for (int c = 0;  c < nchans;  ++c) {
-            size_t chanbytes = m_spec.channelformat(c).size();
             Imf::DeepSlice slice (m_pixeltype[c],
-                                  (char *)(&deepdata.pointers[c]
+                                  (char *)(&pointerbuf[c]
                                            - xbegin*nchans
                                            - ybegin*width*nchans),
                                   sizeof(void*) * nchans, // xstride of pointer array
                                   sizeof(void*) * nchans*width, // ystride of pointer array
-                                  chanbytes); // stride of data sample
+                                  deepdata.samplesize()); // stride of data sample
             frameBuffer.insert (m_spec.channelnames[c].c_str(), slice);
         }
         m_deep_tiled_output_part->setFrameBuffer (frameBuffer);
