@@ -103,6 +103,10 @@ OIIO_EXPORT ImageInput *png_input_imageio_create () { return new PNGInput; }
 
 OIIO_EXPORT int png_imageio_version = OIIO_PLUGIN_VERSION;
 
+OIIO_EXPORT const char* png_imageio_library_version () {
+    return "libpng " PNG_LIBPNG_VER_STRING;
+}
+
 OIIO_EXPORT const char * png_input_extensions[] = {
     "png", NULL
 };
