@@ -328,8 +328,25 @@ BOOST_PYTHON_FUNCTION_OVERLOADS(ImageBuf_get_pixels_bt_overloads,
                                 ImageBuf_get_pixels_bt, 2, 3)
 
 
+
+void
+ImageBuf_set_deep_value (ImageBuf &buf, int x, int y, int z,
+                         int c, int s, float value)
+{
+    buf.set_deep_value (x, y, z, c, s, value);
+}
+
+void
+ImageBuf_set_deep_value_uint (ImageBuf &buf, int x, int y, int z,
+                         int c, int s, uint32_t value)
+{
+    buf.set_deep_value (x, y, z, c, s, value);
+}
+
+
+
 bool
-ImageBuf_set_pixels_tuple (ImageBuf &buf, ROI roi, tuple data)
+ImageBuf_set_pixels_tuple (ImageBuf &buf, ROI roi, const tuple& data)
 {
     if (! roi.defined())
         roi = buf.roi();
@@ -347,8 +364,13 @@ ImageBuf_set_pixels_tuple (ImageBuf &buf, ROI roi, tuple data)
 
 
 bool
-ImageBuf_set_pixels_array (ImageBuf &buf, ROI roi, numeric::array data)
+ImageBuf_set_pixels_array (ImageBuf &buf, ROI roi, const object& data)
 {
+    // If it's a tuple, we handle that with the other function
+    extract<tuple> tup (data);
+    if (tup.check())
+        return ImageBuf_set_pixels_tuple (buf, roi, tup());
+
     if (! roi.defined())
         roi = buf.roi();
     roi.chend = std::min (roi.chend, buf.nchannels()+1);
@@ -356,13 +378,15 @@ ImageBuf_set_pixels_array (ImageBuf &buf, ROI roi, numeric::array data)
     if (size == 0)
         return true;   // done
 
-    TypeDesc type;
-    size_t pylen = 0;
-    const void *addr = python_array_address (data, type, pylen);
-    if (!addr || size > pylen)
+    TypeDesc elementtype;
+    size_t numelements;
+    const void* addr = python_array_address (data, elementtype, numelements);
+    if (!addr || size > numelements)
         return false;   // Not enough data to fill our ROI
-
-    buf.set_pixels (roi, type, addr);
+    std::vector<float> vals (numelements);
+    convert_types (elementtype, addr, TypeDesc::TypeFloat, vals.data(),
+                   int(numelements));
+    buf.set_pixels (roi, TypeDesc::TypeFloat, &vals[0]);
     return true;
 }
 
@@ -376,6 +400,13 @@ ImageBuf_deepdataref (ImageBuf *ib)
 
 
 
+static void
+ImageBuf_deep_alloc_dummy ()
+{
+}
+
+
+
 void declare_imagebuf()
 {
     enum_<ImageBuf::WrapMode>("WrapMode")
@@ -392,9 +423,13 @@ void declare_imagebuf()
         .def(init<const ImageSpec&>())
 
         .def("clear", &ImageBuf::clear)
-        .def("reset", &ImageBuf_reset_name)
-        .def("reset", &ImageBuf_reset_name2)
-        .def("reset", &ImageBuf_reset_name_config)
+        .def("reset", &ImageBuf_reset_name,
+             (arg("name")))
+        .def("reset", &ImageBuf_reset_name2,
+             (arg("name"), arg("subimage")=0, arg("miplevel")=0))
+        .def("reset", &ImageBuf_reset_name_config,
+             (arg("name"), arg("subimage")=0, arg("miplevel")=0,
+              arg("config")=ImageSpec()))
         .def("reset", &ImageBuf_reset_spec)
         .add_property ("initialized", &ImageBuf::initialized)
         .def("init_spec", &ImageBuf::init_spec)
@@ -456,6 +491,8 @@ void declare_imagebuf()
         .add_property("has_error",   &ImageBuf::has_error)
         .def("geterror",    &ImageBuf::geterror)
 
+        .def("pixelindex", &ImageBuf::pixelindex,
+             (arg("x"), arg("y"), arg("z"), arg("check_range")=false))
         .def("copy_metadata", &ImageBuf::copy_metadata)
         .def("copy_pixels", &ImageBuf::copy_pixels)
         .def("copy",  &ImageBuf_copy,
@@ -489,12 +526,23 @@ void declare_imagebuf()
         .add_property("deep", &ImageBuf::deep)
         .def("deep_samples", &ImageBuf::deep_samples,
              (arg("x"), arg("y"), arg("z")=0))
-        .def("set_deep_samples", &ImageBuf::set_deep_samples)
-        .def("deep_value", &ImageBuf::deep_value)
-        .def("deep_value_uint", &ImageBuf::deep_value_uint)
-        .def("set_deep_value", &ImageBuf::set_deep_value)
-        .def("set_deep_value_uint", &ImageBuf::set_deep_value_uint)
-        .def("deep_alloc", &ImageBuf::deep_alloc)
+        .def("set_deep_samples", &ImageBuf::set_deep_samples,
+             (arg("x"), arg("y"), arg("z")=0, arg("nsamples")=1))
+        .def("deep_insert_samples", &ImageBuf::deep_insert_samples,
+             (arg("x"), arg("y"), arg("z")=0, arg("samplepos"), arg("nsamples")=1))
+        .def("deep_erase_samples", &ImageBuf::deep_erase_samples,
+             (arg("x"), arg("y"), arg("z")=0, arg("samplepos"), arg("nsamples")=1))
+        .def("deep_value", &ImageBuf::deep_value,
+             (arg("x"), arg("y"), arg("z")=0, arg("channel"), arg("sample")))
+        .def("deep_value_uint", &ImageBuf::deep_value_uint,
+             (arg("x"), arg("y"), arg("z")=0, arg("channel"), arg("sample")))
+        .def("set_deep_value", &ImageBuf_set_deep_value,
+             (arg("x"), arg("y"), arg("z")=0, arg("channel"),
+              arg("sample"), arg("value")=0.0f))
+        .def("set_deep_value_uint", &ImageBuf_set_deep_value_uint,
+             (arg("x"), arg("y"), arg("z")=0, arg("channel"),
+              arg("sample"), arg("value")=0))
+        .def("deep_alloc", &ImageBuf_deep_alloc_dummy)  // DEPRECATED(1.7)
         .def("deepdata", &ImageBuf_deepdataref,
              return_value_policy<reference_existing_object>())
 
