@@ -233,6 +233,26 @@ IBA_flatten (ImageBuf &dst, const ImageBuf &src, ROI roi, int nthreads)
 
 
 
+bool
+IBA_deep_merge (ImageBuf &dst, const ImageBuf &A, const ImageBuf &B,
+                 bool occlusion_cull, ROI roi, int nthreads)
+{
+    ScopedGILRelease gil;
+    return ImageBufAlgo::deep_merge (dst, A, B, occlusion_cull, roi, nthreads);
+}
+
+
+
+bool
+IBA_copy (ImageBuf &dst, const ImageBuf &src, TypeDesc::BASETYPE convert,
+          ROI roi, int nthreads)
+{
+    ScopedGILRelease gil;
+    return ImageBufAlgo::copy (dst, src, convert, roi, nthreads);
+}
+
+
+
 bool
 IBA_crop (ImageBuf &dst, const ImageBuf &src, ROI roi, int nthreads)
 {
@@ -307,15 +327,6 @@ IBA_flop (ImageBuf &dst, const ImageBuf &src, ROI roi, int nthreads)
 
 
 
-bool
-IBA_flipflop (ImageBuf &dst, const ImageBuf &src, ROI roi, int nthreads)
-{
-    ScopedGILRelease gil;
-    return ImageBufAlgo::flipflop (dst, src, roi, nthreads);
-}
-
-
-
 bool
 IBA_reorient (ImageBuf &dst, const ImageBuf &src, int nthreads)
 {
@@ -705,6 +716,15 @@ bool IBA_unpremult (ImageBuf &dst, const ImageBuf &src,
 
 
 
+bool IBA_computePixelStats (const ImageBuf &src, ImageBufAlgo::PixelStats &stats,
+                            ROI roi, int nthreads)
+{
+    ScopedGILRelease gil;
+    return ImageBufAlgo::computePixelStats (stats, src, roi, nthreads);
+}
+
+
+
 bool IBA_compare (const ImageBuf &A, const ImageBuf &B,
                   float failthresh, float warnthresh,
                   ImageBufAlgo::CompareResults &result,
@@ -853,6 +873,35 @@ IBA_median_filter (ImageBuf &dst, const ImageBuf &src,
 
 
 
+bool
+IBA_dilate (ImageBuf &dst, const ImageBuf &src,
+            int width, int height, ROI roi, int nthreads)
+{
+    ScopedGILRelease gil;
+    return ImageBufAlgo::dilate (dst, src, width, height, roi, nthreads);
+}
+
+
+
+bool
+IBA_erode (ImageBuf &dst, const ImageBuf &src,
+           int width, int height, ROI roi, int nthreads)
+{
+    ScopedGILRelease gil;
+    return ImageBufAlgo::erode (dst, src, width, height, roi, nthreads);
+}
+
+
+
+bool
+IBA_laplacian (ImageBuf &dst, const ImageBuf &src, ROI roi, int nthreads)
+{
+    ScopedGILRelease gil;
+    return ImageBufAlgo::laplacian (dst, src, roi, nthreads);
+}
+
+
+
 bool
 IBA_fft (ImageBuf &dst, const ImageBuf &src, ROI roi, int nthreads)
 {
@@ -926,8 +975,8 @@ IBA_colorconvert (ImageBuf &dst, const ImageBuf &src,
                   ROI roi = ROI::All(), int nthreads = 0)
 {
     ScopedGILRelease gil;
-    return ImageBufAlgo::colorconvert (dst, src, from.c_str(), to.c_str(),
-                                       unpremult, roi, nthreads);
+    return ImageBufAlgo::colorconvert (dst, src, from, to,
+                                       unpremult, NULL, roi, nthreads);
 }
 
 
@@ -935,13 +984,17 @@ IBA_colorconvert (ImageBuf &dst, const ImageBuf &src,
 bool
 IBA_colorconvert_colorconfig (ImageBuf &dst, const ImageBuf &src,
                   const std::string &from, const std::string &to,
-                  bool unpremult = false, const std::string &colorconfig="",
+                  bool unpremult = false,
+                  const std::string &context_key="",
+                  const std::string &context_value="",
+                  const std::string &colorconfig="",
                   ROI roi = ROI::All(), int nthreads = 0)
 {
     ColorConfig config (colorconfig);
     ScopedGILRelease gil;
-    return ImageBufAlgo::colorconvert (dst, src, from.c_str(), to.c_str(),
-                                       unpremult, &config, roi, nthreads);
+    return ImageBufAlgo::colorconvert (dst, src, from, to, unpremult,
+                                       context_key, context_value,
+                                       &config, roi, nthreads);
 }
 
 
@@ -954,10 +1007,9 @@ IBA_ociolook (ImageBuf &dst, const ImageBuf &src, const std::string &looks,
               ROI roi = ROI::All(), int nthreads = 0)
 {
     ScopedGILRelease gil;
-    return ImageBufAlgo::ociolook (dst, src, looks.c_str(),
-                                   from.c_str(), to.c_str(),
+    return ImageBufAlgo::ociolook (dst, src, looks, from, to,
                                    inverse, unpremult,
-                                   context_key.c_str(), context_value.c_str(),
+                                   context_key, context_value, NULL,
                                    roi, nthreads);
 }
 
@@ -973,10 +1025,9 @@ IBA_ociolook_colorconfig (ImageBuf &dst, const ImageBuf &src, const std::string
 {
     ColorConfig config (colorconfig);
     ScopedGILRelease gil;
-    return ImageBufAlgo::ociolook (dst, src, looks.c_str(),
-                                   from.c_str(), to.c_str(),
+    return ImageBufAlgo::ociolook (dst, src, looks, from, to,
                                    inverse, unpremult,
-                                   context_key.c_str(), context_value.c_str(),
+                                   context_key, context_value,
                                    &config, roi, nthreads);
 }
 
@@ -1000,7 +1051,7 @@ IBA_ociodisplay (ImageBuf &dst, const ImageBuf &src,
                                       from == object() ? NULL : from_str.c_str(),
                                       looks == object() ? NULL : looks_str.c_str(),
                                       unpremult,
-                                      context_key.c_str(), context_value.c_str(),
+                                      context_key, context_value, NULL,
                                       roi, nthreads);
 }
 
@@ -1025,8 +1076,7 @@ IBA_ociodisplay_colorconfig (ImageBuf &dst, const ImageBuf &src,
     return ImageBufAlgo::ociodisplay (dst, src, display.c_str(), view.c_str(),
                                       from == object() ? NULL : from_str.c_str(),
                                       looks == object() ? NULL : looks_str.c_str(),
-                                      unpremult,
-                                      context_key.c_str(), context_value.c_str(),
+                                      unpremult, context_key, context_value,
                                       &config, roi, nthreads);
 }
 
@@ -1118,6 +1168,42 @@ IBA_fixNonFinite (ImageBuf &dst, const ImageBuf &src,
 
 
 
+bool
+IBA_render_point (ImageBuf &dst, int x, int y, tuple color_ = tuple())
+{
+    std::vector<float> color;
+    py_to_stdvector (color, color_);
+    color.resize (dst.nchannels(), 1.0f);
+    ScopedGILRelease gil;
+    return ImageBufAlgo::render_point (dst, x, y, color);
+}
+
+
+bool
+IBA_render_line (ImageBuf &dst, int x1, int y1, int x2, int y2,
+                 tuple color_ = tuple(), bool skip_first_point=false)
+{
+    std::vector<float> color;
+    py_to_stdvector (color, color_);
+    color.resize (dst.nchannels(), 1.0f);
+    ScopedGILRelease gil;
+    return ImageBufAlgo::render_line (dst, x1, y1, x2, y2,
+                                      color, skip_first_point);
+}
+
+
+bool
+IBA_render_box (ImageBuf &dst, int x1, int y1, int x2, int y2,
+                 tuple color_ = tuple(), bool fill=false)
+{
+    std::vector<float> color;
+    py_to_stdvector (color, color_);
+    color.resize (dst.nchannels(), 1.0f);
+    ScopedGILRelease gil;
+    return ImageBufAlgo::render_box (dst, x1, y1, x2, y2, color, fill);
+}
+
+
 bool
 IBA_render_text (ImageBuf &dst, int x, int y,
                  const std::string &text,
@@ -1168,6 +1254,108 @@ IBA_make_texture_filename (ImageBufAlgo::MakeTextureMode mode,
 
 
 
+#if PY_MAJOR_VERSION >= 3
+# define PYLONG(x) PyLong_FromLong((long)x)
+#else
+# define PYLONG(x) PyInt_FromLong((long)x)
+#endif
+
+
+static object
+PixelStats_get_min(const ImageBufAlgo::PixelStats& stats)
+{
+    size_t size = stats.min.size();
+    PyObject* result = PyTuple_New(size);
+    for (size_t i = 0; i < size; ++i)
+        PyTuple_SetItem(result, i, PyFloat_FromDouble(stats.min[i]));
+    return object(handle<>(result));
+}
+
+static object
+PixelStats_get_max(const ImageBufAlgo::PixelStats& stats)
+{
+    size_t size = stats.min.size();
+    PyObject* result = PyTuple_New(size);
+    for (size_t i = 0; i < size; ++i)
+        PyTuple_SetItem(result, i, PyFloat_FromDouble(stats.max[i]));
+    return object(handle<>(result));
+}
+
+static object
+PixelStats_get_avg(const ImageBufAlgo::PixelStats& stats)
+{
+    size_t size = stats.min.size();
+    PyObject* result = PyTuple_New(size);
+    for (size_t i = 0; i < size; ++i)
+        PyTuple_SetItem(result, i, PyFloat_FromDouble(stats.avg[i]));
+    return object(handle<>(result));
+}
+
+static object
+PixelStats_get_stddev(const ImageBufAlgo::PixelStats& stats)
+{
+    size_t size = stats.min.size();
+    PyObject* result = PyTuple_New(size);
+    for (size_t i = 0; i < size; ++i)
+        PyTuple_SetItem(result, i, PyFloat_FromDouble(stats.stddev[i]));
+    return object(handle<>(result));
+}
+
+static object
+PixelStats_get_nancount(const ImageBufAlgo::PixelStats& stats)
+{
+    size_t size = stats.min.size();
+    PyObject* result = PyTuple_New(size);
+    for (size_t i = 0; i < size; ++i)
+        PyTuple_SetItem(result, i, PYLONG((long)stats.nancount[i]));
+    return object(handle<>(result));
+}
+
+static object
+PixelStats_get_infcount(const ImageBufAlgo::PixelStats& stats)
+{
+    size_t size = stats.min.size();
+    PyObject* result = PyTuple_New(size);
+    for (size_t i = 0; i < size; ++i)
+        PyTuple_SetItem(result, i, PYLONG((long)stats.infcount[i]));
+    return object(handle<>(result));
+}
+
+static object
+PixelStats_get_finitecount(const ImageBufAlgo::PixelStats& stats)
+{
+    size_t size = stats.min.size();
+    PyObject* result = PyTuple_New(size);
+    for (size_t i = 0; i < size; ++i)
+        PyTuple_SetItem(result, i, PYLONG((long)stats.finitecount[i]));
+    return object(handle<>(result));
+}
+
+static object
+PixelStats_get_sum(const ImageBufAlgo::PixelStats& stats)
+{
+    size_t size = stats.min.size();
+    PyObject* result = PyTuple_New(size);
+    for (size_t i = 0; i < size; ++i)
+        PyTuple_SetItem(result, i, PyFloat_FromDouble(stats.sum[i]));
+    return object(handle<>(result));
+}
+
+static object
+PixelStats_get_sum2(const ImageBufAlgo::PixelStats& stats)
+{
+    size_t size = stats.min.size();
+    PyObject* result = PyTuple_New(size);
+    for (size_t i = 0; i < size; ++i)
+        PyTuple_SetItem(result, i, PyFloat_FromDouble(stats.sum2[i]));
+    return object(handle<>(result));
+}
+
+
+
+
+
+
 void declare_imagebufalgo()
 {
     enum_<ImageBufAlgo::NonFiniteFixMode>("NonFiniteFixMode")
@@ -1186,6 +1374,18 @@ void declare_imagebufalgo()
         .export_values()
     ;
 
+    class_<ImageBufAlgo::PixelStats>("PixelStats")
+        .add_property("min", &PixelStats_get_min)
+        .add_property("max", &PixelStats_get_max)
+        .add_property("avg", &PixelStats_get_avg)
+        .add_property("stddev", &PixelStats_get_stddev)
+        .add_property("nancount", &PixelStats_get_nancount)
+        .add_property("infcount", &PixelStats_get_infcount)
+        .add_property("finitecount", &PixelStats_get_finitecount)
+        .add_property("sum", &PixelStats_get_sum)
+        .add_property("sum2", &PixelStats_get_sum2)
+    ;
+
     class_<ImageBufAlgo::CompareResults>("CompareResults")
         .def_readwrite("meanerror", &ImageBufAlgo::CompareResults::meanerror)
         .def_readwrite("rms_error", &ImageBufAlgo::CompareResults::rms_error)
@@ -1250,6 +1450,16 @@ void declare_imagebufalgo()
               arg("roi")=ROI::All(), arg("nthreads")=0))
         .staticmethod("flatten")
 
+        .def("deep_merge", IBA_deep_merge,
+             (arg("dst"), arg("A"), arg("B"), arg("occlusion_cull")=true,
+              arg("roi")=ROI::All(), arg("nthreads")=0))
+        .staticmethod("deep_merge")
+
+        .def("copy", IBA_copy,
+             (arg("dst"), arg("src"), arg("convert")=TypeDesc::UNKNOWN,
+              arg("roi")=ROI::All(), arg("nthreads")=0))
+        .staticmethod("copy")
+
         .def("crop", IBA_crop,
              (arg("dst"), arg("src"),
               arg("roi")=ROI::All(), arg("nthreads")=0))
@@ -1291,11 +1501,6 @@ void declare_imagebufalgo()
               arg("roi")=ROI::All(), arg("nthreads")=0))
         .staticmethod("flop")
 
-        .def("flipflop", IBA_rotate180,
-             (arg("dst"), arg("src"),
-              arg("roi")=ROI::All(), arg("nthreads")=0))
-        .staticmethod("flipflop")
-
         .def("reorient", IBA_reorient,
              (arg("dst"), arg("src"), arg("nthreads")=0))
         .staticmethod("reorient")
@@ -1442,8 +1647,10 @@ void declare_imagebufalgo()
               arg("roi")=ROI::All(), arg("nthreads")=0))
         .def("colorconvert", &IBA_colorconvert_colorconfig,
              (arg("dst"), arg("src"),
-              arg("from"), arg("to"), 
-              arg("unpremult")=false, arg("colorconfig")="",
+              arg("from"), arg("to"),
+              arg("unpremult")=false,
+              arg("context_key")="", arg("context_value")="",
+              arg("colorconfig")="",
               arg("roi")=ROI::All(), arg("nthreads")=0))
         .staticmethod("colorconvert")
 
@@ -1490,7 +1697,10 @@ void declare_imagebufalgo()
               arg("roi")=ROI::All(), arg("nthreads")=0))
         .staticmethod("ociofiletransform")
 
-        // computePixelStats, 
+        .def("computePixelStats", &IBA_computePixelStats,
+             (arg("src"), arg("stats"),
+              arg("roi")=ROI::All(), arg("nthreads")=0))
+        .staticmethod("computePixelStats")
 
         .def("compare", &IBA_compare,
              (arg("A"), arg("B"), arg("failthresh"), arg("warnthresh"),
@@ -1581,6 +1791,23 @@ void declare_imagebufalgo()
               arg("roi")=ROI::All(), arg("nthreads")=0))
         .staticmethod("median_filter")
 
+        .def("dilate", &IBA_dilate,
+             (arg("dst"), arg("src"),
+              arg("width")=3, arg("height")=-1,
+              arg("roi")=ROI::All(), arg("nthreads")=0))
+        .staticmethod("dilate")
+
+        .def("erode", &IBA_erode,
+             (arg("dst"), arg("src"),
+              arg("width")=3, arg("height")=-1,
+              arg("roi")=ROI::All(), arg("nthreads")=0))
+        .staticmethod("erode")
+
+        .def("laplacian", &IBA_laplacian,
+             (arg("dst"), arg("src"),
+              arg("roi")=ROI::All(), arg("nthreads")=0))
+        .staticmethod("laplacian")
+
         .def("fft", &IBA_fft,
              (arg("dst"), arg("src"),
               arg("roi")=ROI::All(), arg("nthreads")=0))
@@ -1627,6 +1854,20 @@ void declare_imagebufalgo()
               arg("roi")=ROI::All(), arg("nthreads")=0))
         .staticmethod("zover")
 
+        .def("render_point", &IBA_render_point,
+             (arg("dst"), arg("x"), arg("y"), arg("color")=tuple()))
+        .staticmethod("render_point")
+
+        .def("render_line", &IBA_render_line,
+             (arg("dst"), arg("x1"), arg("y1"), arg("x2"), arg("y2"),
+              arg("color")=tuple(), arg("skip_first_point")=false))
+        .staticmethod("render_line")
+
+        .def("render_box", &IBA_render_box,
+             (arg("dst"), arg("x1"), arg("y1"), arg("x2"), arg("y2"),
+              arg("color")=tuple(), arg("fill")=false))
+        .staticmethod("render_box")
+
         .def("render_text", &IBA_render_text,
              (arg("dst"), arg("x"), arg("y"), arg("text"),
               arg("fontsize")=16, arg("fontname")="",
