@@ -165,6 +165,16 @@ namespace RLA_pvt {
         CT_FLOAT = 4
     };
 
+    inline rla_channel_type rla_type (TypeDesc t) {
+        if (t == TypeDesc::UINT16)
+            return CT_WORD;
+        if (t == TypeDesc::UINT32)
+            return CT_DWORD;
+        if (t == TypeDesc::FLOAT)
+            return CT_FLOAT;
+        return CT_BYTE;
+    }
+
 }  // namespace RLA_pvt
 
 
