@@ -165,6 +165,8 @@ RLAOutput::supports (string_view feature) const
         return true;
     if (feature == "nchannels")
         return true;
+    if (feature == "channelformats")
+        return true;
     // Support nothing else nonstandard
     return false;
 }
@@ -238,24 +240,27 @@ RLAOutput::open (const std::string &name, const ImageSpec &userspec,
         int streak;
         // accomodate first 3 channels of the same type as colour ones
         for (streak = 1; streak <= 3 && remaining > 0; ++streak, --remaining)
-            if (m_spec.channelformats[streak] != m_spec.channelformats[0])
+            if (m_spec.channelformats[streak] != m_spec.channelformats[0] ||
+                m_spec.alpha_channel == streak || m_spec.z_channel == streak)
                 break;
-        m_rla.ColorChannelType = m_spec.channelformats[0] == TypeDesc::FLOAT
-            ? CT_FLOAT : CT_BYTE;
+        m_rla.ColorChannelType = rla_type(m_spec.channelformats[0]);
         int bits = m_spec.get_int_attribute ("oiio:BitsPerSample", 0);
         m_rla.NumOfChannelBits = bits ? bits : m_spec.channelformats[0].size () * 8;
         // limit to 3 in case the loop went further
         m_rla.NumOfColorChannels = std::min (streak, 3);
-        // if we have anything left, treat it as alpha
-        if (remaining) {
+        // if we have anything left and it looks like alpha, treat it as alpha
+        if (remaining && m_spec.z_channel != m_rla.NumOfColorChannels) {
             for (streak = 1; remaining > 0; ++streak, --remaining)
                 if (m_spec.channelformats[m_rla.NumOfColorChannels + streak]
                     != m_spec.channelformats[m_rla.NumOfColorChannels])
                     break;
-            m_rla.MatteChannelType = m_spec.channelformats[m_rla.NumOfColorChannels]
-                == TypeDesc::FLOAT ? CT_FLOAT : CT_BYTE;
+            m_rla.MatteChannelType = rla_type(m_spec.channelformats[m_rla.NumOfColorChannels]);
             m_rla.NumOfMatteBits = bits ? bits : m_spec.channelformats[m_rla.NumOfColorChannels].size () * 8;
             m_rla.NumOfMatteChannels = streak;
+        } else {
+            m_rla.MatteChannelType = CT_BYTE;
+            m_rla.NumOfMatteBits = 8;
+            m_rla.NumOfMatteChannels = 0;
         }
         // and if there's something more left, put it in auxiliary
         if (remaining) {
@@ -265,16 +270,15 @@ RLAOutput::open (const std::string &name, const ImageSpec &userspec,
                     != m_spec.channelformats[m_rla.NumOfColorChannels
                         + m_rla.NumOfMatteChannels])
                     break;
-            m_rla.MatteChannelType = m_spec.channelformats[m_rla.NumOfColorChannels
-                    + m_rla.NumOfMatteChannels]
-                == TypeDesc::FLOAT ? CT_FLOAT : CT_BYTE;
+            m_rla.AuxChannelType = rla_type (m_spec.channelformats[m_rla.NumOfColorChannels
+                                                                     + m_rla.NumOfMatteChannels]);
             m_rla.NumOfAuxBits = m_spec.channelformats[m_rla.NumOfColorChannels
                 + m_rla.NumOfMatteChannels].size () * 8;
             m_rla.NumOfAuxChannels = streak;
         }
     } else {
         m_rla.ColorChannelType = m_rla.MatteChannelType = m_rla.AuxChannelType =
-            m_spec.format == TypeDesc::FLOAT ? CT_FLOAT : CT_BYTE;
+            rla_type(m_spec.format);
         int bits = m_spec.get_int_attribute ("oiio:BitsPerSample", 0);
         if (bits) {
             m_rla.NumOfChannelBits = bits;
@@ -295,13 +299,16 @@ RLAOutput::open (const std::string &name, const ImageSpec &userspec,
             --remaining;
         }
         // if there's at least 1 more channel, it's alpha
-        if (remaining-- > 0)
+        if (remaining  && m_spec.z_channel != m_rla.NumOfColorChannels) {
+            --remaining;
             ++m_rla.NumOfMatteChannels;
+        }
         // anything left is auxiliary
         if (remaining > 0)
             m_rla.NumOfAuxChannels = remaining;
     }
-    
+    // std::cout << "color chans " << m_rla.NumOfColorChannels << " a "
+    //           << m_rla.NumOfMatteChannels << " z " << m_rla.NumOfAuxChannels << "\n";
     m_rla.Revision = 0xFFFE;
     
     std::string s = m_spec.get_string_attribute ("oiio:ColorSpace", "Unknown");
@@ -558,6 +565,7 @@ RLAOutput::write_scanline (int y, int z, TypeDesc format,
     const void *origdata = data;
     data = to_native_scanline (format, data, xstride, m_scratch,
                                m_dither, y, z);
+    ASSERT (data != NULL);
     if (data == origdata) {
         m_scratch.assign ((unsigned char *)data,
                           (unsigned char *)data+m_spec.scanline_bytes());
