@@ -58,6 +58,15 @@ SocketOutput::SocketOutput()
 
 
 
+int
+SocketOutput::supports (string_view feature) const
+{
+    return (feature == "alpha" ||
+            feature == "nchannels");
+}
+
+
+
 bool
 SocketOutput::open (const std::string &name, const ImageSpec &newspec,
                     OpenMode mode)
@@ -85,7 +94,10 @@ SocketOutput::write_scanline (int y, int z, TypeDesc format,
     try {
         socket_pvt::socket_write (socket, format, data, m_spec.scanline_bytes ());
     } catch (boost::system::system_error &err) {
-        error ("Error while reading: %s", err.what ());
+        error ("Error while writing: %s", err.what ());
+        return false;
+    } catch (...) {
+        error ("Error while writing: unknown exception");
         return false;
     }
 
@@ -106,7 +118,10 @@ SocketOutput::write_tile (int x, int y, int z,
     try {
         socket_pvt::socket_write (socket, format, data, m_spec.tile_bytes ());
     } catch (boost::system::system_error &err) {
-        error ("Error while reading: %s", err.what ());
+        error ("Error while writing: %s", err.what ());
+        return false;
+    } catch (...) {
+        error ("Error while writing: unknown exception");
         return false;
     }
 
@@ -143,7 +158,10 @@ SocketOutput::send_spec_to_server(const ImageSpec& spec)
                 sizeof (boost::uint32_t)));
         boost::asio::write (socket, buffer (spec_xml.c_str (), spec_xml.length ()));
     } catch (boost::system::system_error &err) {
-        error ("Error while writing: %s", err.what ());
+        error ("Error while send_spec_to_server: %s", err.what ());
+        return false;
+    } catch (...) {
+        error ("Error while send_spec_to_server: unknown exception");
         return false;
     }
 
@@ -184,6 +202,9 @@ SocketOutput::connect_to_server (const std::string &name)
     } catch (boost::system::system_error &err) {
         error ("Error while connecting: %s", err.what ());
         return false;
+    } catch (...) {
+        error ("Error while connecting: unknown exception");
+        return false;
     }
 
     return true;
