@@ -50,9 +50,8 @@ public:
     TGAOutput ();
     virtual ~TGAOutput ();
     virtual const char * format_name (void) const { return "targa"; }
-    virtual bool supports (const std::string &feature) const {
-        // Support nothing nonstandard
-        return false;
+    virtual int supports (string_view feature) const {
+        return (feature == "alpha");
     }
     virtual bool open (const std::string &name, const ImageSpec &spec,
                        OpenMode mode=Create);
