@@ -95,10 +95,11 @@ static int interpmode = TextureOpt::InterpSmartBicubic;
 static float missing[4] = {-1, 0, 0, 1};
 static float fill = -1;  // -1 signifies unset
 static float scalefactor = 1.0f;
-static Imath::V3f offset (0,0,0);
+static Imath::V3f texoffset (0,0,0);
 static bool nountiled = false;
 static bool nounmipped = false;
 static bool gray_to_rgb = false;
+static bool flip_t = false;
 static bool resetstats = false;
 static bool testhash = false;
 static bool wedge = false;
@@ -107,6 +108,7 @@ static int testicwrite = 0;
 static bool test_derivs = false;
 static bool test_statquery = false;
 static Imath::M33f xform;
+static mutex error_mutex;
 void *dummyptr;
 
 typedef void (*Mapping2D)(int,int,float&,float&,float&,float&,float&,float&);
@@ -169,7 +171,7 @@ getargs (int argc, const char *argv[])
                   "--ctr", &test_construction, "Test TextureOpt construction time",
                   "--gettexels", &test_gettexels, "Test TextureSystem::get_texels",
                   "--getimagespec", &test_getimagespec, "Test TextureSystem::get_imagespec",
-                  "--offset %f %f %f", &offset[0], &offset[1], &offset[2], "Offset texture coordinates",
+                  "--offset %f %f %f", &texoffset[0], &texoffset[1], &texoffset[2], "Offset texture coordinates",
                   "--scalest %f %f", &sscale, &tscale, "Scale texture lookups (s, t)",
                   "--cachesize %f", &cachesize, "Set cache size, in MB",
                   "--nodedup %!", &dedup, "Turn off de-duplication",
@@ -178,6 +180,7 @@ getargs (int argc, const char *argv[])
                   "--nountiled", &nountiled, "Reject untiled images",
                   "--nounmipped", &nounmipped, "Reject unmipped images",
                   "--graytorgb", &gray_to_rgb, "Convert gratscale textures to RGB",
+                  "--flipt", &flip_t, "Flip direction of t coordinate",
                   "--derivs", &test_derivs, "Test returning derivatives of texture lookups",
                   "--resetstats", &resetstats, "Print and reset statistics on each iteration",
                   "--testhash", &testhash, "Test the tile hashing function",
@@ -346,7 +349,7 @@ warp_coord (float x, float y)
     Imath::V2f coord = warp (x/output_xres, y/output_yres, xform);
     coord.x *= sscale;
     coord.y *= tscale;
-    coord += Imath::V2f(offset.x, offset.y);
+    coord += Imath::V2f(texoffset.x, texoffset.y);
     return coord;
 }
 
@@ -357,8 +360,8 @@ static void
 map_default (int x, int y, float &s, float &t,
              float &dsdx, float &dtdx, float &dsdy, float &dtdy)
 {
-    s = float(x+0.5f)/output_xres * sscale + offset[0];
-    t = float(y+0.5f)/output_yres * tscale + offset[1];
+    s = float(x+0.5f)/output_xres * sscale + texoffset[0];
+    t = float(y+0.5f)/output_yres * tscale + texoffset[1];
     dsdx = 1.0f/output_xres * sscale;
     dtdx = 0.0f;
     dsdy = 0.0f;
@@ -454,7 +457,7 @@ map_default_3D (int x, int y, Imath::V3f &P,
     P[0] = (float)(x+0.5f)/output_xres * sscale;
     P[1] = (float)(y+0.5f)/output_yres * tscale;
     P[2] = 0.5f * sscale;
-    P += offset;
+    P += texoffset;
     dPdx[0] = 1.0f/output_xres * sscale;
     dPdx[1] = 0;
     dPdx[2] = 0;
@@ -475,19 +478,19 @@ map_warp_3D (int x, int y, Imath::V3f &P,
                              0.5, xform);
     coord.x *= sscale;
     coord.y *= tscale;
-    coord += offset;
+    coord += texoffset;
     Imath::V3f coordx = warp ((float)(x+1)/output_xres,
                               (float)y/output_yres,
                               0.5, xform);
     coordx.x *= sscale;
     coordx.y *= tscale;
-    coordx += offset;
+    coordx += texoffset;
     Imath::V3f coordy = warp ((float)x/output_xres,
                               (float)(y+1)/output_yres,
                               0.5, xform);
     coordy.x *= sscale;
     coordy.y *= tscale;
-    coordy += offset;
+    coordy += texoffset;
     P = coord;
     dPdx = coordx - coord;
     dPdy = coordy - coord;
@@ -526,8 +529,10 @@ plain_tex_region (ImageBuf &image, ustring filename, Mapping2D mapping,
                                   nchannels, result, dresultds, dresultdt);
         if (! ok) {
             std::string e = texsys->geterror ();
-            if (! e.empty())
+            if (! e.empty()) {
+                lock_guard lock (error_mutex);
                 std::cerr << "ERROR: " << e << "\n";
+            }
         }
 
         // Save filtered pixels back to the image.
@@ -552,13 +557,13 @@ test_plain_texture (Mapping2D mapping)
     ImageSpec outspec (output_xres, output_yres, nchannels, TypeDesc::HALF);
     adjust_spec (outspec, dataformatname);
     ImageBuf image (outspec);
-    ImageBufAlgo::zero (image);
+    OIIO::ImageBufAlgo::zero (image);
     ImageBuf image_ds, image_dt;
     if (test_derivs) {
         image_ds.reset (outspec);
-        ImageBufAlgo::zero (image_ds);
+        OIIO::ImageBufAlgo::zero (image_ds);
         image_dt.reset (outspec);
-        ImageBufAlgo::zero (image_dt);
+        OIIO::ImageBufAlgo::zero (image_dt);
     }
 
     ustring filename = filenames[0];
@@ -571,7 +576,7 @@ test_plain_texture (Mapping2D mapping)
             std::cout << "iter " << iter << " file " << filename << "\n";
         }
 
-        ImageBufAlgo::parallel_image (OIIO::bind(plain_tex_region, OIIO::ref(image), filename, mapping,
+        OIIO::ImageBufAlgo::parallel_image (OIIO::bind(plain_tex_region, OIIO::ref(image), filename, mapping,
                                                   test_derivs ? &image_ds : NULL,
                                                   test_derivs ? &image_dt : NULL, _1),
                                       get_roi(image.spec()), nthreads);
@@ -624,8 +629,10 @@ tex3d_region (ImageBuf &image, ustring filename, Mapping3D mapping,
                                      result, dresultds, dresultdt, dresultdr);
         if (! ok) {
             std::string e = texsys->geterror ();
-            if (! e.empty())
+            if (! e.empty()) {
+                lock_guard lock (error_mutex);
                 std::cerr << "ERROR: " << e << "\n";
+            }
         }
 
         // Save filtered pixels back to the image.
@@ -646,14 +653,14 @@ test_texture3d (ustring filename, Mapping3D mapping)
     ImageSpec outspec (output_xres, output_yres, nchannels, TypeDesc::HALF);
     adjust_spec (outspec, dataformatname);
     ImageBuf image (outspec);
-    ImageBufAlgo::zero (image);
+    OIIO::ImageBufAlgo::zero (image);
 
     for (int iter = 0;  iter < iters;  ++iter) {
         // Trick: switch to second texture, if given, for second iteration
         if (iter && filenames.size() > 1)
             filename = filenames[1];
 
-        ImageBufAlgo::parallel_image (OIIO::bind(tex3d_region, OIIO::ref(image), filename, mapping, _1),
+        OIIO::ImageBufAlgo::parallel_image (OIIO::bind(tex3d_region, OIIO::ref(image), filename, mapping, _1),
                                       get_roi(image.spec()), nthreads);
     }
     
@@ -686,35 +693,42 @@ test_getimagespec_gettexels (ustring filename)
     if (! texsys->get_imagespec (filename, 0, spec)) {
         std::cerr << "Could not get spec for " << filename << "\n";
         std::string e = texsys->geterror ();
-        if (! e.empty())
+        if (! e.empty()) {
+            lock_guard lock (error_mutex);
             std::cerr << "ERROR: " << e << "\n";
+        }
         return;
     }
 
     if (! test_gettexels)
         return;
 
-    int w = spec.width / std::max(1,2<<miplevel);
-    int h = spec.height / std::max(1,2<<miplevel);
+    int w = std::min (spec.width, output_xres);
+    int h = std::min (spec.height, output_yres);
     int nchannels = nchannels_override ? nchannels_override : spec.nchannels;
     ImageSpec postagespec (w, h, nchannels, TypeDesc::FLOAT);
     ImageBuf buf (postagespec);
     TextureOpt opt;
     initialize_opt (opt, nchannels);
     std::vector<float> tmp (w*h*nchannels);
-    bool ok = texsys->get_texels (filename, opt, miplevel,
-                                  spec.x+w/2, spec.x+w/2+w,
-                                  spec.y+h/2, spec.y+h/2+h,
-                                  0, 1, 0, nchannels,
-                                  postagespec.format, &tmp[0]);
-    if (! ok)
-        std::cerr << texsys->geterror() << "\n";
+    int x = spec.x + spec.width/2 - w/2;
+    int y = spec.y + spec.height/2 - h/2;
+    for (int i = 0; i < iters; ++i) {
+        bool ok = texsys->get_texels (filename, opt, miplevel,
+                                      x, x+w, y, y+h, 0, 1, 0, nchannels,
+                                      postagespec.format, &tmp[0]);
+        if (! ok)
+            std::cerr << texsys->geterror() << "\n";
+    }
     for (int y = 0;  y < h;  ++y)
         for (int x = 0;  x < w;  ++x) {
-            imagesize_t offset = (y*w + x) * spec.nchannels;
-            buf.setpixel (x, y, &tmp[offset]);
+            imagesize_t texoffset = (y*w + x) * spec.nchannels;
+            buf.setpixel (x, y, &tmp[texoffset]);
         }
-    buf.write ("postage.exr");
+    TypeDesc fmt (dataformatname);
+    if (fmt != TypeDesc::UNKNOWN)
+        buf.set_write_format (fmt);
+    buf.write (output_filename);
 }
 
 
@@ -932,6 +946,7 @@ do_tex_thread_workout (int iterations, int mythread)
                                   result, dresultds, dresultdt);
         }
         if (! ok) {
+            lock_guard lock (error_mutex);
             std::cerr << "Unexpected error: " << texsys->geterror() << "\n";
             return;
         }
@@ -1060,9 +1075,12 @@ test_icwrite (int testicwrite)
                         tile[index+1] = float(yy)/spec.height;
                         tile[index+2] = (!(xx%10) || !(yy%10)) ? 1.0f : 0.0f;
                     }
-                bool ok = ic->add_tile (filename, 0, 0, tx, ty, 0, TypeDesc::FLOAT, &tile[0]);
-                if (! ok)
+                bool ok = ic->add_tile (filename, 0, 0, tx, ty, 0, 0, -1,
+                                        TypeDesc::FLOAT, &tile[0]);
+                if (! ok) {
+                    lock_guard lock (error_mutex);
                     std::cout << "ic->add_tile error: " << ic->geterror() << "\n";
+                }
                 ASSERT (ok);
             }
         }
@@ -1098,6 +1116,7 @@ main (int argc, const char *argv[])
     if (nounmipped)
         texsys->attribute ("accept_unmipped", 0);
     texsys->attribute ("gray_to_rgb", gray_to_rgb);
+    texsys->attribute ("flip_t", flip_t);
 
     if (test_construction) {
         Timer t;
@@ -1121,7 +1140,6 @@ main (int argc, const char *argv[])
     }
 
     if (test_getimagespec) {
-        Timer t;
         ImageSpec spec;
         for (int i = 0;  i < iters;  ++i) {
             texsys->get_imagespec (filenames[0], 0, spec);
@@ -1129,6 +1147,11 @@ main (int argc, const char *argv[])
         iters = 0;
     }
 
+    if (test_gettexels) {
+        test_getimagespec_gettexels (filenames[0]);
+        iters = 0;
+    }
+
     if (testhash) {
         test_hash ();
     }
@@ -1180,6 +1203,7 @@ main (int argc, const char *argv[])
         const char *texturetype = "Plain Texture";
         texsys->get_texture_info (filename, 0, ustring("texturetype"),
                                   TypeDesc::STRING, &texturetype);
+        Timer timer;
         if (! strcmp (texturetype, "Plain Texture")) {
             if (nowarp)
                 test_plain_texture (map_default);
@@ -1203,6 +1227,7 @@ main (int argc, const char *argv[])
             test_environment (filename);
         }
         test_getimagespec_gettexels (filename);
+        std::cout << "Time: " << Strutil::timeintervalformat (timer()) << "\n";
     }
 
     if (test_statquery) {
@@ -1217,16 +1242,23 @@ main (int argc, const char *argv[])
             int timesopened = 0;
             int64_t bytesread = 0;
             float iotime = 0.0f;
+            int64_t data_size = 0, file_size = 0;
             texsys->get_texture_info (all_filenames[i], 0, ustring("stat:timesopened"),
                                       TypeDesc::INT, &timesopened);
             texsys->get_texture_info (all_filenames[i], 0, ustring("stat:bytesread"),
                                       TypeDesc::INT64, &bytesread);
             texsys->get_texture_info (all_filenames[i], 0, ustring("stat:iotime"),
                                       TypeDesc::FLOAT, &iotime);
-            std::cout << Strutil::format ("  %d: %s  opens=%d, read=%s, time=%s\n",
+            texsys->get_texture_info (all_filenames[i], 0, ustring("stat:image_size"),
+                                      TypeDesc::INT64, &data_size);
+            texsys->get_texture_info (all_filenames[i], 0, ustring("stat:file_size"),
+                                      TypeDesc::INT64, &file_size);
+            std::cout << Strutil::format ("  %d: %s  opens=%d, read=%s, time=%s, data=%s, file=%s\n",
                                           i, all_filenames[i], timesopened,
                                           Strutil::memformat(bytesread),
-                                          Strutil::timeintervalformat(iotime,2));
+                                          Strutil::timeintervalformat(iotime,2),
+                                          Strutil::memformat(data_size),
+                                          Strutil::memformat(file_size));
         }
     }
 
