@@ -97,6 +97,7 @@ private:
     int m_photometric;
     unsigned int m_bitspersample;  ///< Of the *file*, not the client's view
     int m_outputchans;   // Number of channels for the output
+    bool m_convert_rgb_to_cmyk;
 
     // Initialize private members to pre-opened state
     void init (void) {
@@ -105,6 +106,7 @@ private:
         m_compression = COMPRESSION_ADOBE_DEFLATE;
         m_photometric = PHOTOMETRIC_RGB;
         m_outputchans = 0;
+        m_convert_rgb_to_cmyk = false;
     }
 
     // Convert planar contiguous to planar separate data format
@@ -115,6 +117,12 @@ private:
     bool put_parameter (const std::string &name, TypeDesc type,
                         const void *data);
     bool write_exif_data ();
+
+    // Make our best guess about whether the spec is describing data that
+    // is in true CMYK values.
+    bool source_is_cmyk (const ImageSpec &spec);
+    // Are we fairly certain that the spec is describing RGB values?
+    bool source_is_rgb (const ImageSpec &spec);
 };
 
 
@@ -127,6 +135,12 @@ OIIO_EXPORT ImageOutput *tiff_output_imageio_create () { return new TIFFOutput;
 
 OIIO_EXPORT int tiff_imageio_version = OIIO_PLUGIN_VERSION;
 
+OIIO_EXPORT const char* tiff_imageio_library_version () {
+    string_view v (TIFFGetVersion());
+    v = v.substr (0, v.find ('\n'));
+    return v.c_str();
+}
+
 OIIO_EXPORT const char * tiff_output_extensions[] = {
     "tiff", "tif", "tx", "env", "sm", "vsm", NULL
 };
@@ -345,18 +359,23 @@ TIFFOutput::open (const std::string &name, const ImageSpec &userspec,
         sampformat = SAMPLEFORMAT_UINT;
         break;
     case TypeDesc::HALF:
-#if 0
         // Adobe extension, see http://chriscox.org/TIFFTN3d1.pdf
         // Unfortunately, Nuke 9.0, and probably many other apps we care
         // about, cannot read 16 bit float TIFFs correctly. Revisit this
         // again in future releases. (comment added Feb 2015)
-        m_bitspersample = 16;
+        // For now, the default is to NOT write this (instead writing float)
+        // unless the "tiff:half" attribute is nonzero -- use the global
+        // OIIO attribute, but override with a specific attribute for this
+        // file.
+        if (m_spec.get_int_attribute("tiff:half", OIIO::get_int_attribute("tiff:half"))) {
+            m_bitspersample = 16;
+        } else {
+            // Silently change requests for unsupported 'half' to 'float'
+            m_bitspersample = 32;
+            m_spec.set_format (TypeDesc::FLOAT);
+        }
         sampformat = SAMPLEFORMAT_IEEEFP;
         break;
-#else
-        // Silently change requests for unsupported 'half' to 'float'
-        m_spec.set_format (TypeDesc::FLOAT);
-#endif
     case TypeDesc::FLOAT:
         m_bitspersample = 32;
         sampformat = SAMPLEFORMAT_IEEEFP;
@@ -375,7 +394,7 @@ TIFFOutput::open (const std::string &name, const ImageSpec &userspec,
     TIFFSetField (m_tif, TIFFTAG_BITSPERSAMPLE, m_bitspersample);
     TIFFSetField (m_tif, TIFFTAG_SAMPLEFORMAT, sampformat);
 
-    m_photometric = (m_spec.nchannels > 1 ? PHOTOMETRIC_RGB : PHOTOMETRIC_MINISBLACK);
+    m_photometric = (m_spec.nchannels >= 3 ? PHOTOMETRIC_RGB : PHOTOMETRIC_MINISBLACK);
 
     string_view comp = m_spec.get_string_attribute("Compression", "zip");
     if (Strutil::iequals (comp, "jpeg") &&
@@ -399,6 +418,11 @@ TIFFOutput::open (const std::string &name, const ImageSpec &userspec,
             // predictors not supported for unusual bit depths (e.g. 10)
             TIFFSetField (m_tif, TIFFTAG_PREDICTOR, PREDICTOR_HORIZONTAL);
         }
+        if (m_compression == COMPRESSION_ADOBE_DEFLATE) {
+            int q = m_spec.get_int_attribute ("tiff:zipquality", -1);
+            if (q >= 0)
+                TIFFSetField (m_tif, TIFFTAG_ZIPQUALITY, OIIO::clamp(q, 1, 9));
+        }
     } else if (m_compression == COMPRESSION_JPEG) {
         TIFFSetField (m_tif, TIFFTAG_JPEGQUALITY,
                       m_spec.get_int_attribute("CompressionQuality", 95));
@@ -416,29 +440,57 @@ TIFFOutput::open (const std::string &name, const ImageSpec &userspec,
         // There are a few ways in which we allow allow the user to specify
         // translation to different photometric types.
         string_view photo = m_spec.get_string_attribute("tiff:ColorSpace");
-        if (Strutil::iequals (photo, "CMYK")) {
-            // CMYK: force to 4 channel output, either uint8 or uint16
+        if (Strutil::iequals (photo, "CMYK") ||
+            Strutil::iequals (photo, "color separated")) {
+            // User has requested via the "tiff:ColorSpace" attribute that
+            // the file be written as color separated channels.
             m_photometric = PHOTOMETRIC_SEPARATED;
-            m_outputchans = 4;
-            TIFFSetField (m_tif, TIFFTAG_SAMPLESPERPIXEL, m_outputchans);
             if (m_spec.format != TypeDesc::UINT8 || m_spec.format != TypeDesc::UINT16) {
                 m_spec.format = TypeDesc::UINT8;
                 m_bitspersample = 8;
                 TIFFSetField (m_tif, TIFFTAG_BITSPERSAMPLE, m_bitspersample);
                 TIFFSetField (m_tif, TIFFTAG_SAMPLEFORMAT, SAMPLEFORMAT_UINT);
             }
+            if (source_is_rgb(m_spec)) {
+                // Case: RGB -> CMYK, do the conversions per pixel
+                m_convert_rgb_to_cmyk = true;
+                m_outputchans = 4;  // output 4, not 4 chans
+                TIFFSetField (m_tif, TIFFTAG_SAMPLESPERPIXEL, m_outputchans);
+                TIFFSetField (m_tif, TIFFTAG_INKSET, INKSET_CMYK);
+            } else if (source_is_cmyk(m_spec)) {
+                // Case: CMYK -> CMYK (do not transform)
+                m_convert_rgb_to_cmyk = false;
+                TIFFSetField (m_tif, TIFFTAG_INKSET, INKSET_CMYK);
+            } else {
+                // Case: arbitrary inks
+                m_convert_rgb_to_cmyk = false;
+                TIFFSetField (m_tif, TIFFTAG_INKSET, INKSET_MULTIINK);
+                std::string inknames;
+                for (int i = 0; i < m_spec.nchannels; ++i) {
+                    if (i)
+                        inknames.insert (inknames.size(), 1, '\0');
+                    if (i < (int)m_spec.channelnames.size())
+                        inknames.insert (inknames.size(), m_spec.channelnames[i]);
+                    else
+                        inknames.insert (inknames.size(), Strutil::format("ink%d", i));
+                }
+                TIFFSetField (m_tif, TIFFTAG_INKNAMES, int(inknames.size()+1), &inknames[0]);
+                TIFFSetField (m_tif, TIFFTAG_NUMBEROFINKS, m_spec.nchannels);
+            }
         }
     }
 
     TIFFSetField (m_tif, TIFFTAG_PHOTOMETRIC, m_photometric);
 
     // ExtraSamples tag
-    if (m_spec.nchannels > 3 && m_photometric != PHOTOMETRIC_SEPARATED) {
+    if ((m_spec.alpha_channel >= 0 || m_spec.nchannels > 3)
+         && m_photometric != PHOTOMETRIC_SEPARATED) {
         bool unass = m_spec.get_int_attribute("oiio:UnassociatedAlpha", 0);
-        short e = m_spec.nchannels-3;
+        int defaultchans = m_spec.nchannels >= 3 ? 3 : 1;
+        short e = m_spec.nchannels - defaultchans;
         std::vector<unsigned short> extra (e);
         for (int c = 0;  c < e;  ++c) {
-            if (m_spec.alpha_channel == (c+3))
+            if (m_spec.alpha_channel == (c+defaultchans))
                 extra[c] = unass ? EXTRASAMPLE_UNASSALPHA : EXTRASAMPLE_ASSOCALPHA;
             else
                 extra[c] = EXTRASAMPLE_UNSPECIFIED;
@@ -602,6 +654,14 @@ TIFFOutput::put_parameter (const std::string &name, TypeDesc type,
             return true;
         }
     }
+    if (Strutil::iequals(name, "Make") && type == TypeDesc::STRING) {
+        TIFFSetField (m_tif, TIFFTAG_MAKE, *(char**)data);
+        return true;
+    }
+    if (Strutil::iequals(name, "Model") && type == TypeDesc::STRING) {
+        TIFFSetField (m_tif, TIFFTAG_MODEL, *(char**)data);
+        return true;
+    }
     if (Strutil::iequals(name, "Software") && type == TypeDesc::STRING) {
         TIFFSetField (m_tif, TIFFTAG_SOFTWARE, *(char**)data);
         return true;
@@ -865,7 +925,7 @@ TIFFOutput::write_scanline (int y, int z, TypeDesc format,
                                m_dither, y, z);
 
     // Handle weird photometric/color spaces
-    if (m_photometric == PHOTOMETRIC_SEPARATED)
+    if (m_photometric == PHOTOMETRIC_SEPARATED && m_convert_rgb_to_cmyk)
         data = convert_to_cmyk (spec().width, data);
 
     // Handle weird bit depths
@@ -958,7 +1018,7 @@ TIFFOutput::write_tile (int x, int y, int z,
                            m_scratch, m_dither, x, y, z);
 
     // Handle weird photometric/color spaces
-    if (m_photometric == PHOTOMETRIC_SEPARATED)
+    if (m_photometric == PHOTOMETRIC_SEPARATED && m_convert_rgb_to_cmyk)
         data = convert_to_cmyk (spec().tile_pixels(), data);
 
     // Handle weird bit depths
@@ -989,7 +1049,6 @@ TIFFOutput::write_tile (int x, int y, int z,
         imagesize_t tile_pixels = m_spec.tile_pixels();
         imagesize_t plane_bytes = tile_pixels * m_spec.format.size();
         DASSERT (plane_bytes*m_spec.nchannels == m_spec.tile_bytes());
-        m_scratch.resize (m_spec.tile_bytes());
 
         boost::scoped_array<char> separate_heap;
         char *separate = NULL;
@@ -1043,5 +1102,51 @@ TIFFOutput::write_tile (int x, int y, int z,
     return true;
 }
 
+
+
+bool
+TIFFOutput::source_is_cmyk (const ImageSpec &spec)
+{
+    if (spec.nchannels != 4) {
+        return false;   // Can't be CMYK if it's not 4 channels
+    }
+    if (Strutil::iequals(spec.channelnames[0], "C") &&
+        Strutil::iequals(spec.channelnames[1], "M") &&
+        Strutil::iequals(spec.channelnames[2], "Y") &&
+        Strutil::iequals(spec.channelnames[3], "K"))
+        return true;
+    if (Strutil::iequals(spec.channelnames[0], "Cyan") &&
+        Strutil::iequals(spec.channelnames[1], "Magenta") &&
+        Strutil::iequals(spec.channelnames[2], "Yellow") &&
+        Strutil::iequals(spec.channelnames[3], "Black"))
+        return true;
+    string_view oiiocs = spec.get_string_attribute("oiio:ColorSpace");
+    if (Strutil::iequals (oiiocs, "CMYK"))
+        return true;
+    return false;
+}
+
+
+
+bool
+TIFFOutput::source_is_rgb (const ImageSpec &spec)
+{
+    string_view oiiocs = spec.get_string_attribute("oiio:ColorSpace");
+    if (Strutil::iequals (oiiocs, "CMYK") ||
+        Strutil::iequals (oiiocs, "color separated"))
+        return false;   // It's a color space mode that means something else
+    if (spec.nchannels != 3)
+        return false;   // Can't be RGB if it's not 3 channels
+    if (Strutil::iequals(spec.channelnames[0], "R") &&
+        Strutil::iequals(spec.channelnames[1], "G") &&
+        Strutil::iequals(spec.channelnames[2], "B"))
+        return true;
+    if (Strutil::iequals(spec.channelnames[0], "Red") &&
+        Strutil::iequals(spec.channelnames[1], "Green") &&
+        Strutil::iequals(spec.channelnames[2], "Blue"))
+        return true;
+    return false;
+}
+
 OIIO_PLUGIN_NAMESPACE_END
 
