@@ -4,8 +4,8 @@
 oiio_images = parent + "/oiio-images/"
 
 # Just for simplicity, make a checkerboard with a solid alpha
-command += (oiio_app("oiiotool") + " --pattern checker 128x128 4 --ch R,G,B,=1.0"
-            + " -d uint8 -o " + oiio_relpath("checker.tif") + " >> out.txt;\n")
+command += oiiotool (" --pattern checker 128x128 4 --ch R,G,B,=1.0"
+            + " -d uint8 -o " + oiio_relpath("checker.tif") )
 
 # Basic test - recreate the grid texture
 command += maketx_command (oiio_images + "grid.tif", "grid.tx", showinfo=True)
@@ -52,14 +52,14 @@ command += maketx_command ("checker.tif", "checker-opaque.tx",
                            "--opaque-detect", showinfo=True)
 
 # Test --monochrome-detect (first create a monochrome image)
-command += (oiio_app("oiiotool") + " --pattern constant:color=.25,.25,.25 256x256 3 "
-            + " -d uint8 -o " + oiio_relpath("gray.tif") + " >> out.txt;\n")
+command += oiiotool (" --pattern constant:color=.25,.25,.25 256x256 3 "
+                    + " -d uint8 -o " + oiio_relpath("gray.tif"))
 command += maketx_command ("gray.tif", "gray-mono.tx",
                            "--monochrome-detect", showinfo=True)
 
 # Test --monochrome-detect on something that is NOT monochrome
-command += (oiio_app("oiiotool") + " --pattern constant:color=.25,.2,.15 256x256 3 "
-            + " -d uint8 -o " + oiio_relpath("pink.tif") + " >> out.txt;\n")
+command += oiiotool (" --pattern constant:color=.25,.2,.15 256x256 3 "
+                    + " -d uint8 -o " + oiio_relpath("pink.tif"))
 command += maketx_command ("pink.tif", "pink-mono.tx",
                            "--monochrome-detect", showinfo=True)
 
@@ -73,7 +73,7 @@ command += maketx_command ("checker.tif", "checker-prman.tx",
 # testsuite/oiiotool-fixnan.  (Use --nomipmap to cut down on stats output)
 # FIXME: would also like to test --checknan, but the problem with that is
 # that is actually FAILS if there's a nan.
-command += maketx_command ("../oiiotool-fixnan/bad.exr",
+command += maketx_command ("../oiiotool-fixnan/src/bad.exr",
                            "nan.exr", "--fixnan box3 --nomipmap",
                            showinfo=True, showinfo_extra="--stats")
 
@@ -83,23 +83,35 @@ command += maketx_command ("checker.tif", "checker-exr.pdq",
 
 # Test that we cleanly replace any existing SHA-1 hash and ConstantColor
 # hint in the ImageDescription of the input file.
-command += (oiio_app("oiiotool") + " --pattern constant:color=1,0,0 64x64 3 "
+command += oiiotool (" --pattern constant:color=1,0,0 64x64 3 "
             + " --caption \"foo SHA-1=1234abcd ConstantColor=[0.0,0,-0.0] bar\""
-            + " -d uint8 -o " + oiio_relpath("small.tif") + " >> out.txt;\n")
+            + " -d uint8 -o " + oiio_relpath("small.tif") )
 command += info_command ("small.tif", safematch=1);
 command += maketx_command ("small.tif", "small.tx",
                            "--oiio --constant-color-detect", showinfo=True)
 
+# Test that the oiio:SHA-1 hash is stable, and that that changing filter and
+# using -hicomp result in different images and different hashes.
+command += maketx_command (oiio_images + "grid.tif", "grid-lanczos3.tx",
+                           extraargs = "-filter lanczos3")
+command += maketx_command (oiio_images + "grid.tif", "grid-lanczos3-hicomp.tx",
+                           extraargs = "-filter lanczos3 -hicomp")
+command += info_command ("grid.tx",
+                         extraargs="--metamatch oiio:SHA-1")
+command += info_command ("grid-lanczos3.tx",
+                         extraargs="--metamatch oiio:SHA-1")
+command += info_command ("grid-lanczos3-hicomp.tx",
+                         extraargs="--metamatch oiio:SHA-1")
+
 # Regression test -- at one point, we had a bug where we were botching
 # the poles of OpenEXR env maps, adding energy.  Check it by creating an
 # all-white image, turning it into an env map, and calculating its
 # statistics (should be 1.0 everywhere).
-command += (oiio_app("oiiotool") + " --pattern constant:color=1,1,1 4x2 3 "
-            + " -d half -o " + oiio_relpath("white.exr") + " >> out.txt;\n")
+command += oiiotool (" --pattern constant:color=1,1,1 4x2 3 "
+            + " -d half -o " + oiio_relpath("white.exr"))
 command += maketx_command ("white.exr", "whiteenv.exr",
                            "--envlatl")
-command += (oiio_app("oiiotool") + "--stats whiteenv.exr"
-            + " >> out.txt;\n")
+command += oiiotool ("--stats whiteenv.exr")
 
 outputs = [ "out.txt" ]
 
