@@ -3,7 +3,7 @@
 imagedir = parent + "oiio-images"
 
 # Run the script 
-command += "python test_imagebuf.py > out.txt ;"
+command += "python src/test_imagebuf.py > out.txt ;"
 
 # compare the outputs
 outputs = [ "out.tif", "outtuple.tif",
