@@ -15,9 +15,8 @@ refdirlist = [
 ]
 
 
-
-# Run the script 
-command += "python test_imagebufalgo.py > out.txt ;"
+# Run the script
+command += "python src/test_imagebufalgo.py > out.txt ;"
 
 # Checkout outputs -- some of the refs are in the oiiotool test dir
 outputs = ["black.tif", "filled.tif", "checker.tif",
@@ -29,7 +28,7 @@ outputs = ["black.tif", "filled.tif", "checker.tif",
            "rotate90.tif", "rotate180.tif", "rotate270.tif",
            "rotated.tif", "rotated-offcenter.tif",
            "warped.tif",
-           "flip.tif", "flop.tif", "flipflop.tif", "reorient1.tif",
+           "flip.tif", "flop.tif", "reorient1.tif",
            "transpose.tif",
            "cshift.tif", "cadd1.exr", "cadd2.exr", "add.exr", "sub.exr",
            "abs.exr", "absdiff.exr",
@@ -42,13 +41,15 @@ outputs = ["black.tif", "filled.tif", "checker.tif",
            "rangecompress.tif", "rangeexpand.tif",
            "resize.tif", "resample.tif",
            "bsplinekernel.exr", "bspline-blur.tif", "tahoe-median.tif",
-           "unsharp.tif", "unsharp-median.tif",
+           "dilate.tif", "erode.tif",
+           "unsharp.tif", "unsharp-median.tif", "tahoe-laplacian.tif",
            "fft.exr", "ifft.exr", "polar.exr", "unpolar.exr",
            "tahoe-filled.tif",
            "box3.exr",
            "a_over_b.exr",
            "tahoe-small.tx",
-           "text.tif"
+           "text.tif",
+           "out.txt"
          ]
     # command += checkref (f)
 
